#import "SBJson.h"
#import "GHBlob.h"
#import "GHErrors.h"
#import "NSString+Base64.h"

@implementation GHBlob

static NSString *type = @"blob";

# pragma mark - Attributes

@synthesize content = _content;
@synthesize encoding = _encoding;
@synthesize delegate = _delegate;

- (NSString *)type {
  return type;
}

- (NSString *)contentAsNSString {
  if (self.content) {
    NSStringEncoding encoding = self.encoding;
    if (encoding == NSProprietaryStringEncoding) {
      encoding = [NSString defaultCStringEncoding];
    }
    return [NSString stringWithCString:[self.content bytes] encoding:encoding];
  }
  return nil;
}

- (BOOL)isText {
  return self.encoding > 0 && self.encoding != NSProprietaryStringEncoding;
}

# pragma mark - Class constructors

+ (id)fetchBlobContentWithRepository:(GHRepository *)repository sha:(NSString *)sha delegate:(id<GHDelegate>)delegate {
  id blob = [[self alloc] initWithRepository:repository sha:sha];
  ((GHBlob *)blob).delegate = delegate;
  [blob fetchContent];
  return blob;
}

# pragma mark - Initializers

- (id)initWithRepository:(GHRepository *)repository sha:(NSString *)sha {
  return [self initWithRepository:repository content:nil encoding:NSASCIIStringEncoding mode:nil path:nil sha:sha size:0];
}

- (id)initWithRepository:(GHRepository *)repository mode:(NSString *)mode path:(NSString *)path sha:(NSString *)sha size:(NSUInteger)size {
  return [self initWithRepository:repository content:nil encoding:NSASCIIStringEncoding mode:mode path:path sha:sha size:size];
}

- (id)initWithRepository:(GHRepository *)repository content:(NSData *)content encoding:(NSStringEncoding)encoding mode:(NSString *)mode path:(NSString *)path sha:(NSString *)sha size:(NSUInteger)size {
  self = [super init];
  if (self) {
    _repository = repository; // TODO: copy
    _content = [content copy];
    _encoding = encoding;
    _mode = [mode copy];
    _path = [path copy];
    _sha = [sha copy];
    _size = size;
  }
  return self;
}

# pragma mark - Github interaction

- (NSString *)apiPath {
  return [NSString stringWithFormat:@"/repos/%@/%@/git/blobs/%@",
          self.repository.owner.login, self.repository.name, self.sha];
}

- (void)fetchContent {
  [self fetchData];
}

# pragma mark - NSCopying delegate methods

- (id)copyWithZone:(NSZone *)zone {
  return [[GHBlob allocWithZone:zone] initWithRepository:self.repository
                                           content:self.content
                                          encoding:self.encoding
                                              mode:self.mode
                                              path:self.path
                                               sha:self.sha
                                              size:self.size];
}

# pragma mark - SBJsonStreamParserAdapter delegate methods

- (void)parser:(SBJsonStreamParser*)parser foundObject:(NSDictionary*)dict {
  if (dict) {
    NSString *content = [dict objectForKey:@"content"];
    NSString *encoding = [dict objectForKey:@"encoding"];
    if (content && encoding) {
      if ((NSNull *)content == [NSNull null]) {
        _content = nil;
      }
      if ((NSNull *)encoding == [NSNull null]) {
        _encoding = 0U;
      } else if ([encoding isEqualToString:@"base64"] && (NSNull *)content != [NSNull null]) {
        // TODO: need to determine if this is textual content, and what the encoding should be, somehow :(
        // DEBUG: until I can determine encoding somehow, assume UTF-8
        // _encoding = NSProprietaryStringEncoding;
        _encoding = NSUTF8StringEncoding;
        _content = [content decodeBase64];
      } else if ([encoding isEqualToString:@"utf-8"] && (NSNull *)content != [NSNull null]) {
        _encoding = NSUTF8StringEncoding;
        _content = [NSData dataWithBytes:[content cStringUsingEncoding:NSUTF8StringEncoding]
                                  length:[content lengthOfBytesUsingEncoding:NSUTF8StringEncoding]];
      }
    }
  }
}

@end
