#import "GHGitObject.h"
#import "GHTree.h"

@interface GHCommit : GHGitObject {
  NSDictionary *_author;
  NSDictionary *_committer;
  NSString *_message;
  NSArray *_parents;
  GHTree *_tree;
}

# pragma mark - Attributes

/**
 * Information about the author of this commit. Keys are:
 *
 * * date
 * * email
 * * name
 */
@property (nonatomic, readonly, copy) NSDictionary *author;
/**
 * Information about the committer for this commit. Keys are:
 *
 * * date
 * * email
 * * name
 */
@property (nonatomic, readonly, copy) NSDictionary *committer;
@property (nonatomic, readonly, copy) NSString *message;
/**
 * The GHCommit objects from which this commit descended.
 */
@property (nonatomic, readonly, copy) NSArray *parents;
@property (nonatomic, readonly, copy) GHTree *tree;

# pragma mark - Initializers

/**
 * Creates a commit with the bare minimum properties to fetch its properties from Github.
 *
 * TODO: The request to fetch the properties will be authenticated with the given credentials. If the repository is open
 * source, the credentials are not necessary.
 *
 * \param repository the Github repository that owns this commit
 * \param sha the sha hash for this commit
 * \return a new GHCommit without other properties
 */
- (id)initWithRepository:(GHRepository *)repository sha:(NSString *)sha;

/**
 * Creates a commit with the given attributes.
 *
 * \param attributes the desired attributes, where each key is a property name or the Github API property name
 * \return a new GHCommit with the given attributes
 */
- (id)initWithAttributes:(NSDictionary *)attributes;

@end
