#import "SBJson.h"
#import "GHCommit.h"
#import "GHRepository.h"

@interface GHCommit ()
- (void)updateWithAttributes:(NSDictionary *)attributes;
@end

@implementation GHCommit

static NSString *type = @"commit";

# pragma mark - Attributes

@synthesize author = _author;
@synthesize committer = _committer;
@synthesize message = _message;
@synthesize parents = _parents;
@synthesize tree = _tree;

- (NSString *)type {
  return type;
}

# pragma mark - Initializers

- (id)initWithRepository:(GHRepository *)repository sha:(NSString *)sha {
  return [self initWithAttributes:[NSDictionary dictionaryWithObjectsAndKeys:repository, @"repository",
                                   sha, @"sha", nil]];
}

- (id)initWithAttributes:(NSDictionary *)attributes {
  self = [super init];
  if (self) {
    [self updateWithAttributes:attributes];
  }
  return self;
}

# pragma mark - Github interaction

- (NSString *)apiPath {
  return [NSString stringWithFormat:@"/repos/%@/%@/git/commits/%@",
          self.repository.owner.login, self.repository.name, self.sha];
}

# pragma mark - SBJsonStreamParser delegate methods

- (void)parser:(SBJsonStreamParser *)parser foundObject:(NSDictionary *)dict {
  [self updateWithAttributes:dict];
}

# pragma mark - private

- (void)updateWithAttributes:(NSDictionary *)attributes {
  if (attributes) {
    GHRepository *repository = [attributes valueForKey:@"repository"];
    if ((NSNull *)repository == [NSNull null]) {
      _repository = nil;
    } else if (repository) {
      _repository = repository;
    }
    NSString *sha = [attributes valueForKey:@"sha"];
    if ((NSNull *)sha == [NSNull null]) {
      _sha = nil;
    } else if (sha) {
      _sha = [sha copy];
    }
    NSDictionary *author = [attributes valueForKey:@"author"];
    if ((NSNull *)author == [NSNull null]) {
      _author = nil;
    } else if (author) {
      _author = [author copy];
    }
    NSDictionary *committer = [attributes valueForKey:@"committer"];
    if ((NSNull *)committer == [NSNull null]) {
      _committer = nil;
    } else if (committer) {
      _committer = [committer copy];
    }
    NSString *message = [attributes valueForKey:@"message"];
    if ((NSNull *)message == [NSNull null]) {
      _message = nil;
    } else if (message) {
      _message = [message copy];
    }
    NSArray *parents = [attributes valueForKey:@"parents"];
    if ((NSNull *)parents == [NSNull null]) {
      _parents = nil;
    } else if (parents) {
      NSMutableArray *newParents = [NSMutableArray array];
      for (id parent in parents) {
        if ([parent isKindOfClass:[GHCommit class]]) {
          [newParents addObject:parent];
        } else if ([parent isKindOfClass:[NSDictionary class]]) {
          NSMutableDictionary *newParentAttributes = [parent mutableCopy];
          [newParentAttributes setValue:self.repository forKey:@"repository"];
          [newParents addObject:[[GHCommit alloc] initWithAttributes:newParentAttributes]];
        }
      }
      _parents = newParents;
    }
    id tree = [attributes valueForKey:@"tree"];
    if ((NSNull *)tree == [NSNull null]) {
      _tree = nil;
    } else if (tree) {
      if ([tree isKindOfClass:[GHTree class]]) {
        _tree = tree;
      } else if ([tree isKindOfClass:[NSDictionary class]]) {
        _tree = [[GHTree alloc] initWithRepository:self.repository sha:[tree objectForKey:@"sha"]];
      }
    }
  }
}

@end
