#import "GHAbstractResource.h"

/**
 * Represents an object from the Git data model as represented by the Github API.
 *
 * Retrieval of an object's data from the Github API is done using the #repository and #sha to generate the URL. TODO:
 * If the repository is not open source, credentials must also be provided to authenticate the request.
 */
@interface GHGitObject : GHAbstractResource {
  NSString *_sha;
}

/**
 * SHA hash identifying this object in its repository.
 */
@property (nonatomic, readonly, copy) NSString *sha;

/**
 * Indicates what type of git object this is: a ref, commit, tree, or blob.
 *
 * Subclasses must implement the -type method.
 */
@property (nonatomic, readonly, copy) NSString *type;

@end
