#import "SBJson.h"
#import "GHGitObject.h"
#import "GHErrors.h"

@implementation GHGitObject

@synthesize sha = _sha;
@dynamic type;

# pragma mark - Github interaction

- (void)fetchDataWithParameters:(NSDictionary *)parameters {
  // Ensure all required attributes are present
  if (!(self.repository && self.sha)) {
    if ([self.delegate respondsToSelector:@selector(githubObject:cannotFetchWithError:)]) {
      NSUInteger errorCode = 0;
      NSString *description = nil;
      NSString *recoverySuggestion = nil;
      NSArray *recoveryOptions = nil;
      if (!self.repository) {
        errorCode = GHNoRepositoryError;
        description = NSLocalizedStringFromTable(@"github.no_repository_error.description", GITHUB_STRINGS_TABLE, @"GHNoRepositoryError NSLocalizedDescriptionKey");
        recoverySuggestion = NSLocalizedStringFromTable(@"github.no_repository_error.recovery_suggestion", GITHUB_STRINGS_TABLE, @"GHNoRepositoryError NSLocalizedRecoverySuggestionErrorKey");
        recoveryOptions = [NSArray arrayWithObject:NSLocalizedStringFromTable(@"github.common.recovery_options.acknowledgement", GITHUB_STRINGS_TABLE, @"NSLocalizedRecoveryOptionsErrorKey - Acknowledgment (OK); have instructed the user what to do to fix the issue")];
      } else if (!self.sha) {
        errorCode = GHNoShaError;
        description = NSLocalizedStringFromTable(@"github.no_sha_error.description", GITHUB_STRINGS_TABLE, @"GHNoRepositoryError NSLocalizedDescriptionKey");
        recoverySuggestion = NSLocalizedStringFromTable(@"github.no_sha_error.recovery_suggestion", GITHUB_STRINGS_TABLE, @"GHNoRepositoryError NSLocalizedRecoverySuggestionErrorKey");
        recoveryOptions = [NSArray arrayWithObject:NSLocalizedStringFromTable(@"github.common.recovery_options.acknowledgement", GITHUB_STRINGS_TABLE, @"NSLocalizedRecoveryOptionsErrorKey - Acknowledgment (OK); have instructed the user what to do to fix the issue")];
      }
      NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:description, NSLocalizedDescriptionKey,
                                recoverySuggestion, NSLocalizedRecoverySuggestionErrorKey,
                                recoveryOptions, NSLocalizedRecoveryOptionsErrorKey, nil];
      NSError *error = [NSError errorWithDomain:GITHUB_ERROR_DOMAIN code:errorCode userInfo:userInfo];
      [self.delegate githubObject:self cannotFetchWithError:error];
    }
  } else {
    [super fetchDataWithParameters:parameters];
  }
}

@end
