#import "GHAbstractResource.h"

/**
 * An organization in the Github data model, with member users and repositories.
 */
@interface GHOrganization : GHAbstractResource

# pragma mark - Attributes

@property (nonatomic, readonly, copy) NSString *avatarURL;
@property (nonatomic, readonly, strong) NSString *billingEmail;
@property (nonatomic, readonly, copy) NSString *blog;
@property (nonatomic, readonly, assign) NSUInteger collaboratorCount;
@property (nonatomic, readonly, copy) NSString *company;
@property (nonatomic, readonly, strong) NSDate *createdAt;
@property (nonatomic, readonly, assign) NSUInteger diskUsage;
@property (nonatomic, readonly, copy) NSString *email;
@property (nonatomic, readonly, assign) NSUInteger followerCount;
@property (nonatomic, readonly, assign) NSUInteger followingCount;
/**
 * The URL to the organization's HTML profile page.
 */
@property (nonatomic, readonly, copy) NSString *githubURL;
@property (nonatomic, readonly, copy) NSString *location;
@property (nonatomic, readonly, copy) NSString *login;
@property (nonatomic, readonly, copy) NSString *name;
@property (nonatomic, readonly, assign) NSUInteger ownedRepositoryCount;
/**
 * Information about the organization's Github plan.
 *
 * Keys:
 * * name
 * * space
 * * private_repos (TODO: privateRepositoryCount)
 */
@property (nonatomic, readonly, copy) NSDictionary *plan;
@property (nonatomic, readonly, assign) NSUInteger privateGistCount;
@property (nonatomic, readonly, assign) NSUInteger privateRepositoryCount;
@property (nonatomic, readonly, assign) NSUInteger publicGistCount;
@property (nonatomic, readonly, assign) NSUInteger publicRepositoryCount;

# pragma mark - Github API attribute names

@property (nonatomic, readonly, copy) NSString *avatar_url;
@property (nonatomic, readonly, strong) NSString *billing_email;
@property (nonatomic, readonly, assign) NSUInteger collaborators;
@property (nonatomic, readonly, strong) NSDate *created_at;
@property (nonatomic, readonly, assign) NSUInteger disk_usage;
@property (nonatomic, readonly, assign) NSUInteger followers;
@property (nonatomic, readonly, assign) NSUInteger following;
@property (nonatomic, readonly, copy) NSString *html_url;
@property (nonatomic, readonly, assign) NSUInteger owned_private_repos;
@property (nonatomic, readonly, assign) NSUInteger private_gists;
@property (nonatomic, readonly, assign) NSUInteger public_gists;
@property (nonatomic, readonly, assign) NSUInteger public_repos;
@property (nonatomic, readonly, assign) NSUInteger total_private_repos;

# pragma mark - Initializers

/**
 * Creates an organization with a login to provide the location of a resource.
 *
 * \param login the organization login
 */
- (id)initWithLogin:(NSString *)login;

/**
 * Creates an organization with the given attributes.
 *
 * \param attributes the desired attributes, where each key is a property name or the Github API property name
 */
- (id)initWithAttributes:(NSDictionary *)attributes;

# pragma mark - Github interaction

/**
 * Starts a fetch of a list of this organization's repositories from the Github API. The delegate will be notified when
 * the fetch is complete, if the delegate is a GHOrganizationDelegate.
 */
- (void)fetchRepositories;

@end
