#import "SBJson.h"
#import "GHOrganization.h"
#import "GHOrganizationDelegate.h"
#import "GHRepository.h"

@interface GHOrganization ()
- (void)updateFromGithubAttributes:(NSDictionary *)attributes;
@end

@implementation GHOrganization

# pragma mark - Attributes

@synthesize avatarURL = _avatarURL;
@synthesize billingEmail = _billingEmail;
@synthesize blog = _blog;
@synthesize collaboratorCount = _collaboratorCount;
@synthesize company = _company;
@synthesize createdAt = _createdAt;
@synthesize diskUsage = _diskUsage;
@synthesize email = _email;
@synthesize followerCount = _followerCount;
@synthesize followingCount = _followingCount;
@synthesize githubURL = _githubURL;
@synthesize location = _location;
@synthesize login = _login;
@synthesize name = _name;
@synthesize ownedRepositoryCount = _ownedRepositoryCount;
@synthesize plan = _plan;
@synthesize privateGistCount = _privateGistCount;
@synthesize privateRepositoryCount = _privateRepositoryCount;
@synthesize publicGistCount = _publicGistCount;
@synthesize publicRepositoryCount = _publicRepositoryCount;

# pragma mark - Github API attribute names

@dynamic avatar_url;
@dynamic billing_email;
@dynamic collaborators;
@dynamic created_at;
@dynamic disk_usage;
@dynamic followers;
@dynamic following;
@dynamic html_url;
@dynamic owned_private_repos;
@dynamic private_gists;
@dynamic public_gists;
@dynamic public_repos;
@dynamic total_private_repos;

- (NSString *)avatar_url {
  return self.avatarURL;
}

- (NSString *)billing_email {
  return self.billingEmail;
}

- (NSUInteger)collaborators {
  return self.collaboratorCount;
}

- (NSDate *)created_at {
  return self.createdAt;
}

- (NSUInteger)disk_usage {
  return self.diskUsage;
}

- (NSUInteger)followers {
  return self.followerCount;
}

- (NSUInteger)following {
  return self.followingCount;
}

- (NSString *)html_url {
  return self.githubURL;
}

- (NSUInteger)owned_private_repos {
  return self.ownedRepositoryCount;
}

- (NSUInteger)private_gists {
  return self.privateGistCount;
}

- (NSUInteger)public_gists {
  return self.publicGistCount;
}

- (NSUInteger)public_repos {
  return self.publicRepositoryCount;
}

- (NSUInteger)total_private_repos {
  return self.privateRepositoryCount;
}

# pragma mark - Initializers

- (id)init {
  return [self initWithAttributes:nil];
}

- (id)initWithLogin:(NSString *)login {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  if (login) {
    [attributes setValue:login forKey:@"login"];
  }
  return [self initWithAttributes:attributes];
}

- (id)initWithAttributes:(NSDictionary *)attributes {
  self = [super init];
  if (self) {
    [self updateFromGithubAttributes:attributes];
    // provide special cases for those attributes named differently from the ones returned by Github
    NSString *avatarURL = [attributes valueForKey:@"avatarURL"];
    if (avatarURL) {
      _avatarURL = [avatarURL copy];
    }
    NSString *billingEmail = [attributes valueForKey:@"billingEmail"];
    if (billingEmail) {
      _billingEmail = [billingEmail copy];
    }
    NSNumber *collaboratorCount = [attributes valueForKey:@"collaboratorCount"];
    if (collaboratorCount) {
      _collaboratorCount = [collaboratorCount intValue];
    }
    NSDate *createdAt = [attributes valueForKey:@"createdAt"];
    if (createdAt) {
      _createdAt = createdAt;
    }
    NSNumber *diskUsage = [attributes valueForKey:@"diskUsage"];
    if (diskUsage) {
      _diskUsage = [diskUsage intValue];
    }
    NSNumber *followerCount = [attributes valueForKey:@"followerCount"];
    if (followerCount) {
      _followerCount = [followerCount intValue];
    }
    NSNumber *followingCount = [attributes valueForKey:@"followingCount"];
    if (followingCount) {
      _followingCount = [followingCount intValue];
    }
    NSString *githubURL = [attributes valueForKey:@"githubURL"];
    if (githubURL) {
      _githubURL = [githubURL copy];
    }
    NSNumber *ownedRepositoryCount = [attributes valueForKey:@"ownedRepositoryCount"];
    if (ownedRepositoryCount) {
      _ownedRepositoryCount = [ownedRepositoryCount intValue];
    }
    NSNumber *privateGistCount = [attributes valueForKey:@"privateGistCount"];
    if (privateGistCount) {
      _privateGistCount = [privateGistCount intValue];
    }
    NSNumber *privateRepositoryCount = [attributes valueForKey:@"privateRepositoryCount"];
    if (privateRepositoryCount) {
      _privateRepositoryCount = [privateRepositoryCount intValue];
    }
    NSNumber *publicGistCount = [attributes valueForKey:@"publicGistCount"];
    if (publicGistCount) {
      _publicGistCount = [publicGistCount intValue];
    }
    NSNumber *publicRepositoryCount = [attributes valueForKey:@"publicRepositoryCount"];
    if (publicRepositoryCount) {
      _publicRepositoryCount = [publicRepositoryCount intValue];
    }
  }
  return self;
}

# pragma mark - Github interaction

- (NSString *)apiPath {
  return [NSString stringWithFormat:@"/orgs/%@", self.login];
}

- (void)fetchRepositories {
  [self fetchDataAtApiPath:[[self apiPath] stringByAppendingString:@"/repos"] withParameters:nil];
}

# pragma mark - SBJsonStreamParserAdapter delegate methods

- (void)parser:(SBJsonStreamParser *)parser foundArray:(NSArray *)array {
  // found repositories
  if (array && [self.delegate respondsToSelector:@selector(organization:didFindRepository:)]) {
    for (NSDictionary *repositoryAttributes in array) {
      NSLog(@"Organization found repository with attributes: %@", [repositoryAttributes description]);
      GHRepository *repository = [[GHRepository alloc] initWithAttributes:repositoryAttributes];
      [(id<GHOrganizationDelegate>)self.delegate organization:self didFindRepository:repository];
    }
  }
}

- (void)parser:(SBJsonStreamParser *)parser foundObject:(NSDictionary *)dict {
  // found organization details
  if (dict) {
    [self updateFromGithubAttributes:dict];
  }
}

# pragma mark - private

- (void)updateFromGithubAttributes:(NSDictionary *)attributes {
  NSString *avatarURL = [attributes valueForKey:@"avatar_url"];
  if ((NSNull *)avatarURL == [NSNull null]) {
    _avatarURL = nil;
  } else if (avatarURL) {
    _avatarURL = [avatarURL copy];
  }
  NSString *billingEmail = [attributes valueForKey:@"billing_email"];
  if ((NSNull *)billingEmail == [NSNull null]) {
    _billingEmail = nil;
  } else if (billingEmail) {
    _billingEmail = [billingEmail copy];
  }
  NSString *blog = [attributes valueForKey:@"blog"];
  if ((NSNull *)blog == [NSNull null]) {
    _blog = nil;
  } else if (blog) {
    _blog = [blog copy];
  }
  NSNumber *collaboratorCount = [attributes valueForKey:@"collaborators"];
  if ((NSNull *)collaboratorCount == [NSNull null]) {
    _collaboratorCount = 0U;
  } else if (collaboratorCount) {
    _collaboratorCount = [collaboratorCount intValue];
  }
  NSString *company = [attributes valueForKey:@"company"];
  if ((NSNull *)company == [NSNull null]) {
    _company = nil;
  } else if (company) {
    _company = [company copy];
  }
  NSString *createdAtTimestamp = [attributes valueForKey:@"created_at"];
  if ((NSNull *)createdAtTimestamp == [NSNull null]) {
    _createdAt = nil;
  } else if (createdAtTimestamp) {
    NSDateFormatter *timestampFormatter = [[NSDateFormatter alloc] init];
    [timestampFormatter setDateFormat:[self.class timestampFormat]];
    _createdAt = [timestampFormatter dateFromString:createdAtTimestamp];
  }
  NSNumber *diskUsage = [attributes valueForKey:@"disk_usage"];
  if ((NSNull *)diskUsage == [NSNull null]) {
    _diskUsage = 0U;
  } else if (diskUsage) {
    _diskUsage = [diskUsage intValue];
  }
  NSString *email = [attributes valueForKey:@"email"];
  if ((NSNull *)email == [NSNull null]) {
    _email = nil;
  } else if (email) {
    _email = [email copy];
  }
  NSNumber *followerCount = [attributes valueForKey:@"followers"];
  if ((NSNull *)followerCount == [NSNull null]) {
    _followerCount = 0U;
  } else if (followerCount) {
    _followerCount = [followerCount intValue];
  }
  NSNumber *followingCount = [attributes valueForKey:@"following"];
  if ((NSNull *)followingCount == [NSNull null]) {
    _followingCount = 0U;
  } else if (followingCount) {
    _followingCount = [followingCount intValue];
  }
  NSString *githubURL = [attributes valueForKey:@"html_url"];
  if ((NSNull *)githubURL == [NSNull null]) {
    _githubURL = nil;
  } else if (githubURL) {
    _githubURL = [githubURL copy];
  }
  NSString *location = [attributes valueForKey:@"location"];
  if ((NSNull *)location == [NSNull null]) {
    _location = nil;
  } else if (location) {
    _location = [location copy];
  }
  NSString *login = [attributes valueForKey:@"login"];
  if ((NSNull *)login == [NSNull null]) {
    _login = nil;
  } else if (login) {
    _login = [login copy];
  }
  NSString *name = [attributes valueForKey:@"name"];
  if ((NSNull *)name == [NSNull null]) {
    _name = nil;
  } else if (name) {
    _name = [name copy];
  }
  NSNumber *ownedRepositoryCount = [attributes valueForKey:@"owned_private_repos"];
  if ((NSNull *)ownedRepositoryCount == [NSNull null]) {
    _ownedRepositoryCount = 0U;
  } else if (ownedRepositoryCount) {
    _ownedRepositoryCount = [ownedRepositoryCount intValue];
  }
  NSDictionary *plan = [attributes valueForKey:@"plan"];
  if ((NSNull *)plan == [NSNull null]) {
    _plan = nil;
  } else if (plan) {
    _plan = [plan copy];
  }
  NSNumber *privateGistCount = [attributes valueForKey:@"private_gists"];
  if ((NSNull *)privateGistCount == [NSNull null]) {
    _privateGistCount = 0U;
  } else if (privateGistCount) {
    _privateGistCount = [privateGistCount intValue];
  }
  NSNumber *privateRepositoryCount = [attributes valueForKey:@"total_private_repos"];
  if ((NSNull *)privateRepositoryCount == [NSNull null]) {
    _privateRepositoryCount = 0U;
  } else if (privateRepositoryCount) {
    _privateRepositoryCount = [privateRepositoryCount intValue];
  }
  NSNumber *publicGistCount = [attributes valueForKey:@"public_gists"];
  if ((NSNull *)publicGistCount == [NSNull null]) {
    _publicGistCount = 0U;
  } else if (publicGistCount) {
    _publicGistCount = [publicGistCount intValue];
  }
  NSNumber *publicRepositoryCount = [attributes valueForKey:@"public_repos"];
  if ((NSNull *)publicRepositoryCount == [NSNull null]) {
    _publicRepositoryCount = 0U;
  } else if (publicRepositoryCount) {
    _publicRepositoryCount = [publicRepositoryCount intValue];
  }
}

@end
