#import "SBJson.h"
#import "GHRef.h"
#import "GHRepository.h"

@implementation GHRef

# pragma mark - Attributes

@synthesize object = _object;
@synthesize ref = _ref;

# pragma mark - Initializers

- (id)initWithRepository:(GHRepository *)repository ref:(NSString *)ref {
  return [self initWithRepository:repository ref:ref object:nil];
}

- (id)initWithRepository:(GHRepository *)repository ref:(NSString *)ref object:(GHCommit *)object {
  self = [super init];
  if (self) {
    _repository = repository;
    _ref = [ref copy];
    _object = object;
  }
  return self;
}

# pragma mark - Github interaction

- (NSString *)apiPath {
  return [NSString stringWithFormat:@"/repos/%@/%@/git/%@", self.repository.owner.login, self.repository.name, self.ref];
}

# pragma mark - SBJsonStreamParser delegate methods

- (void)parser:(SBJsonStreamParser *)parser foundObject:(NSDictionary *)dict {
  if (dict) {
    NSString *ref = [dict objectForKey:@"ref"];
    if ((NSNull *)ref == [NSNull null]) {
      _ref = nil;
    } else if (ref) {
      _ref = [ref copy];
    }
    NSDictionary *object = [dict objectForKey:@"object"];
    if ((NSNull *)object == [NSNull null]) {
      _object = nil;
    } else if (object) {
      NSMutableDictionary *commitAttributes = [object mutableCopy];
      [commitAttributes setValue:self.repository forKey:@"repository"];
      _object = [[GHCommit alloc] initWithAttributes:commitAttributes];
    }
  }
}

@end
