#import "GHAbstractResource.h"
#import "GHOrganization.h"
#import "GHUser.h"

/**
 * A Github repository. Useful in two contexts:
 *
 * * the owner and name information are required for many resources within a repository
 * * obtaining repository information
 *
 * This repository is read-only. To update a repository, call #mutableCopy to modify the repository.
 */
@interface GHRepository : GHAbstractResource

# pragma mark - Attributes

@property (nonatomic, readonly, copy) NSString *cloneURL;
@property (nonatomic, readonly, copy) NSDate *createdAt;
@property (nonatomic, readonly, copy) NSString *description;
@property (nonatomic, readonly, assign) BOOL fork;
@property (nonatomic, readonly, assign) NSUInteger forkCount;
/**
 * The URL to the repository's HTML page.
 */
@property (nonatomic, readonly, copy) NSString *githubURL;
@property (nonatomic, readonly, copy) NSString *gitURL;
@property (nonatomic, readonly, assign) BOOL hasDownloads;
@property (nonatomic, readonly, assign) BOOL hasIssues;
@property (nonatomic, readonly, assign) BOOL hasWiki;
@property (nonatomic, readonly, copy) NSString *language;
@property (nonatomic, readonly, copy) NSString *masterBranch;
@property (nonatomic, readonly, copy) NSString *name;
@property (nonatomic, readonly, assign) NSUInteger openIssueCount;
@property (nonatomic, readonly, strong) GHOrganization *organization;
/**
 * The user or organization who owns this repository. The login attribute of this object is the username appearing in
 * the Github URLs to this resource.
 */
@property (nonatomic, readonly, copy) GHUser *owner;
@property (nonatomic, readonly, strong) GHRepository *parent;
@property (nonatomic, readonly, assign) BOOL private;
@property (nonatomic, readonly, copy) NSDate *pushedAt;
@property (nonatomic, readonly, assign) NSUInteger size;
@property (nonatomic, readonly, strong) GHRepository *source;
@property (nonatomic, readonly, copy) NSString *sshURL;
@property (nonatomic, readonly, copy) NSString *svnURL;
@property (nonatomic, readonly, assign) NSUInteger watcherCount;

# pragma mark - Github API attribute names

@property (nonatomic, readonly, copy) NSString *clone_url;
@property (nonatomic, readonly, copy) NSDate *created_at;
@property (nonatomic, readonly, assign) NSUInteger forks;
@property (nonatomic, readonly, copy) NSString *git_url;
@property (nonatomic, readonly, assign) BOOL has_downloads;
@property (nonatomic, readonly, assign) BOOL has_issues;
@property (nonatomic, readonly, assign) BOOL has_wiki;
@property (nonatomic, readonly, copy) NSString *html_url;
@property (nonatomic, readonly, copy) NSString *master_branch;
@property (nonatomic, readonly, assign) NSUInteger open_issues;
@property (nonatomic, readonly, copy) NSDate *pushed_at;
@property (nonatomic, readonly, copy) NSString *ssh_url;
@property (nonatomic, readonly, copy) NSString *svn_url;
@property (nonatomic, readonly, assign) NSUInteger watchers;

# pragma mark - Initializers

/**
 * Creates a repository with an owner and a name to locate a resource.
 *
 * \param owner the username of the repository's owner
 * \param name the name of the repository
 */
- (id)initWithOwner:(NSString *)owner name:(NSString *)name;

/**
 * Creates a repository with the given attributes.
 *
 * \param attributes the desired attributes, where each key is a property name
 */
- (id)initWithAttributes:(NSDictionary *)attributes;

# pragma mark - Github interaction

/**
 * Starts a fetch of the master branch HEAD ref from the Github API. The delegate will be notified when the fetch is
 * complete, if the delegate is a GHRepositoryDelegate.
 */
- (void)fetchMasterHeadRef;

/**
 * Starts a fetch of the given ref from the Github API. The delegate will be notified when the fetch is complete, if the
 * delegate is a GHRepositoryDelegate.
 */
- (void)fetchRef:(NSString *)ref;

@end
