#import "SBJson.h"
#import "GHRepository.h"
#import "GHRepositoryDelegate.h"
#import "GHRef.h"
#import "GHOrganization.h"

@interface GHRepository ()
- (void)updateFromGithubAttributes:(NSDictionary *)attributes;
@end

@implementation GHRepository

# pragma mark - Attributes

@dynamic clone_url;
@synthesize cloneURL = _cloneURL;
@dynamic created_at;
@synthesize createdAt = _createdAt;
@synthesize description = _description;
@synthesize fork = _fork;
@synthesize forkCount = _forkCount;
@dynamic forks;
@dynamic git_url;
@synthesize githubURL = _githubURL;
@synthesize gitURL = _gitURL;
@dynamic has_downloads;
@dynamic has_issues;
@dynamic has_wiki;
@synthesize hasDownloads = _hasDownloads;
@synthesize hasIssues = _hasIssues;
@synthesize hasWiki = _hasWiki;
@dynamic html_url;
@synthesize language = _language;
@dynamic master_branch;
@synthesize masterBranch = _masterBranch;
@synthesize name = _name;
@dynamic open_issues;
@synthesize openIssueCount = _openIssueCount;
@synthesize organization = _organization;
@synthesize owner = _owner;
@synthesize parent = _parent;
@synthesize private = _private;
@dynamic pushed_at;
@synthesize pushedAt = _pushedAt;
@synthesize size = _size;
@synthesize source = _source;
@dynamic ssh_url;
@dynamic svn_url;
@synthesize sshURL = _sshURL;
@synthesize svnURL = _svnURL;
@synthesize watcherCount = _watcherCount;
@dynamic watchers;

# pragma mark - Github API attribute names

- (NSString *)clone_url {
  return self.cloneURL;
}

- (NSDate *)created_at {
  return self.createdAt;
}

- (NSUInteger)forks {
  return self.forkCount;
}

- (NSString *)git_url {
  return self.gitURL;
}

- (BOOL)has_downloads {
  return self.hasDownloads;
}

- (BOOL)has_issues {
  return self.hasIssues;
}

- (BOOL)has_wiki {
  return self.hasWiki;
}

- (NSString *)html_url {
  return self.githubURL;
}

- (NSString *)master_branch {
  return self.masterBranch;
}

- (NSUInteger)open_issues {
  return self.openIssueCount;
}

- (NSDate *)pushed_at {
  return self.pushedAt;
}

- (NSString *)ssh_url {
  return self.sshURL;
}

- (NSString *)svn_url {
  return self.svnURL;
}

- (NSUInteger)watchers {
  return self.watcherCount;
}

# pragma mark - Initializers

- (id)init {
  return [self initWithAttributes:nil];
}

- (id)initWithOwner:(NSString *)owner name:(NSString *)name {
  return [self initWithAttributes:[NSDictionary dictionaryWithObjectsAndKeys:name, @"name",
                                   [[GHUser alloc] initWithLogin:owner], @"owner", nil]];
}

- (id)initWithAttributes:(NSDictionary *)attributes {
  self = [super init];
  if (self) {
    [self updateFromGithubAttributes:attributes];
    // provide special cases for those attributes named differently from the ones returned by Github
    NSString *cloneURL = [attributes valueForKey:@"cloneURL"];
    if (cloneURL) {
      _cloneURL = [cloneURL copy];
    }
    NSDate *createdAt = [attributes valueForKey:@"createdAt"];
    if (createdAt) {
      _createdAt = [createdAt copy];
    }
    NSNumber *forkCount = [attributes valueForKey:@"forkCount"];
    if (forkCount) {
      _forkCount = [forkCount integerValue];
    }
    NSString *githubURL = [attributes valueForKey:@"githubURL"];
    if (githubURL) {
      _githubURL = [githubURL copy];
    }
    NSString *gitURL = [attributes valueForKey:@"gitURL"];
    if (gitURL) {
      _gitURL = [gitURL copy];
    }
    NSNumber *hasDownloads = [attributes valueForKey:@"hasDownloads"];
    if (hasDownloads) {
      _hasDownloads = [hasDownloads boolValue];
    }
    NSNumber *hasIssues = [attributes valueForKey:@"hasIssues"];
    if (hasIssues) {
      _hasIssues = [hasIssues boolValue];
    }
    NSNumber *hasWiki = [attributes valueForKey:@"hasWiki"];
    if (hasWiki) {
      _hasWiki = [hasWiki boolValue];
    }
    NSString *masterBranch = [attributes valueForKey:@"masterBranch"];
    if (masterBranch) {
      _masterBranch = [masterBranch copy];
    }
    NSNumber *openIssueCount = [attributes valueForKey:@"openIssueCount"];
    if (openIssueCount) {
      _openIssueCount = [openIssueCount integerValue];
    }
    id organization = [attributes valueForKey:@"organization"];
    if ([organization isKindOfClass:[GHOrganization class]]) {
      _organization = organization;
    }
    id owner = [attributes valueForKey:@"owner"];
    if ([owner isKindOfClass:[GHUser class]]) {
      _owner = owner;
    }
    id parent = [attributes valueForKey:@"parent"];
    if ([parent isKindOfClass:[GHRepository class]]) {
      _parent = parent;
    }
    NSDate *pushedAt = [attributes valueForKey:@"pushedAt"];
    if (pushedAt) {
      _pushedAt = [pushedAt copy];
    }
    id source = [attributes valueForKey:@"source"];
    if ([source isKindOfClass:[GHRepository class]]) {
      _source = source;
    }
    NSString *sshURL = [attributes valueForKey:@"sshURL"];
    if (sshURL) {
      _sshURL = [sshURL copy];
    }
    NSString *svnURL = [attributes valueForKey:@"svnURL"];
    if (svnURL) {
      _svnURL = [svnURL copy];
    }
    NSNumber *watcherCount = [attributes valueForKey:@"watcherCount"];
    if (watcherCount) {
      _watcherCount = [watcherCount integerValue];
    }
  }
  return self;
}

# pragma mark - Github interaction

- (NSString *)apiPath {
  return [NSString stringWithFormat:@"/repos/%@/%@", self.owner.login, self.name];
}

- (void)fetchMasterHeadRef {
  NSString *ref = self.masterBranch;
  if (!ref) {
    ref = @"master";
  }
  ref = [NSString stringWithFormat:@"heads/%@", ref];
  [self fetchRef:ref];
}

- (void)fetchRef:(NSString *)ref {
  if (ref) {
    [self fetchDataAtApiPath:[NSString stringWithFormat:@"%@/git/refs/%@", [self apiPath], ref] withParameters:nil];
  }
}

# pragma mark - SBJsonStreamParserAdapter delegate methods

- (void)parser:(SBJsonStreamParser *)parser foundObject:(NSDictionary *)dict {
  if (dict) {
    NSLog(@"Found object: %@", [dict description]);
    NSLog(@"Ref: %@", [dict objectForKey:@"ref"]);
    NSLog(@"Responds: %@", [self.delegate respondsToSelector:@selector(repository:didFindRef:)] ? @"YES" : @"NO");
    if ([dict objectForKey:@"name"]) {
      // repository attributes
      NSLog(@"Found repository object: %@", [dict description]);
      [self updateFromGithubAttributes:dict];
    } else if ([dict objectForKey:@"ref"] && [self.delegate respondsToSelector:@selector(repository:didFindRef:)]) {
      // ref attributes
      NSLog(@"Repository found ref with attributes: %@", [dict description]);
      NSString *refString = [dict objectForKey:@"ref"];
      NSDictionary *objectAttributes = [dict objectForKey:@"object"];
      GHCommit *object = [[GHCommit alloc] initWithAttributes:objectAttributes];
      GHRef *ref = [[GHRef alloc] initWithRepository:self ref:refString object:object];
      [(id<GHRepositoryDelegate>)self.delegate repository:self didFindRef:ref];
    }
  }
}

# pragma mark - private

- (void)updateFromGithubAttributes:(NSDictionary *)attributes {
  NSDateFormatter *timestampFormatter = [[NSDateFormatter alloc] init];
  [timestampFormatter setDateFormat:[self.class timestampFormat]];
  NSString *cloneURL = [attributes valueForKey:@"clone_url"];
  if ((NSNull *)cloneURL == [NSNull null]) {
    _cloneURL = nil;
  } else if (cloneURL) {
    _cloneURL = [cloneURL copy];
  }
  NSString *createdAtTimestamp = [attributes valueForKey:@"created_at"];
  if ((NSNull *)createdAtTimestamp == [NSNull null]) {
    _createdAt = nil;
  } else if (createdAtTimestamp) {
    _createdAt = [timestampFormatter dateFromString:createdAtTimestamp];
  }
  NSString *description = [attributes valueForKey:@"description"];
  if ((NSNull *)description == [NSNull null]) {
    _description = nil;
  } else if (description) {
    _description = [description copy];
  }
  NSNumber *fork = [attributes valueForKey:@"fork"];
  if ((NSNull *)fork == [NSNull null]) {
    _fork = NO;
  } else if (fork) {
    _fork = [fork boolValue];
  }
  NSNumber *forkCount = [attributes valueForKey:@"forks"];
  if ((NSNull *)forkCount == [NSNull null]) {
    _forkCount = 0U;
  } else if (forkCount) {
    _forkCount = [forkCount integerValue];
  }
  NSString *gitURL = [attributes valueForKey:@"git_url"];
  if ((NSNull *)gitURL == [NSNull null]) {
    _gitURL = nil;
  } else if (gitURL) {
    _gitURL = [gitURL copy];
  }
  NSString *githubURL = [attributes valueForKey:@"html_url"];
  if ((NSNull *)githubURL == [NSNull null]) {
    _githubURL = nil;
  } else if (githubURL) {
    _githubURL = [githubURL copy];
  }
  NSNumber *hasDownloads = [attributes valueForKey:@"has_downloads"];
  if ((NSNull *)hasDownloads == [NSNull null]) {
    _hasDownloads = NO;
  } else if (hasDownloads) {
    _hasDownloads = [hasDownloads boolValue];
  }
  NSNumber *hasIssues = [attributes valueForKey:@"has_issues"];
  if ((NSNull *)hasIssues == [NSNull null]) {
    _hasIssues = NO;
  } else if (hasIssues) {
    _hasIssues = [hasIssues boolValue];
  }
  NSNumber *hasWiki = [attributes valueForKey:@"has_wiki"];
  if ((NSNull *)hasWiki == [NSNull null]) {
    _hasWiki = NO;
  } else if (hasWiki) {
    _hasWiki = [hasWiki boolValue];
  }
  NSString *language = [attributes valueForKey:@"language"];
  if ((NSNull *)language == [NSNull null]) {
    _language = nil;
  } else if (language) {
    _language = [language copy];
  }
  NSString *masterBranch = [attributes valueForKey:@"master_branch"];
  if ((NSNull *)masterBranch == [NSNull null]) {
    _masterBranch = nil;
  } else if (masterBranch) {
    _masterBranch = [masterBranch copy];
  }
  NSString *name = [attributes valueForKey:@"name"];
  if ((NSNull *)name == [NSNull null]) {
    _name = nil;
  } else if (name) {
    _name = [name copy];
  }
  NSNumber *openIssueCount = [attributes valueForKey:@"open_issues"];
  if ((NSNull *)openIssueCount == [NSNull null]) {
    _openIssueCount = 0U;
  } else if (openIssueCount) {
    _openIssueCount = [openIssueCount integerValue];
  }
  id organization = [attributes valueForKey:@"organization"];
  if ((NSNull *)organization == [NSNull null]) {
    _organization = nil;
  } else if ([organization isKindOfClass:[NSDictionary class]]) {
    _organization = [[GHOrganization alloc] initWithAttributes:organization];
  }
  id owner = [attributes valueForKey:@"owner"];
  if ((NSNull *)owner == [NSNull null]) {
    _owner = nil;
  } else if ([owner isKindOfClass:[NSDictionary class]]) {
    _owner = [[GHUser alloc] initWithAttributes:owner];
  }
  id parent = [attributes valueForKey:@"parent"];
  if ((NSNull *)parent == [NSNull null]) {
    _parent = nil;
  } else if ([parent isKindOfClass:[NSDictionary class]]) {
    _parent = [[GHRepository alloc] initWithAttributes:parent];
  }
  NSNumber *private = [attributes valueForKey:@"private"];
  if ((NSNull *)private == [NSNull null]) {
    _private = NO;
  } else if (private) {
    _private = [private boolValue];
  }
  NSString *pushedAtTimestamp = [attributes valueForKey:@"pushed_at"];
  if ((NSNull *)pushedAtTimestamp == [NSNull null]) {
    _pushedAt = nil;
  } else if (pushedAtTimestamp) {
    _pushedAt = [timestampFormatter dateFromString:pushedAtTimestamp];
  }
  NSNumber *size = [attributes valueForKey:@"size"];
  if ((NSNull *)size == [NSNull null]) {
    _size = 0U;
  } else if (size) {
    _size = [size integerValue];
  }
  id source = [attributes valueForKey:@"source"];
  if ((NSNull *)source == [NSNull null]) {
    _source = nil;
  } else if ([source isKindOfClass:[NSDictionary class]]) {
    _source = [[GHRepository alloc] initWithAttributes:source];
  }
  NSString *sshURL = [attributes valueForKey:@"ssh_url"];
  if ((NSNull *)sshURL == [NSNull null]) {
    _sshURL = nil;
  } else if (sshURL) {
    _sshURL = [sshURL copy];
  }
  NSString *svnURL = [attributes valueForKey:@"svn_url"];
  if ((NSNull *)svnURL == [NSNull null]) {
    _svnURL = nil;
  } else if (svnURL) {
    _svnURL = [svnURL copy];
  }
  NSNumber *watcherCount = [attributes valueForKey:@"watchers"];
  if ((NSNull *)watcherCount == [NSNull null]) {
    _watcherCount = 0U;
  } else if (watcherCount) {
    _watcherCount = [watcherCount integerValue];
  }
}

@end
