#import "SBJson.h"
#import "GHTree.h"
#import "GHErrors.h"
#import "GHBlob.h"

@implementation GHTree

static NSString *type = @"tree";

# pragma mark - Attributes

@synthesize leaves = _leaves;
@synthesize delegate = _delegate;

- (NSString *)type {
  return type;
}

# pragma mark - Initializers

- (id)initWithRepository:(GHRepository *)repository sha:(NSString *)sha {
  return [self initWithRepository:repository leaves:nil mode:nil path:nil sha:sha];
}

- (id)initWithRepository:(GHRepository *)repository mode:(NSString *)mode path:(NSString *)path sha:(NSString *)sha {
  return [self initWithRepository:repository leaves:nil mode:mode path:path sha:sha];
}

- (id)initWithRepository:(GHRepository *)repository leaves:(NSArray *)leaves mode:(NSString *)mode path:(NSString *)path sha:(NSString *)sha {
  self = [super init];
  if (self) {
    _repository = repository; // TODO: copy
    _leaves = [leaves copy];
    _mode = [mode copy];
    _path = [path copy];
    _sha = [sha copy];
  }
  return self;
}

# pragma mark - Github interaction

- (NSString *)apiPath {
  return [NSString stringWithFormat:@"/repos/%@/%@/git/trees/%@",
          self.repository.owner.login, self.repository.name, self.sha];
}

- (void)fetchImmediateLeaves {
  [self fetchData];
}

- (void)fetchLeaves {
  [self fetchDataWithParameters:[NSDictionary dictionaryWithObjectsAndKeys:@"1", @"recursive", nil]];
}

# pragma mark - NSCopying

- (id)copyWithZone:(NSZone *)zone {
  return [[GHTree allocWithZone:zone] initWithRepository:self.repository leaves:self.leaves mode:self.mode path:self.path sha:self.sha];
}

# pragma mark - NSFastEnumeration

- (NSUInteger)countByEnumeratingWithState:(NSFastEnumerationState *)state objects:(__unsafe_unretained id [])buffer count:(NSUInteger)len {
  return [self.leaves countByEnumeratingWithState:state objects:buffer count:len];
}

# pragma mark - SBJsonStreamParserAdapter delegate methods

- (void)parser:(SBJsonStreamParser*)parser foundObject:(NSDictionary*)dict {
  if (dict) {
    NSString *sha = [dict objectForKey:@"sha"];
    NSArray *leaves = [dict objectForKey:@"tree"];
    if (sha && leaves) {
      if ((NSNull *)leaves == [NSNull null]) {
        _leaves = [NSArray array];
      } else {
        NSMutableArray *newLeaves = [NSMutableArray arrayWithCapacity:[leaves count]];
        for (NSDictionary *leafAttributes in leaves) {
          NSString *type = [leafAttributes objectForKey:@"type"];
          NSString *mode = [leafAttributes objectForKey:@"mode"];
          NSString *path = [leafAttributes objectForKey:@"path"];
          NSString *leafSha = [leafAttributes objectForKey:@"sha"];
          if ([type isEqualToString:@"blob"]) {
            NSUInteger size = [((NSNumber *)[leafAttributes objectForKey:@"size"]) intValue];
            GHTreeLeaf *blob = [[GHBlob alloc] initWithRepository:self.repository mode:mode path:path sha:leafSha size:size];
            [newLeaves addObject:blob];
          } else if ([type isEqualToString:@"tree"]) {
            GHTreeLeaf *tree = [[GHTree alloc] initWithRepository:self.repository mode:mode path:path sha:leafSha];
            [newLeaves addObject:tree];
          }
        }
        _leaves = newLeaves;
      }
      if ((NSNull *)sha == [NSNull null]) {
        _sha = nil;
      } else {
        _sha = [sha copy];
      }
    }
  }
}

@end
