#import "GHGitObject.h"

/**
 * A tree leaf is an element contained in a GHTree. It contains information about the file, including its mode
 * (permissions) and path.
 *
 * This class is intended to be an abstract class. Subclasses should implement initializers or setter methods to set
 * these attributes.
 */
@interface GHTreeLeaf : GHGitObject {
  NSString *_mode;
  NSString *_path;
  NSUInteger _size;
}

# pragma mark - Attributes

/**
 * File permissions.
 */
@property (nonatomic, readonly, copy) NSString *mode;

/**
 * Path information, including filename.
 */
@property (nonatomic, readonly, copy) NSString *path;

/**
 * The size of the file, if this leaf is a blob.
 */
@property (nonatomic, readonly, assign) NSUInteger size;

@end
