#import "GithubTests.h"
#import "GHAbstractResource.h"

@interface GHAbstractResourceTests : GithubTests {
  GHAbstractResource *abstractResource;
}

# pragma mark - Attributes

/** repository tests */

/**
 * Should have a readable GHRepository attribute called repository.
 */
- (void)testRepository_GHRepository;

/**
 * Should not allow writes to repository.
 */
- (void)testSetRepository_NoMethod;

/** delegate tests */

/**
 * Should have a readable id<GHDelegate> attribute called delegate.
 */
- (void)testDelegate_GHDelegate;

/**
 * Should allow writes to delegate.
 */
- (void)testSetDelegate_GHDelegate_AssignedDelegate;

# pragma mark - Class methods

/** requestForPath: tests */

/**
 * Should create a request to the base Github API URL for a nil path.
 */
- (void)testRequestForPath_Nil_BaseURL;

/**
 * Should create a request to the given path at the Github API.
 */
- (void)testRequestForPath_Path_URLWithPath;

/** requestForPath:onBehalfOfUser: tests */

/**
 * Should create a request to the base Github API URL with no Authorization header for a nil path and nil user.
 */
//- (void)testRequestForPathOnBehalfOfUser_NilPathNilUser_BaseURLNoAuthorization;

/**
 * Should create a request to the base Github API URL with an Authorization header for a nil path and a Github user.
 */
//- (void)testRequestForPathOnBehalfOfUser_NilPathGithubUser_BaseURLWithAuthorization;

/**
 * Should create a request to the given path at the Github API with no Authorization header for a path and nil user.
 */
//- (void)testRequestForPathOnBehalfOfUser_PathNilUser_URLWithPathNoAuthorization;

/**
 * Should create a request to the given path at the Github API with an Authorization header for a path and a Github
 * user.
 */
//- (void)testRequestForPathOnBehalfOfUser_PathGithubUser_URLWithPathAndAuthorization;

# pragma mark - Instance methods

/** cancelConnection:withHTTPResponse: tests */

/**
 * Should handle nil connection.
 */
- (void)testCancelConnectionWithHTTPResponse_NilConnection_NoException;

/**
 * Should handle no delegate.
 */
- (void)testCancelConnectionWithHTTPResponse_NilDelegate_NoException;

/**
 * Should handle a delegate that does not respond to the githubObject:didCancelFetchWithError: selector.
 */
- (void)testCancelConnectionWithHTTPResponse_UnimplementedMethodDelegate_NoException;

/**
 * Should cancel the given NSURLConnection.
 */
- (void)testCancelConnectionWithHTTPResponse_Connection_CanceledConnection;

/**
 * Should notify the delegate of the canceled connection with the 304 Not Modified HTTP response if the delegate
 * implements the cancellation method.
 */
- (void)testCancelConnectionWithHTTPResponse_NotModifiedHTTPResponse_NotifiedDelegateResourceNotModified;

/**
 * Should notify the delegate of the canceled connection with the 401 Unauthorized HTTP response if the delegate
 * implements the cancellation method.
 */
- (void)testCancelConnectionWithHTTPResponse_UnauthorizedHTTPResponse_NotifiedDelegateResourceUnauthorized;

/**
 * Should notify the delegate of the canceled connection with the 403 Forbidden HTTP response if the delegate implements
 * the cancellation method.
 */
- (void)testCancelConnectionWithHTTPResponse_ForbiddenHTTPResponse_NotifiedDelegateResourceForbidden;

/**
 * Should notify the delegate of the canceled connection with the 404 Not Found HTTP response if the delegate implements
 * the cancellation method.
 */
- (void)testCancelConnectionWithHTTPResponse_NotFoundHTTPResponse_NotifiedDelegateResourceNotFound;

/**
 * Should notify the delegate of the canceled connection with an unknown response if the response is nil and the delegate
 * implements the cancellation method.
 */
- (void)testCancelConnectionWithHTTPResponse_NilHTTPResponse_NotifiedDelegateFetchCanceled;

/** cancelConnection:withResponse: tests */

/**
 * Should handle nil connection.
 */
- (void)testCancelConnectionWithResponse_NilConnection_NoException;

/**
 * Should handle no delegate.
 */
- (void)testCancelConnectionWithResponse_NilDelegate_NoException;

/**
 * Should handle a delegate that does not respond to the githubObject:didCancelFetchWithError: selector.
 */
- (void)testCancelConnectionWithResponse_UnimplementedMethodDelegate_NoException;

/**
 * Should cancel the given NSURLConnection.
 */
- (void)testCancelConnectionWithResponse_Connection_CanceledConnection;

/**
 * Should notify the delegate of the canceled connection with an unknown response if the response is nil and the
 * delegate implements the cancellation method.
 */
- (void)testCancelConnectionWithResponse_NilResponse_NotifiedDelegateFetchCanceled;

/**
 * Should notify the delegate of the canceled connection with an unknown response if there is a response and the
 * delegate implements the cancellation method.
 */
- (void)testCancelConnectionWithResponse_Response_NotifiedDelegateFetchCanceled;

# pragma mark - Github interaction

/** apiPath tests */

/**
 * Should return the root path.
 */
- (void)testApiPath_Nil_RootPath;

/** fetchDataAtApiPath:withParameters: tests */

/**
 * Should not create an NSURLConnection given a nil path.
 */
- (void)testFetchDataAtApiPathWithParameters_NilPath_NoConnection;

/**
 * Should create an NSURLConnection to the URL indicated by path.
 */
- (void)testFetchDataAtApiPathWithParameters_PathNoParameters_ConnectionToPath;

/**
 * Should create an NSURLConnection to the URL indicated by path, with the given parameters.
 */
- (void)testFetchDataAtApiPathWithParameters_PathWithParameters_ConnectionToPathWithParameters;

/** fetchDataWithParameters: tests */

/**
 * Should call fetchDataAtApiPath:withParameters: with the given parameters.
 */
- (void)testFetchDataWithParameters_FetchDataAtApiPathWithParameters;

/** fetchData tests */

/**
 * Should call fetchDataWithParameters: with no parameters.
 */
- (void)testFetchData_FetchDataWithNoParameters;

/** apiURL tests */

/**
 * Should concatenate the output of apiPath to the Github API base URL.
 */
- (void)testApiURL_Path_GithubBaseURLWithPath;

# pragma mark - NSURLConnection delegate methods

/** connection:didReceiveResponse: tests */

/**
 * Should not throw an exception for a nil connection and a nil response.
 */
- (void)testConnectionDidReceiveResponse_NilConnectionAndResponse_NoException;

/**
 * Should not destroy the existing JSON parser for a nil response.
 */
- (void)testConnectionDidReceiveResponse_NilResponse_ExistingJSONParser;

/**
 * Should cancel the connection and notify the delegate for a non-HTTP response.
 */
- (void)testConnectionDidReceiveResponse_NonHTTPResponse_ConnectionCanceledDelegateNotified;

/**
 * Should destroy the existing JSON parser for a non-HTTP response.
 */
- (void)testConnectionDidReceiveResponse_NonHTTPResponse_NilJSONParser;

/**
 * Should cancel the connection and notify the delegate for a non-successful HTTP response.
 */
- (void)testConnectionDidReceiveResponse_NonSuccessHTTPResponse_ConnectionCanceledDelegateNotified;

/**
 * Should destroy the existing JSON parser for a non-successful HTTP response.
 */
- (void)testConnectionDidReceiveResponse_NonSuccessHTTPResponse_NilJSONParser;

/**
 * Should create a new JSON parser for a successful response.
 */
- (void)testConnectionDidReceiveResponse_SuccessfulResponse_NewJSONParser;

/** connection:didReceiveData: tests */

/**
 * Should not throw an exception for a nil connection and nil data.
 */
- (void)testConnectionDidReceiveData_NilConnectionAndData_NoException;

/**
 * Should not throw an exception if there is no JSON parser.
 */
- (void)testConnectionDidReceiveData_NilParser_NoException;

/**
 * Should parse the given data.
 */
- (void)testConnectionDidReceiveData_Data_ParsedData;

/** connection:didFailWithError: tests */

/**
 * Should handle no delegate.
 */
- (void)testConnectionDidFailWithError_NilDelegate_NoException;

/**
 * Should handle a delegate that does not respond to the githubObject:didFailWithError: selector.
 */
- (void)testConnectionDidFailWithError_UnimplementedMethodDelegate_NoException;

/**
 * Should not throw an exception for a nil connection.
 */
- (void)testConnectionDidFailWithError_NilConnectionAndError_NoException;

/**
 * Should destroy the JSON parser and Github connection.
 */
- (void)testConnectionDidFailWithError_DestroyParserAndConnection;

/**
 * Should notify the delegate.
 */
- (void)testConnectionDidFailWithError_NotifiedDelegate;

/** connectionDidFinishLoading: tests */

/**
 * Should not throw an exception for a nil connection.
 */
- (void)testConnectionDidFinishLoading_NilConnection_NoException;

/**
 * Should destroy the JSON parser and Github connection.
 */
- (void)testConnectionDidFinishLoading_DestroyParserAndConnection;

/**
 * Should notify the delegate.
 */
- (void)testConnectionDidFinishLoading_NotifiedDelegate;

@end
