#import "GithubTests.h"
#import "GHBlob.h"

@interface GHBlobTests : GithubTests {
  GHBlob *blob;
}

# pragma mark - Class constructors

/** fetchBlobContentWithRepository:sha:delegate: tests */

/**
 * Should create a blob with nil attributes if nil attributes are given.
 */
- (void)testFetchBlobContentWithRepositoryShaDelegate_NilAttributes_NewBlobWithNilAttributes;

/**
 * Should create a blob with the correct attributes.
 */
- (void)testFetchBlobContentWithRepositoryShaDelegate_Attributes_NewBlobWithAttributes;

/**
 * Should start a content fetch on the new blob.
 */
- (void)testFetchBlobContentWithRepositoryShaDelegate_StartedFetch;

# pragma mark - Attributes

/** content tests */

/**
 * Should have a readable data attribute called content.
 */
- (void)testContent_NSData;

/**
 * Should not allow writes to content.
 */
- (void)testSetContent_NoMethod;

/** encoding tests */

/**
 * Should have an NSStringEncoding attribute called encoding.
 */
- (void)testEncoding_NSStringEncoding;

/**
 * Should not allow writes to encoding.
 */
- (void)testSetEncoding_NoMethod;

/** type tests */

/**
 * Should be a blob type.
 */
- (void)testType_Blob;

/** contentAsNSString tests */

/**
 * Should return nil if the content is nil.
 */
- (void)testContentAsNSString_NilContent_Nil;

/**
 * Should convert the content to an NSString using the stored encoding.
 */
- (void)testContentAsNSString_Encoding_StringUsingEncoding;

/**
 * Should convert the content to an NSString using the default encoding if proprietary encoding is stored. Proprietary
 * encoding indicates non-text content.
 */
- (void)testContentAsNSString_ProprietaryEncoding_StringUsingDefaultEncoding;

/** isText tests */

/**
 * Should return YES if the encoding is a valid text encoding.
 */
- (void)testIsText_TextEncoding_Yes;

/**
 * Should return NO if the encoding is NSProprietaryStringEncoding.
 */
- (void)testIsText_NSProprietaryStringEncoding_No;

/**
 * Should return NO if the encoding is 0.
 */
- (void)testIsText_0Encoding_No;

# pragma mark - Initializers

/** initWithRepository:sha: tests */

/**
 * Should create a blob with nil attributes if nil attributes are given.
 */
- (void)testInitWithRepositorySha_NilAttributes_NewBlobWithNilAttributes;

/**
 * Should create a blob with the given attributes and nil content and encoding.
 */
- (void)testInitWithRepositorySha_Attributes_NewBlobWithCorrectAttributes;

/** initWithRepository:mode:path:sha:size: tests */

/**
 * Should create a blob with nil attributes if nil attributes are given.
 */
- (void)testInitWithRepositoryModePathShaSize_NilAttributes_NewBlobWithNilAttributes;

/**
 * Should create a blob with the given attributes and nil content and encoding.
 */
- (void)testInitWithRepositoryModePathShaSize_Attributes_NewBlobWithCorrectAttributes;

/** initWithRepository:content:encoding:mode:path:sha:size: tests */

/**
 * Should create a blob with nil attributes if nil attributes are given.
 */
- (void)testInitWithRepositoryContentEncodingModePathShaSize_NilAttributes_NewBlobWithNilAttributes;

/**
 * Should create a blob with the given attributes.
 */
- (void)testInitWithRepositoryContentEncodingModePathShaSize_Attributes_NewBlobWithAttributes;

# pragma mark - Github interaction

/** apiPath tests */

/**
 * Should return the path to this blob's sha.
 */
- (void)testApiPath_BlobPath;

/** fetchContent tests */

/**
 * Should call fetchData.
 */
- (void)testFetchContent_FetchData;

# pragma mark - NSCopying

/**
 * Should allow copying via copyWithZone:.
 */
- (void)testCopying_CopyWithZone_SameObjects;

/**
 * Should allow copying via copy.
 */
- (void)testCopying_Copy_SameObjects;

# pragma mark - SBJsonStreamParserAdapter delegate methods

/** parser:foundObject: tests */

/**
 * Should not store anything given nil.
 */
- (void)testParserFoundObject_Nil_UnchangedAttributes;

/**
 * Should not store anything given an object with the wrong keys.
 */
- (void)testParserFoundObject_DifferentKeys_UnchangedAttributes;

/**
 * Should store UTF-8 encoded data correctly.
 */
- (void)testParserFoundObject_UTF8Content_UTF8EncodedContent;

/**
 * Should store base64-encoded data correctly.
 */
- (void)testParserFoundObject_Base64Content_UTF8EncodedContent;

/**
 * Should overwrite the existing data with the data found in the object.
 */
- (void)testParserFoundObject_ExistingData_UpdatedAttributes;

/**
 * Should store NSNull data correctly.
 */
- (void)testParserFoundObject_BlobWithNSNullAttributes_CorrectAttributes;

@end
