#import "GithubTests.h"
#import "GHCommit.h"

@interface GHCommitTests : GithubTests {
  GHCommit *commit;
}

# pragma mark - Attributes

/** author tests */

/**
 * Should have a dictionary property called author.
 */
- (void)testAuthor_NSDictionary;

/**
 * Should not allow writes to author.
 */
- (void)testSetAuthor_NoMethod;

/** committer tests */

/**
 * Should have a dictionary property called committer.
 */
- (void)testCommitter_NSDictionary;

/**
 * Should not allow writes to committer.
 */
- (void)testSetCommitter_NoMethod;

/** message tests */

/**
 * Should have a string property called message.
 */
- (void)testMessage_NSString;

/**
 * Should not allow writes to message.
 */
- (void)testSetMessage_NoMethod;

/** parents tests */

/**
 * Should have an array property called parents.
 */
- (void)testParents_NSArray;

/**
 * Should not allow writes to parents.
 */
- (void)testSetParents_NoMethod;

/** tree tests */

/**
 * Should have a tree property called tree.
 */
- (void)testTree_GHTree;

/**
 * Should not allow writes to tree.
 */
- (void)testSetTree_NoMethod;

/** type tests */

/**
 * Should be a commit type.
 */
- (void)testType_Commit;

# pragma mark - Initializers

/** initWithRepository:sha: tests */

/**
 * Should create a commit with nil attributes if nil attributes are given.
 */
- (void)testInitWithRepositorySha_NilAttributes_NewCommitWithNilAttributes;

/**
 * Should create a commit with the given attributes and nil other attributes.
 */
- (void)testInitWithRepositorySha_Attributes_NewCommitWithCorrectAttributes;

/** initWithAttributes: tests */

/**
 * Should create a commit with nil attributes given a nil attribute dictionary.
 */
- (void)testInitWithAttributes_Nil_NilAttributes;

/**
 * Should create a commit with nil attributes given an empty dictionary.
 */
- (void)testInitWithAttributes_Empty_NilAttributes;

/**
 * Should create a commit with the given attributes, and no extras.
 */
- (void)testInitWithAttributes_Attributes_StoredAttributes;

/**
 * Should create a commit with the given NSNull Github API returned attributes.
 */
- (void)testInitWithAttributes_GithubAPINullAttributes_StoredAttributes;

# pragma mark - Github interaction

/** apiPath tests */

/**
 * Should return the commit path with the repository owner and name.
 */
- (void)testApiPath_CommitPath;

# pragma mark - SBJsonStreamParser delegate methods

/** parser:foundObject: tests */

/**
 * Should not store anything given nil.
 */
- (void)testParserFoundObject_Nil_UnchangedAttributes;

/**
 * Should not store anything given an object with the wrong keys.
 */
- (void)testParserFoundObject_DifferentKeys_UnchangedAttributes;

/**
 * Should store data correctly.
 */
- (void)testParserFoundObject_Commit_CorrectAttributes;

/**
 * Should overwrite the existing data with the data found in the object.
 */
- (void)testParserFoundObject_ExistingData_UpdatedAttributes;

/**
 * Should store NSNull data correctly.
 */
- (void)testParserFoundObject_CommitWithNSNullAttributes_CorrectAttributes;

@end
