#import "GHOrganizationTests.h"
#import "GHOrganizationDelegate.h"
#import <OCMock/OCMock.h>
#import "GHRepository.h"

@interface MockGHOrganizationDelegate : MockGHDelegate <GHOrganizationDelegate>
@property (nonatomic, copy, readonly) NSArray *repositories;
@property (nonatomic, strong) GHOrganization *organization;
@end
@implementation MockGHOrganizationDelegate
@synthesize repositories = _repositories;
@synthesize organization = _organization;
- (void)organization:(GHOrganization *)organization didFindRepository:(GHRepository *)repository {
  self.called = YES;
  self.organization = organization;
  if (!_repositories) {
    _repositories = [NSArray array];
  }
  _repositories = [_repositories arrayByAddingObject:repository];
}
@end

@implementation GHOrganizationTests

- (void)setUp {
  [super setUp];
  organization = [[GHOrganization alloc] init];
}

- (void)tearDown {
  organization = nil;
  [super tearDown];
}

# pragma mark - Attributes

/** avatarURL tests */

- (void)testAvatarURL_NSString {
  [organization setValue:@"test" forKey:@"_avatarURL"];
  STAssertTrue([@"test" isEqualToString:organization.avatarURL], @"should have set avatarURL");
}

- (void)testSetAvatarURL_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setAvatarURL:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setAvatarURL: selector");
}

/** billingEmail tests */

- (void)testBillingEmail_NSString {
  [organization setValue:@"test" forKey:@"_billingEmail"];
  STAssertTrue([@"test" isEqualToString:organization.billingEmail], @"should have set billing email");
}

- (void)testSetBillingEmail_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setBillingEmail:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setBillingEmail: selector");
}

/** blog tests */

- (void)testBlog_NSString {
  [organization setValue:@"test" forKey:@"_blog"];
  STAssertTrue([@"test" isEqualToString:organization.blog], @"should have set blog");
}

- (void)testSetBlog_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setBlog:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setBlog: selector");
}

/** collaboratorCount tests */

- (void)testCollaboratorCount_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_collaboratorCount"];
  STAssertEquals(1U, organization.collaboratorCount, @"should have set collaboratorCount");
}

- (void)testSetCollaboratorCount_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setCollaboratorCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCollaboratorCount: selector");
}

/** company tests */

- (void)testCompany_NSString {
  [organization setValue:@"test" forKey:@"_company"];
  STAssertTrue([@"test" isEqualToString:organization.company], @"should have set company");
}

- (void)testSetCompany_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setCompany:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCompany: selector");
}

/** createdAt tests */

- (void)testCreatedAt_NSDate {
  NSDate *date = [NSDate date];
  [organization setValue:date forKey:@"_createdAt"];
  STAssertTrue([date isEqualToDate:organization.createdAt], @"should have set createdAt");
}

- (void)testSetCreatedAt_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setCreatedAt:) withObject:[NSDate date]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCreatedAt: selector");
}

/** diskUsage tests */

- (void)testDiskUsage_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_diskUsage"];
  STAssertEquals(1U, organization.diskUsage, @"should have set diskUsage");
}

- (void)testSetDiskUsage_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setDiskUsage:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setDiskUsage: selector");
}

/** email tests */

- (void)testEmail_NSString {
  [organization setValue:@"test" forKey:@"_email"];
  STAssertTrue([@"test" isEqualToString:organization.email], @"should have set email");
}

- (void)testSetEmail_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setEmail:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setEmail: selector");
}

/** followerCount tests */

- (void)testFollowerCount_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_followerCount"];
  STAssertEquals(1U, organization.followerCount, @"should have set followerCount");
}

- (void)testSetFollowerCount_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setFollowerCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setFollowerCount: selector");
}

/** followingCount tests */

- (void)testFollowingCount_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_followingCount"];
  STAssertEquals(1U, organization.followingCount, @"should have set followingCount");
}

- (void)testSetFollowingCount_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setFollowingCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setFollowingCount: selector");
}

/** githubURL tests */

- (void)testGithubURL_NSString {
  [organization setValue:@"test" forKey:@"_githubURL"];
  STAssertTrue([@"test" isEqualToString:organization.githubURL], @"should have set githubURL");
}

- (void)testSetGithubURL_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setGithubURL:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setGithubURL: selector");
}

/** location tests */

- (void)testLocation_NSString {
  [organization setValue:@"test" forKey:@"_location"];
  STAssertTrue([@"test" isEqualToString:organization.location], @"should have set location");
}

- (void)testSetLocation_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setLocation:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setLocation: selector");
}

/** login tests */

- (void)testLogin_NSString {
  [organization setValue:@"test" forKey:@"_login"];
  STAssertTrue([@"test" isEqualToString:organization.login], @"should have set login");
}

- (void)testSetLogin_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setLogin:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setLogin: selector");
}

/** name tests */

- (void)testName_NSString {
  [organization setValue:@"test" forKey:@"_name"];
  STAssertTrue([@"test" isEqualToString:organization.name], @"should have set name");
}

- (void)testSetName_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setName:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setName: selector");
}

/** ownedRepositoryCount tests */

- (void)testOwnedRepositoryCount_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_ownedRepositoryCount"];
  STAssertEquals(1U, organization.ownedRepositoryCount, @"should have set ownedRepositoryCount");
}

- (void)testSetOwnedRepositoryCount_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setOwnedRepositoryCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setOwnedRepositoryCount: selector");
}

/** plan tests */

- (void)testPlan_NSDictionary {
  [organization setValue:[NSDictionary dictionary] forKey:@"_plan"];
  STAssertEquals([NSDictionary dictionary], organization.plan, @"should have set plan");
}

- (void)testSetPlan_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setPlan:) withObject:[NSDictionary dictionary]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPlan: selector");
}

/** privateGistCount tests */

- (void)testPrivateGistCount_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_privateGistCount"];
  STAssertEquals(1U, organization.privateGistCount, @"should have set privateGistCount");
}

- (void)testSetPrivateGistCount_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setPrivateGistCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPrivateGistCount: selector");
}

/** privateRepositoryCount tests */

- (void)testPrivateRepositoryCount_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_privateRepositoryCount"];
  STAssertEquals(1U, organization.privateRepositoryCount, @"should have set privateRepositoryCount");
}

- (void)testSetPrivateRepositoryCount_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setPrivateRepositoryCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPrivateRepositoryCount: selector");
}

/** publicGistCount tests */

- (void)testPublicGistCount_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_publicGistCount"];
  STAssertEquals(1U, organization.publicGistCount, @"should have set publicGistCount");
}

- (void)testSetPublicGistCount_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setPublicGistCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPublicGistCount: selector");
}

/** publicRepositoryCount tests */

- (void)testPublicRepositoryCount_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_publicRepositoryCount"];
  STAssertEquals(1U, organization.publicRepositoryCount, @"should have set publicRepositoryCount");
}

- (void)testSetPublicRepositoryCount_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setPublicRepositoryCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPublicRepositoryCount: selector");
}

# pragma mark - Github API attribute names

/** avatar_url tests */

- (void)testAvatar_url_NSString {
  [organization setValue:@"test" forKey:@"_avatarURL"];
  STAssertTrue([@"test" isEqualToString:organization.avatar_url], @"should have set avatar_url");
}

- (void)testSetAvatar_url_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setAvatar_url:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setAvatar_url: selector");
}

/** billing_email tests */

- (void)testBilling_email_NSString {
  [organization setValue:@"test" forKey:@"_billingEmail"];
  STAssertTrue([@"test" isEqualToString:organization.billing_email], @"should have set billing email");
}

- (void)testSetBilling_email_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setBilling_email:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setBilling_email: selector");
}

/** collaborators tests */

- (void)testCollaborators_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_collaboratorCount"];
  STAssertEquals(1U, organization.collaborators, @"should have set collaborators");
}

- (void)testSetCollaborators_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setCollaborators:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCollaborators: selector");
}

/** created_at tests */

- (void)testCreated_at_NSDate {
  NSDate *date = [NSDate date];
  [organization setValue:date forKey:@"_createdAt"];
  STAssertTrue([date isEqualToDate:organization.created_at], @"should have set created_at");
}

- (void)testSetCreated_at_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setCreated_at:) withObject:[NSDate date]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCreated_at: selector");
}

/** disk_usage tests */

- (void)testDisk_usage_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_diskUsage"];
  STAssertEquals(1U, organization.disk_usage, @"should have set disk_usage");
}

- (void)testSetDisk_usage_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setDisk_usage:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setDisk_usage: selector");
}

/** followers tests */

- (void)testFollowers_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_followerCount"];
  STAssertEquals(1U, organization.followers, @"should have set followers");
}

- (void)testSetFollowers_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setFollowers:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setFollowers: selector");
}

/** following tests */

- (void)testFollowing_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_followingCount"];
  STAssertEquals(1U, organization.following, @"should have set following");
}

- (void)testSetFollowing_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setFollowing:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setFollowing: selector");
}

/** html_url tests */

- (void)testHtml_url_NSString {
  [organization setValue:@"test" forKey:@"_githubURL"];
  STAssertTrue([@"test" isEqualToString:organization.html_url], @"should have set html_url");
}

- (void)testSetHtml_url_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setHtml_url:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setHtml_url: selector");
}

/** owned_private_repos tests */

- (void)testOwned_private_repos_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_ownedRepositoryCount"];
  STAssertEquals(1U, organization.owned_private_repos, @"should have set owned_private_repos");
}

- (void)testSetOwned_private_repos_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setOwned_private_repos:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setOwned_private_repos: selector");
}

/** private_gists tests */

- (void)testPrivate_gists_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_privateGistCount"];
  STAssertEquals(1U, organization.private_gists, @"should have set private_gists");
}

- (void)testSetPrivate_gists_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setPrivate_gists:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPrivate_gists: selector");
}

/** public_gists tests */

- (void)testPublic_gists_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_publicGistCount"];
  STAssertEquals(1U, organization.public_gists, @"should have set public_gists");
}

- (void)testSetPublic_gists_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setPublic_gists:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPublic_gists: selector");
}

/** public_repos tests */

- (void)testPublic_repos_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_publicRepositoryCount"];
  STAssertEquals(1U, organization.public_repos, @"should have set public_repos");
}

- (void)testSetPublic_repos_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setPublic_repos:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPublic_repos: selector");
}

/** total_private_repos tests */

- (void)testTotal_private_repos_NSUInteger {
  [organization setValue:[NSNumber numberWithInteger:1U] forKey:@"_privateRepositoryCount"];
  STAssertEquals(1U, organization.total_private_repos, @"should have set total_private_repos");
}

- (void)testSetTotal_private_repos_NoMethod {
  STAssertThrowsSpecificNamed([organization performSelector:@selector(setTotal_private_repos:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setTotal_private_repos: selector");
}

# pragma mark - Initializers

/** initWithLogin: tests */

- (void)testInitWithLogin_NilLogin_NilAttributes {
  organization = [organization initWithLogin:nil];
  STAssertNil(organization.avatarURL, @"should have nil avatar URL");
  STAssertNil(organization.billingEmail, @"should have nil billingEmail");
  STAssertNil(organization.blog, @"should have nil blog");
  STAssertEquals(0U, organization.collaboratorCount, @"should have 0 collaborators");
  STAssertNil(organization.company, @"should have nil company");
  STAssertNil(organization.createdAt, @"should have nil createdAt");
  STAssertEquals(0U, organization.diskUsage, @"should have 0 disk usage");
  STAssertNil(organization.email, @"should have nil email");
  STAssertEquals(0U, organization.followerCount, @"should have 0 followers");
  STAssertEquals(0U, organization.followingCount, @"should have 0 following");
  STAssertNil(organization.githubURL, @"should have nil Github URL");
  STAssertNil(organization.location, @"should have nil location");
  STAssertNil(organization.login, @"should have nil login");
  STAssertNil(organization.name, @"should have nil name");
  STAssertEquals(0U, organization.ownedRepositoryCount, @"should have 0 owned repositories");
  STAssertNil(organization.plan, @"should have nil plan");
  STAssertEquals(0U, organization.privateGistCount, @"should have 0 private gists");
  STAssertEquals(0U, organization.privateRepositoryCount, @"should have 0 private repositories");
  STAssertEquals(0U, organization.publicGistCount, @"should have 0 public gists");
  STAssertEquals(0U, organization.publicRepositoryCount, @"should have 0 public repositories");
}

- (void)testInitWithLogin_Login_StoredLogin {
  organization = [organization initWithLogin:@"tester"];
  STAssertNil(organization.avatarURL, @"should have nil avatar URL");
  STAssertNil(organization.billingEmail, @"should have nil billingEmail");
  STAssertNil(organization.blog, @"should have nil blog");
  STAssertEquals(0U, organization.collaboratorCount, @"should have 0 collaborators");
  STAssertNil(organization.company, @"should have nil company");
  STAssertNil(organization.createdAt, @"should have nil createdAt");
  STAssertEquals(0U, organization.diskUsage, @"should have 0 disk usage");
  STAssertNil(organization.email, @"should have nil email");
  STAssertEquals(0U, organization.followerCount, @"should have 0 followers");
  STAssertEquals(0U, organization.followingCount, @"should have 0 following");
  STAssertNil(organization.githubURL, @"should have nil Github URL");
  STAssertNil(organization.location, @"should have nil location");
  STAssertTrue([@"tester" isEqualToString:organization.login], @"should have set login");
  STAssertNil(organization.name, @"should have nil name");
  STAssertEquals(0U, organization.ownedRepositoryCount, @"should have 0 owned repositories");
  STAssertNil(organization.plan, @"should have nil plan");
  STAssertEquals(0U, organization.privateGistCount, @"should have 0 private gists");
  STAssertEquals(0U, organization.privateRepositoryCount, @"should have 0 private repositories");
  STAssertEquals(0U, organization.publicGistCount, @"should have 0 public gists");
  STAssertEquals(0U, organization.publicRepositoryCount, @"should have 0 public repositories");
}

/** initWithAttributes: tests */

- (void)testInitWithAttributes_Nil_NilAttributes {
  organization = [organization initWithAttributes:nil];
  STAssertNil(organization.avatarURL, @"should have nil avatar URL");
  STAssertNil(organization.billingEmail, @"should have nil billingEmail");
  STAssertNil(organization.blog, @"should have nil blog");
  STAssertEquals(0U, organization.collaboratorCount, @"should have 0 collaborators");
  STAssertNil(organization.company, @"should have nil company");
  STAssertNil(organization.createdAt, @"should have nil createdAt");
  STAssertEquals(0U, organization.diskUsage, @"should have 0 disk usage");
  STAssertNil(organization.email, @"should have nil email");
  STAssertEquals(0U, organization.followerCount, @"should have 0 followers");
  STAssertEquals(0U, organization.followingCount, @"should have 0 following");
  STAssertNil(organization.githubURL, @"should have nil Github URL");
  STAssertNil(organization.location, @"should have nil location");
  STAssertNil(organization.login, @"should have nil login");
  STAssertNil(organization.name, @"should have nil name");
  STAssertEquals(0U, organization.ownedRepositoryCount, @"should have 0 owned repositories");
  STAssertNil(organization.plan, @"should have nil plan");
  STAssertEquals(0U, organization.privateGistCount, @"should have 0 private gists");
  STAssertEquals(0U, organization.privateRepositoryCount, @"should have 0 private repositories");
  STAssertEquals(0U, organization.publicGistCount, @"should have 0 public gists");
  STAssertEquals(0U, organization.publicRepositoryCount, @"should have 0 public repositories");
}

- (void)testInitWithAttributes_Empty_NilAttributes {
  organization = [organization initWithAttributes:[NSDictionary dictionary]];
  STAssertNil(organization.avatarURL, @"should have nil avatar URL");
  STAssertNil(organization.billingEmail, @"should have nil billingEmail");
  STAssertNil(organization.blog, @"should have nil blog");
  STAssertEquals(0U, organization.collaboratorCount, @"should have 0 collaborators");
  STAssertNil(organization.company, @"should have nil company");
  STAssertNil(organization.createdAt, @"should have nil createdAt");
  STAssertEquals(0U, organization.diskUsage, @"should have 0 disk usage");
  STAssertNil(organization.email, @"should have nil email");
  STAssertEquals(0U, organization.followerCount, @"should have 0 followers");
  STAssertEquals(0U, organization.followingCount, @"should have 0 following");
  STAssertNil(organization.githubURL, @"should have nil Github URL");
  STAssertNil(organization.location, @"should have nil location");
  STAssertNil(organization.login, @"should have nil login");
  STAssertNil(organization.name, @"should have nil name");
  STAssertEquals(0U, organization.ownedRepositoryCount, @"should have 0 owned repositories");
  STAssertNil(organization.plan, @"should have nil plan");
  STAssertEquals(0U, organization.privateGistCount, @"should have 0 private gists");
  STAssertEquals(0U, organization.privateRepositoryCount, @"should have 0 private repositories");
  STAssertEquals(0U, organization.publicGistCount, @"should have 0 public gists");
  STAssertEquals(0U, organization.publicRepositoryCount, @"should have 0 public repositories");
}

- (void)testInitWithAttributes_Attributes_StoredAttributes {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  [attributes setValue:@"testAvatarURL" forKey:@"avatarURL"];
  [attributes setValue:@"testBillingEmail" forKey:@"billingEmail"];
  [attributes setValue:@"testBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"collaboratorCount"];
  [attributes setValue:@"testCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate date];
  [attributes setValue:createdAt forKey:@"createdAt"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"diskUsage"];
  [attributes setValue:@"testEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followerCount"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followingCount"];
  [attributes setValue:@"testGithubURL" forKey:@"githubURL"];
  [attributes setValue:@"testGravatarId" forKey:@"gravatarId"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hireable"];
  [attributes setValue:@"testLocation" forKey:@"location"];
  [attributes setValue:@"testLogin" forKey:@"login"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"ownedRepositoryCount"];
  [attributes setValue:[NSDictionary dictionary] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"privateGistCount"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"privateRepositoryCount"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"publicGistCount"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"publicRepositoryCount"];
  organization = [organization initWithAttributes:attributes];
  STAssertTrue([@"testAvatarURL" isEqualToString:organization.avatarURL], @"should have stored avatar URL");
  STAssertTrue([@"testBillingEmail" isEqualToString:organization.billingEmail], @"should have stored billingEmail");
  STAssertTrue([@"testBlog" isEqualToString:organization.blog], @"should have stored blog");
  STAssertEquals(1U, organization.collaboratorCount, @"should have stored collaborators");
  STAssertTrue([@"testCompany" isEqualToString:organization.company], @"should have stored company");
  STAssertTrue([createdAt isEqualToDate:organization.createdAt], @"should have stored createdAt");
  STAssertEquals(1U, organization.diskUsage, @"should have stored disk usage");
  STAssertTrue([@"testEmail" isEqualToString:organization.email], @"should have stored email");
  STAssertEquals(1U, organization.followerCount, @"should have stored followers");
  STAssertEquals(1U, organization.followingCount, @"should have stored following");
  STAssertTrue([@"testGithubURL" isEqualToString:organization.githubURL], @"should have stored Github URL");
  STAssertTrue([@"testLocation" isEqualToString:organization.location], @"should have stored location");
  STAssertTrue([@"testLogin" isEqualToString:organization.login], @"should have stored login");
  STAssertTrue([@"testName" isEqualToString:organization.name], @"should have stored name");
  STAssertEquals(1U, organization.ownedRepositoryCount, @"should have stored owned repositories");
  STAssertTrue([[NSDictionary dictionary] isEqualToDictionary:organization.plan], @"should have stored plan");
  STAssertEquals(1U, organization.privateGistCount, @"should have stored private gists");
  STAssertEquals(1U, organization.privateRepositoryCount, @"should have stored private repositories");
  STAssertEquals(1U, organization.publicGistCount, @"should have stored public gists");
  STAssertEquals(1U, organization.publicRepositoryCount, @"should have stored public repositories");
}

- (void)testInitWithAttributes_GithubAPIAttributes_StoredAttributes {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  [attributes setValue:@"testAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"testBillingEmail" forKey:@"billing_email"];
  [attributes setValue:@"testBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"collaborators"];
  [attributes setValue:@"testCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHOrganization timestampFormat]];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"disk_usage"];
  [attributes setValue:@"testEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"following"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hireable"];
  [attributes setValue:@"testLocation" forKey:@"location"];
  [attributes setValue:@"testLogin" forKey:@"login"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionary] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_repos"];
  organization = [organization initWithAttributes:attributes];
  STAssertTrue([@"testAvatarURL" isEqualToString:organization.avatarURL], @"should have stored avatar URL");
  STAssertTrue([@"testBillingEmail" isEqualToString:organization.billingEmail], @"should have stored billingEmail");
  STAssertTrue([@"testBlog" isEqualToString:organization.blog], @"should have stored blog");
  STAssertEquals(1U, organization.collaboratorCount, @"should have stored collaborators");
  STAssertTrue([@"testCompany" isEqualToString:organization.company], @"should have stored company");
  STAssertTrue([createdAt isEqualToDate:organization.createdAt], @"should have stored createdAt");
  STAssertEquals(1U, organization.diskUsage, @"should have stored disk usage");
  STAssertTrue([@"testEmail" isEqualToString:organization.email], @"should have stored email");
  STAssertEquals(1U, organization.followerCount, @"should have stored followers");
  STAssertEquals(1U, organization.followingCount, @"should have stored following");
  STAssertTrue([@"testGithubURL" isEqualToString:organization.githubURL], @"should have stored Github URL");
  STAssertTrue([@"testLocation" isEqualToString:organization.location], @"should have stored location");
  STAssertTrue([@"testLogin" isEqualToString:organization.login], @"should have stored login");
  STAssertTrue([@"testName" isEqualToString:organization.name], @"should have stored name");
  STAssertEquals(1U, organization.ownedRepositoryCount, @"should have stored owned repositories");
  STAssertTrue([[NSDictionary dictionary] isEqualToDictionary:organization.plan], @"should have stored plan");
  STAssertEquals(1U, organization.privateGistCount, @"should have stored private gists");
  STAssertEquals(1U, organization.privateRepositoryCount, @"should have stored private repositories");
  STAssertEquals(1U, organization.publicGistCount, @"should have stored public gists");
  STAssertEquals(1U, organization.publicRepositoryCount, @"should have stored public repositories");
}

- (void)testInitWithAttributes_GithubAPINullAttributes_StoredAttributes {
  NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
  [attributes setValue:[NSNull null] forKey:@"avatar_url"];
  [attributes setValue:[NSNull null] forKey:@"billing_email"];
  [attributes setValue:[NSNull null] forKey:@"blog"];
  [attributes setValue:[NSNull null] forKey:@"collaborators"];
  [attributes setValue:[NSNull null] forKey:@"company"];
  [attributes setValue:[NSNull null] forKey:@"created_at"];
  [attributes setValue:[NSNull null] forKey:@"disk_usage"];
  [attributes setValue:[NSNull null] forKey:@"email"];
  [attributes setValue:[NSNull null] forKey:@"followers"];
  [attributes setValue:[NSNull null] forKey:@"following"];
  [attributes setValue:[NSNull null] forKey:@"html_url"];
  [attributes setValue:[NSNull null] forKey:@"gravatar_id"];
  [attributes setValue:[NSNull null] forKey:@"hireable"];
  [attributes setValue:[NSNull null] forKey:@"location"];
  [attributes setValue:[NSNull null] forKey:@"login"];
  [attributes setValue:[NSNull null] forKey:@"name"];
  [attributes setValue:[NSNull null] forKey:@"owned_private_repos"];
  [attributes setValue:[NSNull null] forKey:@"plan"];
  [attributes setValue:[NSNull null] forKey:@"private_gists"];
  [attributes setValue:[NSNull null] forKey:@"total_private_repos"];
  [attributes setValue:[NSNull null] forKey:@"public_gists"];
  [attributes setValue:[NSNull null] forKey:@"public_repos"];
  organization = [organization initWithAttributes:attributes];
  STAssertNil(organization.avatarURL, @"should have nil avatar URL");
  STAssertNil(organization.billingEmail, @"should have nil billing email");
  STAssertNil(organization.blog, @"should have nil blog");
  STAssertEquals(0U, organization.collaboratorCount, @"should have 0 collaborators");
  STAssertNil(organization.company, @"should have nil company");
  STAssertNil(organization.createdAt, @"should have nil createdAt");
  STAssertEquals(0U, organization.diskUsage, @"should have 0 disk usage");
  STAssertNil(organization.email, @"should have nil email");
  STAssertEquals(0U, organization.followerCount, @"should have 0 followers");
  STAssertEquals(0U, organization.followingCount, @"should have 0 following");
  STAssertNil(organization.githubURL, @"should have nil Github URL");
  STAssertNil(organization.location, @"should have nil location");
  STAssertNil(organization.login, @"should have nil login");
  STAssertNil(organization.name, @"should have nil name");
  STAssertEquals(0U, organization.ownedRepositoryCount, @"should have 0 owned repositories");
  STAssertNil(organization.plan, @"should have nil plan");
  STAssertEquals(0U, organization.privateGistCount, @"should have 0 private gists");
  STAssertEquals(0U, organization.privateRepositoryCount, @"should have 0 private repositories");
  STAssertEquals(0U, organization.publicGistCount, @"should have 0 public gists");
  STAssertEquals(0U, organization.publicRepositoryCount, @"should have 0 public repositories");
}

# pragma mark - Github interaction

/** apiPath tests */

- (void)testApiPath_PublicOrganizationPath {
  NSString *login = @"tester";
  organization = [organization initWithLogin:login];
  NSString *path = [NSString stringWithFormat:@"/orgs/%@", login];
  STAssertTrue([path isEqualToString:[organization apiPath]], @"should return public organization path");
}

/** fetchRepositories tests */

- (void)testFetchRepositories_FetchDataAtApiPathWithNoParameters {
  id mockOrganization = [OCMockObject partialMockForObject:organization];
  NSString *repoPath = [[mockOrganization apiPath] stringByAppendingString:@"/repos"];
  [[mockOrganization expect] fetchDataAtApiPath:[OCMArg checkWithSelector:@selector(isEqualToString:) onObject:repoPath] withParameters:nil];
  [mockOrganization fetchRepositories];
  STAssertNoThrow([mockOrganization verify], @"should have called fetchDataAtApiPath:/orgs/:organization/repos withParameters:nil");
}

# pragma mark - SBJsonStreamParserAdapter delegate methods

/** parser:foundArray: tests */

- (void)testParserFoundArray_Nil_UnnotifiedDelegate {
  MockGHOrganizationDelegate *delegate = [[MockGHOrganizationDelegate alloc] init];
  organization.delegate = delegate;
  id parser = [OCMockObject mockForClass:[SBJsonStreamParserAdapter class]];
  [organization parser:parser foundArray:nil];
  STAssertFalse(delegate.called, @"should not have called the delegate for an empty array");
}

- (void)testParserFoundArray_Empty_UnnotifiedDelegate {
  MockGHOrganizationDelegate *delegate = [[MockGHOrganizationDelegate alloc] init];
  organization.delegate = delegate;
  id parser = [OCMockObject mockForClass:[SBJsonStreamParserAdapter class]];
  [organization parser:parser foundArray:[NSArray array]];
  STAssertFalse(delegate.called, @"should not have called the delegate for an empty array");
}

- (void)testParserFoundArray_Repositories_NotifiedDelegate {
  MockGHOrganizationDelegate *delegate = [[MockGHOrganizationDelegate alloc] init];
  organization.delegate = delegate;
  NSMutableDictionary *repository1Attributes = [NSDictionary dictionaryWithObjectsAndKeys:@"repository1", @"name", nil];
  NSMutableDictionary *repository2Attributes = [NSDictionary dictionaryWithObjectsAndKeys:@"repository2", @"name", nil];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParserAdapter class]];
  [organization parser:parser foundArray:[NSArray arrayWithObjects:repository1Attributes, repository2Attributes, nil]];
  STAssertEquals(2U, [delegate.repositories count], @"should have found 2 repositories");
  GHRepository *repository1 = [delegate.repositories objectAtIndex:0U];
  STAssertTrue([@"repository1" isEqualToString:repository1.name], @"should have found first repository correctly");
  GHRepository *repository2 = [delegate.repositories objectAtIndex:1U];
  STAssertTrue([@"repository2" isEqualToString:repository2.name], @"should have found second repository correctly");
}

/** parser:foundObject: tests */

- (void)testParserFoundObject_Nil_UnchangedAttributes {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  [attributes setValue:@"testAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"testBillingEmail" forKey:@"billing_email"];
  [attributes setValue:@"testBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"collaborators"];
  [attributes setValue:@"testCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHOrganization timestampFormat]];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"disk_usage"];
  [attributes setValue:@"testEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"following"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hireable"];
  [attributes setValue:@"testLocation" forKey:@"location"];
  [attributes setValue:@"testLogin" forKey:@"login"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionary] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_repos"];
  organization = [organization initWithAttributes:attributes];
  [organization parser:nil foundObject:nil];
  STAssertTrue([@"testAvatarURL" isEqualToString:organization.avatarURL], @"should have stored avatar URL");
  STAssertTrue([@"testBillingEmail" isEqualToString:organization.billingEmail], @"should have stored billing email");
  STAssertTrue([@"testBlog" isEqualToString:organization.blog], @"should have stored blog");
  STAssertEquals(1U, organization.collaboratorCount, @"should have stored collaborators");
  STAssertTrue([@"testCompany" isEqualToString:organization.company], @"should have stored company");
  STAssertTrue([createdAt isEqualToDate:organization.createdAt], @"should have stored createdAt");
  STAssertEquals(1U, organization.diskUsage, @"should have stored disk usage");
  STAssertTrue([@"testEmail" isEqualToString:organization.email], @"should have stored email");
  STAssertEquals(1U, organization.followerCount, @"should have stored followers");
  STAssertEquals(1U, organization.followingCount, @"should have stored following");
  STAssertTrue([@"testGithubURL" isEqualToString:organization.githubURL], @"should have stored Github URL");
  STAssertTrue([@"testLocation" isEqualToString:organization.location], @"should have stored location");
  STAssertTrue([@"testLogin" isEqualToString:organization.login], @"should have stored login");
  STAssertTrue([@"testName" isEqualToString:organization.name], @"should have stored name");
  STAssertEquals(1U, organization.ownedRepositoryCount, @"should have stored owned repositories");
  STAssertTrue([[NSDictionary dictionary] isEqualToDictionary:organization.plan], @"should have stored plan");
  STAssertEquals(1U, organization.privateGistCount, @"should have stored private gists");
  STAssertEquals(1U, organization.privateRepositoryCount, @"should have stored private repositories");
  STAssertEquals(1U, organization.publicGistCount, @"should have stored public gists");
  STAssertEquals(1U, organization.publicRepositoryCount, @"should have stored public repositories");
}

- (void)testParserFoundObject_DifferentKeys_UnchangedAttributes {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  [attributes setValue:@"testAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"testBillingEmail" forKey:@"billing_email"];
  [attributes setValue:@"testBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"collaborators"];
  [attributes setValue:@"testCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHOrganization timestampFormat]];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"disk_usage"];
  [attributes setValue:@"testEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"following"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hireable"];
  [attributes setValue:@"testLocation" forKey:@"location"];
  [attributes setValue:@"testLogin" forKey:@"login"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionary] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_repos"];
  organization = [organization initWithAttributes:attributes];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParser class]];
  [organization parser:parser foundObject:[NSDictionary dictionaryWithObject:@"testing" forKey:@"loginn"]];
  STAssertTrue([@"testAvatarURL" isEqualToString:organization.avatarURL], @"should have stored avatar URL");
  STAssertTrue([@"testBillingEmail" isEqualToString:organization.billingEmail], @"should have stored billing email");
  STAssertTrue([@"testBlog" isEqualToString:organization.blog], @"should have stored blog");
  STAssertEquals(1U, organization.collaboratorCount, @"should have stored collaborators");
  STAssertTrue([@"testCompany" isEqualToString:organization.company], @"should have stored company");
  STAssertTrue([createdAt isEqualToDate:organization.createdAt], @"should have stored createdAt");
  STAssertEquals(1U, organization.diskUsage, @"should have stored disk usage");
  STAssertTrue([@"testEmail" isEqualToString:organization.email], @"should have stored email");
  STAssertEquals(1U, organization.followerCount, @"should have stored followers");
  STAssertEquals(1U, organization.followingCount, @"should have stored following");
  STAssertTrue([@"testGithubURL" isEqualToString:organization.githubURL], @"should have stored Github URL");
  STAssertTrue([@"testLocation" isEqualToString:organization.location], @"should have stored location");
  STAssertTrue([@"testLogin" isEqualToString:organization.login], @"should have stored login");
  STAssertTrue([@"testName" isEqualToString:organization.name], @"should have stored name");
  STAssertEquals(1U, organization.ownedRepositoryCount, @"should have stored owned repositories");
  STAssertTrue([[NSDictionary dictionary] isEqualToDictionary:organization.plan], @"should have stored plan");
  STAssertEquals(1U, organization.privateGistCount, @"should have stored private gists");
  STAssertEquals(1U, organization.privateRepositoryCount, @"should have stored private repositories");
  STAssertEquals(1U, organization.publicGistCount, @"should have stored public gists");
  STAssertEquals(1U, organization.publicRepositoryCount, @"should have stored public repositories");
}

- (void)testParserFoundObject_Organization_CorrectAttributes {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  [attributes setValue:@"testAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"testBillingEmail" forKey:@"billing_email"];
  [attributes setValue:@"testBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"collaborators"];
  [attributes setValue:@"testCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHOrganization timestampFormat]];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"disk_usage"];
  [attributes setValue:@"testEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"following"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hireable"];
  [attributes setValue:@"testLocation" forKey:@"location"];
  [attributes setValue:@"testLogin" forKey:@"login"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionary] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_repos"];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParser class]];
  [organization parser:parser foundObject:attributes];
  STAssertTrue([@"testAvatarURL" isEqualToString:organization.avatarURL], @"should have stored avatar URL");
  STAssertTrue([@"testBillingEmail" isEqualToString:organization.billingEmail], @"should have stored billing email");
  STAssertTrue([@"testBlog" isEqualToString:organization.blog], @"should have stored blog");
  STAssertEquals(1U, organization.collaboratorCount, @"should have stored collaborators");
  STAssertTrue([@"testCompany" isEqualToString:organization.company], @"should have stored company");
  STAssertTrue([createdAt isEqualToDate:organization.createdAt], @"should have stored createdAt");
  STAssertEquals(1U, organization.diskUsage, @"should have stored disk usage");
  STAssertTrue([@"testEmail" isEqualToString:organization.email], @"should have stored email");
  STAssertEquals(1U, organization.followerCount, @"should have stored followers");
  STAssertEquals(1U, organization.followingCount, @"should have stored following");
  STAssertTrue([@"testGithubURL" isEqualToString:organization.githubURL], @"should have stored Github URL");
  STAssertTrue([@"testLocation" isEqualToString:organization.location], @"should have stored location");
  STAssertTrue([@"testLogin" isEqualToString:organization.login], @"should have stored login");
  STAssertTrue([@"testName" isEqualToString:organization.name], @"should have stored name");
  STAssertEquals(1U, organization.ownedRepositoryCount, @"should have stored owned repositories");
  STAssertTrue([[NSDictionary dictionary] isEqualToDictionary:organization.plan], @"should have stored plan");
  STAssertEquals(1U, organization.privateGistCount, @"should have stored private gists");
  STAssertEquals(1U, organization.privateRepositoryCount, @"should have stored private repositories");
  STAssertEquals(1U, organization.publicGistCount, @"should have stored public gists");
  STAssertEquals(1U, organization.publicRepositoryCount, @"should have stored public repositories");
}

- (void)testParserFoundObject_ExistingData_UpdatedAttributes {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  [attributes setValue:@"oldAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"oldBillingEmail" forKey:@"billing_email"];
  [attributes setValue:@"oldBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"collaborators"];
  [attributes setValue:@"oldCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1100];
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHOrganization timestampFormat]];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"disk_usage"];
  [attributes setValue:@"oldEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"following"];
  [attributes setValue:@"oldGithubURL" forKey:@"html_url"];
  [attributes setValue:@"oldGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:NO] forKey:@"hireable"];
  [attributes setValue:@"oldLocation" forKey:@"location"];
  [attributes setValue:@"oldLogin" forKey:@"login"];
  [attributes setValue:@"oldName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionaryWithObject:@"medium" forKey:@"name"] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"public_repos"];
  organization = [organization initWithAttributes:attributes];
  [attributes setValue:@"testAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"testBillingEmail" forKey:@"billing_email"];
  [attributes setValue:@"testBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"collaborators"];
  [attributes setValue:@"testCompany" forKey:@"company"];
  createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"disk_usage"];
  [attributes setValue:@"testEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"following"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hireable"];
  [attributes setValue:@"testLocation" forKey:@"location"];
  [attributes setValue:@"testLogin" forKey:@"login"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionary] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_repos"];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParser class]];
  [organization parser:parser foundObject:attributes];
  STAssertTrue([@"testAvatarURL" isEqualToString:organization.avatarURL], @"should have stored avatar URL");
  STAssertTrue([@"testBillingEmail" isEqualToString:organization.billingEmail], @"should have stored billing email");
  STAssertTrue([@"testBlog" isEqualToString:organization.blog], @"should have stored blog");
  STAssertEquals(1U, organization.collaboratorCount, @"should have stored collaborators");
  STAssertTrue([@"testCompany" isEqualToString:organization.company], @"should have stored company");
  STAssertTrue([createdAt isEqualToDate:organization.createdAt], @"should have stored createdAt");
  STAssertEquals(1U, organization.diskUsage, @"should have stored disk usage");
  STAssertTrue([@"testEmail" isEqualToString:organization.email], @"should have stored email");
  STAssertEquals(1U, organization.followerCount, @"should have stored followers");
  STAssertEquals(1U, organization.followingCount, @"should have stored following");
  STAssertTrue([@"testGithubURL" isEqualToString:organization.githubURL], @"should have stored Github URL");
  STAssertTrue([@"testLocation" isEqualToString:organization.location], @"should have stored location");
  STAssertTrue([@"testLogin" isEqualToString:organization.login], @"should have stored login");
  STAssertTrue([@"testName" isEqualToString:organization.name], @"should have stored name");
  STAssertEquals(1U, organization.ownedRepositoryCount, @"should have stored owned repositories");
  STAssertTrue([[NSDictionary dictionary] isEqualToDictionary:organization.plan], @"should have stored plan");
  STAssertEquals(1U, organization.privateGistCount, @"should have stored private gists");
  STAssertEquals(1U, organization.privateRepositoryCount, @"should have stored private repositories");
  STAssertEquals(1U, organization.publicGistCount, @"should have stored public gists");
  STAssertEquals(1U, organization.publicRepositoryCount, @"should have stored public repositories");
}

- (void)testParserFoundObject_OrganizationWithNSNullAttributes_CorrectAttributes {
  NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
  [attributes setValue:@"oldAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"oldBillingEmail" forKey:@"billing_email"];
  [attributes setValue:@"oldBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"collaborators"];
  [attributes setValue:@"oldCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1100];
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHOrganization timestampFormat]];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"disk_usage"];
  [attributes setValue:@"oldEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"following"];
  [attributes setValue:@"oldGithubURL" forKey:@"html_url"];
  [attributes setValue:@"oldGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:NO] forKey:@"hireable"];
  [attributes setValue:@"oldLocation" forKey:@"location"];
  [attributes setValue:@"oldLogin" forKey:@"login"];
  [attributes setValue:@"oldName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionaryWithObject:@"medium" forKey:@"name"] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"public_repos"];
  organization = [organization initWithAttributes:attributes];
  [attributes setValue:[NSNull null] forKey:@"avatar_url"];
  [attributes setValue:[NSNull null] forKey:@"billing_email"];
  [attributes setValue:[NSNull null] forKey:@"blog"];
  [attributes setValue:[NSNull null] forKey:@"collaborators"];
  [attributes setValue:[NSNull null] forKey:@"company"];
  [attributes setValue:[NSNull null] forKey:@"created_at"];
  [attributes setValue:[NSNull null] forKey:@"disk_usage"];
  [attributes setValue:[NSNull null] forKey:@"email"];
  [attributes setValue:[NSNull null] forKey:@"followers"];
  [attributes setValue:[NSNull null] forKey:@"following"];
  [attributes setValue:[NSNull null] forKey:@"html_url"];
  [attributes setValue:[NSNull null] forKey:@"gravatar_id"];
  [attributes setValue:[NSNull null] forKey:@"hireable"];
  [attributes setValue:[NSNull null] forKey:@"location"];
  [attributes setValue:[NSNull null] forKey:@"login"];
  [attributes setValue:[NSNull null] forKey:@"name"];
  [attributes setValue:[NSNull null] forKey:@"owned_private_repos"];
  [attributes setValue:[NSNull null] forKey:@"plan"];
  [attributes setValue:[NSNull null] forKey:@"private_gists"];
  [attributes setValue:[NSNull null] forKey:@"total_private_repos"];
  [attributes setValue:[NSNull null] forKey:@"public_gists"];
  [attributes setValue:[NSNull null] forKey:@"public_repos"];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParser class]];
  [organization parser:parser foundObject:attributes];
  STAssertNil(organization.avatarURL, @"should have nil avatar URL");
  STAssertNil(organization.billingEmail, @"should have nil billing email");
  STAssertNil(organization.blog, @"should have nil blog");
  STAssertEquals(0U, organization.collaboratorCount, @"should have 0 collaborators");
  STAssertNil(organization.company, @"should have nil company");
  STAssertNil(organization.createdAt, @"should have nil createdAt");
  STAssertEquals(0U, organization.diskUsage, @"should have 0 disk usage");
  STAssertNil(organization.email, @"should have nil email");
  STAssertEquals(0U, organization.followerCount, @"should have 0 followers");
  STAssertEquals(0U, organization.followingCount, @"should have 0 following");
  STAssertNil(organization.githubURL, @"should have nil Github URL");
  STAssertNil(organization.location, @"should have nil location");
  STAssertNil(organization.login, @"should have nil login");
  STAssertNil(organization.name, @"should have nil name");
  STAssertEquals(0U, organization.ownedRepositoryCount, @"should have 0 owned repositories");
  STAssertNil(organization.plan, @"should have nil plan");
  STAssertEquals(0U, organization.privateGistCount, @"should have 0 private gists");
  STAssertEquals(0U, organization.privateRepositoryCount, @"should have 0 private repositories");
  STAssertEquals(0U, organization.publicGistCount, @"should have 0 public gists");
  STAssertEquals(0U, organization.publicRepositoryCount, @"should have 0 public repositories");
}

@end
