#import "GHRepositoryTests.h"
#import <OCMock/OCMock.h>
#import "GHRepositoryDelegate.h"
#import "GHRef.h"
#import "GHOrganization.h"

@interface MockGHRepositoryDelegate : MockGHDelegate <GHRepositoryDelegate>
@property (nonatomic, copy, readonly) GHRef *ref;
@property (nonatomic, strong) GHRepository *repository;
@end
@implementation MockGHRepositoryDelegate
@synthesize ref = _ref;
@synthesize repository = _repository;
- (void)repository:(GHRepository *)repository didFindRef:(GHRef *)ref {
  self.called = YES;
  self.repository = repository;
  _ref = ref;
}
@end

@implementation GHRepositoryTests

- (void)setUp {
  [super setUp];
  repository = [[GHRepository alloc] init];
}

- (void)tearDown {
  repository = nil;
  [super tearDown];
}

# pragma mark - Attributes

/** cloneURL tests */

- (void)testCloneURL_NSString {
  [repository setValue:@"test" forKey:@"_cloneURL"];
  STAssertTrue([@"test" isEqualToString:repository.cloneURL], @"should have set cloneURL");
}

- (void)testSetCloneURL_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setCloneURL:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCloneURL: selector");
}

/** createdAt tests */

- (void)testCreatedAt_NSDate {
  NSDate *date = [NSDate date];
  [repository setValue:date forKey:@"_createdAt"];
  STAssertTrue([date isEqualToDate:repository.createdAt], @"should have set createdAt");
}

- (void)testSetCreatedAt_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setCreatedAt:) withObject:[NSDate date]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCreatedAt: selector");
}

/** description tests */

- (void)testDescription_NSString {
  [repository setValue:@"test" forKey:@"_description"];
  STAssertTrue([@"test" isEqualToString:repository.description], @"should have set description");
}

- (void)testSetDescription_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setDescription:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setDescription: selector");
}

/** fork tests */

- (void)testFork_Bool {
  [repository setValue:[NSNumber numberWithBool:YES] forKey:@"_fork"];
  STAssertEquals(YES, repository.fork, @"should have set fork");
}

- (void)testSetFork_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setFork:) withObject:[NSNumber numberWithBool:YES]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setFork: selector");
}

/** forkCount tests */

- (void)testForkCount_NSUInteger {
  [repository setValue:[NSNumber numberWithInteger:1U] forKey:@"_forkCount"];
  STAssertEquals(1U, repository.forkCount, @"should have set forkCount");
}

- (void)testSetForkCount_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setForkCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setForkCount: selector");
}

/** githubURL tests */

- (void)testGithubURL_NSString {
  [repository setValue:@"test" forKey:@"_githubURL"];
  STAssertTrue([@"test" isEqualToString:repository.githubURL], @"should have set githubURL");
}

- (void)testSetGithubURL_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setGithubURL:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setGithubURL: selector");
}

/** gitURL tests */

- (void)testGitURL_NSString {
  [repository setValue:@"test" forKey:@"_gitURL"];
  STAssertTrue([@"test" isEqualToString:repository.gitURL], @"should have set gitURL");
}

- (void)testSetGitURL_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setGitURL:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setGitURL: selector");
}

/** hasDownloads tests */

- (void)testHasDownloads_Bool {
  [repository setValue:[NSNumber numberWithBool:YES] forKey:@"_hasDownloads"];
  STAssertEquals(YES, repository.hasDownloads, @"should have set hasDownloads");
}

- (void)testSetHasDownloads_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setHasDownloads:) withObject:[NSNumber numberWithBool:YES]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setHasDownloads: selector");
}

/** hasIssues tests */

- (void)testHasIssues_Bool {
  [repository setValue:[NSNumber numberWithBool:YES] forKey:@"_hasIssues"];
  STAssertEquals(YES, repository.hasIssues, @"should have set hasIssues");
}

- (void)testSetHasIssues_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setHasIssues:) withObject:[NSNumber numberWithBool:YES]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setHasIssues: selector");
}

/** hasWiki tests */

- (void)testHasWiki_Bool {
  [repository setValue:[NSNumber numberWithBool:YES] forKey:@"_hasWiki"];
  STAssertEquals(YES, repository.hasWiki, @"should have set hasWiki");
}

- (void)testSetHasWiki_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setHasWiki:) withObject:[NSNumber numberWithBool:YES]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setHasWiki: selector");
}

/** language tests */

- (void)testLanguage_NSString {
  [repository setValue:@"test" forKey:@"_language"];
  STAssertTrue([@"test" isEqualToString:repository.language], @"should have set language");
}

- (void)testSetLanguage_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setLanguage:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setLanguage: selector");
}

/** masterBranch tests */

- (void)testMasterBranch_NSString {
  [repository setValue:@"test" forKey:@"_masterBranch"];
  STAssertTrue([@"test" isEqualToString:repository.masterBranch], @"should have set masterBranch");
}

- (void)testSetMasterBranch_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setMasterBranch:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setMasterBranch: selector");
}

/** name tests */

- (void)testName_NSString {
  [repository setValue:@"test" forKey:@"_name"];
  STAssertTrue([@"test" isEqualToString:repository.name], @"should have set name");
}

- (void)testSetName_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setName:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setName: selector");
}

/** openIssueCount tests */

- (void)testOpenIssueCount_NSUInteger {
  [repository setValue:[NSNumber numberWithInteger:1U] forKey:@"_openIssueCount"];
  STAssertEquals(1U, repository.openIssueCount, @"should have set openIssueCount");
}

- (void)testSetOpenIssueCount_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setOpenIssueCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setOpenIssueCount: selector");
}

/** organization tests */

- (void)testOrganization_GHOrganization {
  GHOrganization *organization = [[GHOrganization alloc] init];
  [repository setValue:organization forKey:@"_organization"];
  STAssertEquals(organization, repository.organization, @"should have set organization");
}

- (void)testSetOrganization_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setOrganization:)
                                               withObject:[[GHOrganization alloc] init]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setOrganization: selector");
}

/** owner tests */

- (void)testOwner_GHUser {
  GHUser *user = [[GHUser alloc] init];
  [repository setValue:user forKey:@"_owner"];
  STAssertEquals(user, repository.owner, @"should have set owner");
}

- (void)testSetOwner_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setOwner:) withObject:[[GHUser alloc] init]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setOwner: selector");
}

/** parent tests */

- (void)testParent_GHRepository {
  GHRepository *repo = [[GHRepository alloc] init];
  [repository setValue:repo forKey:@"_parent"];
  STAssertEquals(repo, repository.parent, @"should have set parent");
}

- (void)testSetParent_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setParent:) withObject:[[GHRepository alloc] init]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setParent: selector");
}

/** private tests */

- (void)testPrivate_Bool {
  [repository setValue:[NSNumber numberWithBool:YES] forKey:@"_private"];
  STAssertEquals(YES, repository.private, @"should have set private");
}

- (void)testSetPrivate_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setPrivate:) withObject:[NSNumber numberWithBool:YES]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPrivate: selector");
}

/** pushedAt tests */

- (void)testPushedAt_NSDate {
  NSDate *date = [NSDate date];
  [repository setValue:date forKey:@"_pushedAt"];
  STAssertTrue([date isEqualToDate:repository.pushedAt], @"should have set pushedAt");
}

- (void)testSetPushedAt_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setPushedAt:) withObject:[NSDate date]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPushedAt: selector");
}

/** size tests */

- (void)testSize_NSUInteger {
  [repository setValue:[NSNumber numberWithInteger:1U] forKey:@"_size"];
  STAssertEquals(1U, repository.size, @"should have set size");
}

- (void)testSetSize_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setSize:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setSize: selector");
}

/** source tests */

- (void)testSource_GHRepository {
  GHRepository *repo = [[GHRepository alloc] init];
  [repository setValue:repo forKey:@"_source"];
  STAssertEquals(repo, repository.source, @"should have set source");
}

- (void)testSetSource_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setSource:) withObject:[[GHRepository alloc] init]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setSource: selector");
}

/** sshURL tests */

- (void)testSshURL_NSString {
  [repository setValue:@"test" forKey:@"_sshURL"];
  STAssertTrue([@"test" isEqualToString:repository.sshURL], @"should have set sshURL");
}

- (void)testSetSshURL_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setSshURL:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setSshURL: selector");
}

/** svnURL tests */

- (void)testSvnURL_NSString {
  [repository setValue:@"test" forKey:@"_svnURL"];
  STAssertTrue([@"test" isEqualToString:repository.svnURL], @"should have set svnURL");
}

- (void)testSetSvnURL_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setSvnURL:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setSvnURL: selector");
}

/** watcherCount tests */

- (void)testWatcherCount_NSUInteger {
  [repository setValue:[NSNumber numberWithInteger:1U] forKey:@"_watcherCount"];
  STAssertEquals(1U, repository.watcherCount, @"should have set watcherCount");
}

- (void)testSetWatcherCount_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setWatcherCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setWatcherCount: selector");
}

# pragma mark - Github API attribute names

/** clone_url tests */

- (void)testClone_url_NSString {
  [repository setValue:@"test" forKey:@"_cloneURL"];
  STAssertTrue([@"test" isEqualToString:repository.clone_url], @"should have set clone_url");
}

- (void)testSetClone_url_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setClone_url:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setClone_url: selector");
}

/** created_at tests */

- (void)testCreated_at_NSDate {
  NSDate *date = [NSDate date];
  [repository setValue:date forKey:@"_createdAt"];
  STAssertTrue([date isEqualToDate:repository.created_at], @"should have set created_at");
}

- (void)testSetCreated_at_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setCreated_at:) withObject:[NSDate date]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCreated_at: selector");
}

/** forks tests */

- (void)testForks_NSUInteger {
  [repository setValue:[NSNumber numberWithInteger:1U] forKey:@"_forkCount"];
  STAssertEquals(1U, repository.forks, @"should have set forks");
}

- (void)testSetForks_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setForks:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setForks: selector");
}

/** git_url tests */

- (void)testGit_url_NSString {
  [repository setValue:@"test" forKey:@"_gitURL"];
  STAssertTrue([@"test" isEqualToString:repository.git_url], @"should have set git_url");
}

- (void)testSetGit_url_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setGit_url:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setGit_url: selector");
}

/** has_downloads tests */

- (void)testHas_downloads_Bool {
  [repository setValue:[NSNumber numberWithBool:YES] forKey:@"_hasDownloads"];
  STAssertEquals(YES, repository.has_downloads, @"should have set has_downloads");
}

- (void)testSetHas_downloads_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setHas_downloads:) withObject:[NSNumber numberWithBool:YES]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setHas_downloads: selector");
}

/** has_issues tests */

- (void)testHas_issues_Bool {
  [repository setValue:[NSNumber numberWithBool:YES] forKey:@"_hasIssues"];
  STAssertEquals(YES, repository.has_issues, @"should have set has_issues");
}

- (void)testSetHas_issues_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setHas_issues:) withObject:[NSNumber numberWithBool:YES]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setHas_issues: selector");
}

/** has_wiki tests */

- (void)testHas_wiki_Bool {
  [repository setValue:[NSNumber numberWithBool:YES] forKey:@"_hasWiki"];
  STAssertEquals(YES, repository.has_wiki, @"should have set has_wiki");
}

- (void)testSetHas_wiki_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setHas_wiki:) withObject:[NSNumber numberWithBool:YES]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setHas_wiki: selector");
}

/** html_url tests */

- (void)testHtml_url_NSString {
  [repository setValue:@"test" forKey:@"_githubURL"];
  STAssertTrue([@"test" isEqualToString:repository.html_url], @"should have set html_url");
}

- (void)testSetHtml_url_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setHtml_url:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setHtml_url: selector");
}

/** master_branch tests */

- (void)testMaster_branch_NSString {
  [repository setValue:@"test" forKey:@"_masterBranch"];
  STAssertTrue([@"test" isEqualToString:repository.master_branch], @"should have set master_branch");
}

- (void)testSetMaster_branch_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setMaster_branch:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setMaster_branch: selector");
}

/** open_issues tests */

- (void)testOpen_issues_NSUInteger {
  [repository setValue:[NSNumber numberWithInteger:1U] forKey:@"_openIssueCount"];
  STAssertEquals(1U, repository.open_issues, @"should have set open_issues");
}

- (void)testSetOpen_issues_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setOpen_issues:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setOpen_issues: selector");
}

/** pushed_at tests */

- (void)testPushed_at_NSDate {
  NSDate *date = [NSDate date];
  [repository setValue:date forKey:@"_pushedAt"];
  STAssertTrue([date isEqualToDate:repository.pushed_at], @"should have set pushed_at");
}

- (void)testSetPushed_at_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setPushed_at:) withObject:[NSDate date]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPushed_at: selector");
}

/** ssh_url tests */

- (void)testSsh_url_NSString {
  [repository setValue:@"test" forKey:@"_sshURL"];
  STAssertTrue([@"test" isEqualToString:repository.ssh_url], @"should have set ssh_url");
}

- (void)testSetSsh_url_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setSsh_url:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setSsh_url: selector");
}

/** svn_url tests */

- (void)testSvn_url_NSString {
  [repository setValue:@"test" forKey:@"_svnURL"];
  STAssertTrue([@"test" isEqualToString:repository.svn_url], @"should have set svn_url");
}

- (void)testSetSvn_url_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setSvn_url:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setSvn_url: selector");
}

/** watchers tests */

- (void)testWatchers_NSUInteger {
  [repository setValue:[NSNumber numberWithInteger:1U] forKey:@"_watcherCount"];
  STAssertEquals(1U, repository.watchers, @"should have set watchers");
}

- (void)testSetWatchers_NoMethod {
  STAssertThrowsSpecificNamed([repository performSelector:@selector(setWatchers:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setWatchers: selector");
}

# pragma mark - Initializers

/** initWithOwner:name: tests */

- (void)testInitWithOwnerName_NilAttributes_NilAttributes {
  repository = [repository initWithOwner:nil name:nil];
  STAssertNil(repository.cloneURL, @"should not have set cloneURL");
  STAssertNil(repository.createdAt, @"should not have set createdAt");
  STAssertNil(repository.description, @"should not have set description");
  STAssertFalse(repository.fork, @"should not have set fork to true");
  STAssertEquals(0U, repository.forkCount, @"should have set forkCount to 0");
  STAssertNil(repository.githubURL, @"should not have set githubURL");
  STAssertNil(repository.gitURL, @"should not have set gitURL");
  STAssertFalse(repository.hasDownloads, @"should not have set hasDownloads to true");
  STAssertFalse(repository.hasIssues, @"should not have set hasIssues to true");
  STAssertFalse(repository.hasWiki, @"should not have set hasWiki to true");
  STAssertNil(repository.language, @"should not have set language");
  STAssertNil(repository.masterBranch, @"should not have set masterBranch");
  STAssertNil(repository.name, @"should not have set name");
  STAssertEquals(0U, repository.openIssueCount, @"should have set openIssueCount to 0");
  STAssertNil(repository.organization, @"should not have set organization");
  STAssertNil(repository.owner, @"should not have set owner");
  STAssertNil(repository.parent, @"should not have set parent");
  STAssertFalse(repository.private, @"should not have set private to true");
  STAssertNil(repository.pushedAt, @"should not have set pushedAt");
  STAssertEquals(0U, repository.size, @"should have set size to 0");
  STAssertNil(repository.source, @"should not have set source");
  STAssertNil(repository.sshURL, @"should not have set sshURL");
  STAssertNil(repository.svnURL, @"should not have set svnURL");
  STAssertEquals(0U, repository.watcherCount, @"should have set watcherCount to 0");
}

- (void)testInitWithOwnerName_LoginAndName_StoredLoginAndName {
  repository = [repository initWithOwner:@"tester" name:@"testing"];
  STAssertNil(repository.cloneURL, @"should not have set cloneURL");
  STAssertNil(repository.createdAt, @"should not have set createdAt");
  STAssertNil(repository.description, @"should not have set description");
  STAssertFalse(repository.fork, @"should not have set fork to true");
  STAssertEquals(0U, repository.forkCount, @"should have set forkCount to 0");
  STAssertNil(repository.githubURL, @"should not have set githubURL");
  STAssertNil(repository.gitURL, @"should not have set gitURL");
  STAssertFalse(repository.hasDownloads, @"should not have set hasDownloads to true");
  STAssertFalse(repository.hasIssues, @"should not have set hasIssues to true");
  STAssertFalse(repository.hasWiki, @"should not have set hasWiki to true");
  STAssertNil(repository.language, @"should not have set language");
  STAssertNil(repository.masterBranch, @"should not have set masterBranch");
  STAssertTrue([@"testing" isEqualToString:repository.name], @"should have set name");
  STAssertEquals(0U, repository.openIssueCount, @"should have set openIssueCount to 0");
  STAssertNil(repository.organization, @"should not have set organization");
  STAssertTrue([@"tester" isEqualToString:repository.owner.login], @"should have set owner login");
  STAssertNil(repository.parent, @"should not have set parent");
  STAssertFalse(repository.private, @"should not have set private to true");
  STAssertNil(repository.pushedAt, @"should not have set pushedAt");
  STAssertEquals(0U, repository.size, @"should have set size to 0");
  STAssertNil(repository.source, @"should not have set source");
  STAssertNil(repository.sshURL, @"should not have set sshURL");
  STAssertNil(repository.svnURL, @"should not have set svnURL");
  STAssertEquals(0U, repository.watcherCount, @"should have set watcherCount to 0");
}

/** initWithAttributes: tests */

- (void)testInitWithAttributes_Nil_NilAttributes {
  repository = [repository initWithAttributes:nil];
  STAssertNil(repository.cloneURL, @"should not have set cloneURL");
  STAssertNil(repository.createdAt, @"should not have set createdAt");
  STAssertNil(repository.description, @"should not have set description");
  STAssertFalse(repository.fork, @"should not have set fork to true");
  STAssertEquals(0U, repository.forkCount, @"should have set forkCount to 0");
  STAssertNil(repository.githubURL, @"should not have set githubURL");
  STAssertNil(repository.gitURL, @"should not have set gitURL");
  STAssertFalse(repository.hasDownloads, @"should not have set hasDownloads to true");
  STAssertFalse(repository.hasIssues, @"should not have set hasIssues to true");
  STAssertFalse(repository.hasWiki, @"should not have set hasWiki to true");
  STAssertNil(repository.language, @"should not have set language");
  STAssertNil(repository.masterBranch, @"should not have set masterBranch");
  STAssertNil(repository.name, @"should not have set name");
  STAssertEquals(0U, repository.openIssueCount, @"should have set openIssueCount to 0");
  STAssertNil(repository.organization, @"should not have set organization");
  STAssertNil(repository.owner, @"should not have set owner");
  STAssertNil(repository.parent, @"should not have set parent");
  STAssertFalse(repository.private, @"should not have set private to true");
  STAssertNil(repository.pushedAt, @"should not have set pushedAt");
  STAssertEquals(0U, repository.size, @"should have set size to 0");
  STAssertNil(repository.source, @"should not have set source");
  STAssertNil(repository.sshURL, @"should not have set sshURL");
  STAssertNil(repository.svnURL, @"should not have set svnURL");
  STAssertEquals(0U, repository.watcherCount, @"should have set watcherCount to 0");
}

- (void)testInitWithAttributes_Empty_NilAttributes {
  repository = [repository initWithAttributes:[NSDictionary dictionary]];
  STAssertNil(repository.cloneURL, @"should not have set cloneURL");
  STAssertNil(repository.createdAt, @"should not have set createdAt");
  STAssertNil(repository.description, @"should not have set description");
  STAssertFalse(repository.fork, @"should not have set fork to true");
  STAssertEquals(0U, repository.forkCount, @"should have set forkCount to 0");
  STAssertNil(repository.githubURL, @"should not have set githubURL");
  STAssertNil(repository.gitURL, @"should not have set gitURL");
  STAssertFalse(repository.hasDownloads, @"should not have set hasDownloads to true");
  STAssertFalse(repository.hasIssues, @"should not have set hasIssues to true");
  STAssertFalse(repository.hasWiki, @"should not have set hasWiki to true");
  STAssertNil(repository.language, @"should not have set language");
  STAssertNil(repository.masterBranch, @"should not have set masterBranch");
  STAssertNil(repository.name, @"should not have set name");
  STAssertEquals(0U, repository.openIssueCount, @"should have set openIssueCount to 0");
  STAssertNil(repository.organization, @"should not have set organization");
  STAssertNil(repository.owner, @"should not have set owner");
  STAssertNil(repository.parent, @"should not have set parent");
  STAssertFalse(repository.private, @"should not have set private to true");
  STAssertNil(repository.pushedAt, @"should not have set pushedAt");
  STAssertEquals(0U, repository.size, @"should have set size to 0");
  STAssertNil(repository.source, @"should not have set source");
  STAssertNil(repository.sshURL, @"should not have set sshURL");
  STAssertNil(repository.svnURL, @"should not have set svnURL");
  STAssertEquals(0U, repository.watcherCount, @"should have set watcherCount to 0");
}

- (void)testInitWithAttributes_Attributes_StoredAttributes {
  NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
  [attributes setValue:@"testCloneURL" forKey:@"cloneURL"];
  NSDate *createdAt = [NSDate date];
  [attributes setValue:createdAt forKey:@"createdAt"];
  [attributes setValue:@"testDescription" forKey:@"description"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"fork"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"forkCount"];
  [attributes setValue:@"testGithubURL" forKey:@"githubURL"];
  [attributes setValue:@"testGitURL" forKey:@"gitURL"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasDownloads"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasIssues"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasWiki"];
  [attributes setValue:@"testLanguage" forKey:@"language"];
  [attributes setValue:@"testMasterBranch" forKey:@"masterBranch"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"openIssueCount"];
  GHOrganization *organization = [[GHOrganization alloc] init];
  [attributes setValue:organization forKey:@"organization"];
  GHUser *owner = [[GHUser alloc] init];
  [attributes setValue:owner forKey:@"owner"];
  GHRepository *parent = [[GHRepository alloc] init];
  [attributes setValue:parent forKey:@"parent"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"private"];
  NSDate *pushedAt = [NSDate dateWithTimeIntervalSinceNow:1000];
  [attributes setValue:pushedAt forKey:@"pushedAt"];
  [attributes setValue:[NSNumber numberWithInteger:3U] forKey:@"size"];
  GHRepository *source = [[GHRepository alloc] init];
  [attributes setValue:source forKey:@"source"];
  [attributes setValue:@"testSshURL" forKey:@"sshURL"];
  [attributes setValue:@"testSvnURL" forKey:@"svnURL"];
  [attributes setValue:[NSNumber numberWithInteger:4U] forKey:@"watcherCount"];
  repository = [repository initWithAttributes:attributes];
  STAssertTrue([@"testCloneURL" isEqualToString:repository.cloneURL], @"should have set cloneURL");
  STAssertTrue([createdAt isEqualToDate:repository.createdAt], @"should have set createdAt");
  STAssertTrue([@"testDescription" isEqualToString:repository.description], @"should have set description");
  STAssertTrue(repository.fork, @"should have set fork");
  STAssertEquals(1U, repository.forkCount, @"should have set forkCount");
  STAssertTrue([@"testGithubURL" isEqualToString:repository.githubURL], @"should have set githubURL");
  STAssertTrue([@"testGitURL" isEqualToString:repository.gitURL], @"should have set gitURL");
  STAssertTrue(repository.hasDownloads, @"should have set hasDownloads to true");
  STAssertTrue(repository.hasIssues, @"should have set hasIssues to true");
  STAssertTrue(repository.hasWiki, @"should have set hasWiki to true");
  STAssertTrue([@"testLanguage" isEqualToString:repository.language], @"should have set language");
  STAssertTrue([@"testMasterBranch" isEqualToString:repository.masterBranch], @"should have set masterBranch");
  STAssertTrue([@"testName" isEqualToString:repository.name], @"should have set name");
  STAssertEquals(2U, repository.openIssueCount, @"should have set openIssueCount");
  STAssertEquals(organization, repository.organization, @"should have set organization");
  STAssertEquals(owner, repository.owner, @"should have set owner");
  STAssertEquals(parent, repository.parent, @"should have set parent");
  STAssertTrue(repository.private, @"should have set private");
  STAssertTrue([pushedAt isEqualToDate:repository.pushedAt], @"should have set pushedAt");
  STAssertEquals(3U, repository.size, @"should have set size");
  STAssertEquals(source, repository.source, @"should have set source");
  STAssertTrue([@"testSshURL" isEqualToString:repository.sshURL], @"should have set sshURL");
  STAssertTrue([@"testSvnURL" isEqualToString:repository.svnURL], @"should have set svnURL");
  STAssertEquals(4U, repository.watcherCount, @"should have set watcherCount");
}

- (void)testInitWithAttributes_GithubAPIAttributes_StoredAttributes {
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHUser timestampFormat]];
  NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
  [attributes setValue:@"testCloneURL" forKey:@"clone_url"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:@"testDescription" forKey:@"description"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"fork"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"forks"];
  [attributes setValue:@"testGitURL" forKey:@"git_url"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"has_downloads"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"has_issues"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"has_wiki"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testLanguage" forKey:@"language"];
  [attributes setValue:@"testMasterBranch" forKey:@"master_branch"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"open_issues"];
  NSDictionary *organizationAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testOrganization", @"login", nil];
  [attributes setValue:organizationAttributes forKey:@"organization"];
  NSDictionary *ownerAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testOwner", @"login", nil];
  [attributes setValue:ownerAttributes forKey:@"owner"];
  NSDictionary *parentAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testParent", @"name", nil];
  [attributes setValue:parentAttributes forKey:@"parent"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"private"];
  NSDate *pushedAt = [NSDate dateWithTimeIntervalSince1970:2000];
  [attributes setValue:[formatter stringFromDate:pushedAt] forKey:@"pushed_at"];
  [attributes setValue:[NSNumber numberWithInteger:3U] forKey:@"size"];
  NSDictionary *sourceAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testSource", @"name", nil];
  [attributes setValue:sourceAttributes forKey:@"source"];
  [attributes setValue:@"testSshURL" forKey:@"ssh_url"];
  [attributes setValue:@"testSvnURL" forKey:@"svn_url"];
  [attributes setValue:[NSNumber numberWithInteger:4U] forKey:@"watchers"];
  repository = [repository initWithAttributes:attributes];
  STAssertTrue([@"testCloneURL" isEqualToString:repository.cloneURL], @"should have set cloneURL");
  STAssertTrue([createdAt isEqualToDate:repository.createdAt], @"should have set createdAt");
  STAssertTrue([@"testDescription" isEqualToString:repository.description], @"should have set description");
  STAssertTrue(repository.fork, @"should have set fork");
  STAssertEquals(1U, repository.forkCount, @"should have set forkCount");
  STAssertTrue([@"testGithubURL" isEqualToString:repository.githubURL], @"should have set githubURL");
  STAssertTrue([@"testGitURL" isEqualToString:repository.gitURL], @"should have set gitURL");
  STAssertTrue(repository.hasDownloads, @"should have set hasDownloads to true");
  STAssertTrue(repository.hasIssues, @"should have set hasIssues to true");
  STAssertTrue(repository.hasWiki, @"should have set hasWiki to true");
  STAssertTrue([@"testLanguage" isEqualToString:repository.language], @"should have set language");
  STAssertTrue([@"testMasterBranch" isEqualToString:repository.masterBranch], @"should have set masterBranch");
  STAssertTrue([@"testName" isEqualToString:repository.name], @"should have set name");
  STAssertEquals(2U, repository.openIssueCount, @"should have set openIssueCount");
  STAssertTrue([@"testOrganization" isEqualToString:repository.organization.login], @"should have set organization");
  STAssertTrue([@"testOwner" isEqualToString:repository.owner.login], @"should have set owner");
  STAssertTrue([@"testParent" isEqualToString:repository.parent.name], @"should have set parent");
  STAssertTrue(repository.private, @"should have set private");
  STAssertTrue([pushedAt isEqualToDate:repository.pushedAt], @"should have set pushedAt");
  STAssertEquals(3U, repository.size, @"should have set size");
  STAssertTrue([@"testSource" isEqualToString:repository.source.name], @"should have set source");
  STAssertTrue([@"testSshURL" isEqualToString:repository.sshURL], @"should have set sshURL");
  STAssertTrue([@"testSvnURL" isEqualToString:repository.svnURL], @"should have set svnURL");
  STAssertEquals(4U, repository.watcherCount, @"should have set watcherCount");
}

- (void)testInitWithAttributes_GithubAPINullAttributes_StoredAttributes {
  NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
  [attributes setValue:[NSNull null] forKey:@"clone_url"];
  [attributes setValue:[NSNull null] forKey:@"created_at"];
  [attributes setValue:[NSNull null] forKey:@"description"];
  [attributes setValue:[NSNull null] forKey:@"fork"];
  [attributes setValue:[NSNull null] forKey:@"forks"];
  [attributes setValue:[NSNull null] forKey:@"git_url"];
  [attributes setValue:[NSNull null] forKey:@"has_downloads"];
  [attributes setValue:[NSNull null] forKey:@"has_issues"];
  [attributes setValue:[NSNull null] forKey:@"has_wiki"];
  [attributes setValue:[NSNull null] forKey:@"html_url"];
  [attributes setValue:[NSNull null] forKey:@"language"];
  [attributes setValue:[NSNull null] forKey:@"master_branch"];
  [attributes setValue:[NSNull null] forKey:@"name"];
  [attributes setValue:[NSNull null] forKey:@"open_issues"];
  [attributes setValue:[NSNull null] forKey:@"organization"];
  [attributes setValue:[NSNull null] forKey:@"owner"];
  [attributes setValue:[NSNull null] forKey:@"private"];
  [attributes setValue:[NSNull null] forKey:@"pushed_at"];
  [attributes setValue:[NSNull null] forKey:@"size"];
  [attributes setValue:[NSNull null] forKey:@"ssh_url"];
  [attributes setValue:[NSNull null] forKey:@"svn_url"];
  [attributes setValue:[NSNull null] forKey:@"watchers"];
  repository = [repository initWithAttributes:attributes];
  STAssertNil(repository.cloneURL, @"should not have set cloneURL");
  STAssertNil(repository.createdAt, @"should not have set createdAt");
  STAssertNil(repository.description, @"should not have set description");
  STAssertFalse(repository.fork, @"should not have set fork to true");
  STAssertEquals(0U, repository.forkCount, @"should have set forkCount to 0");
  STAssertNil(repository.githubURL, @"should not have set githubURL");
  STAssertNil(repository.gitURL, @"should not have set gitURL");
  STAssertFalse(repository.hasDownloads, @"should have set hasDownloads to true");
  STAssertFalse(repository.hasIssues, @"should have set hasIssues to true");
  STAssertFalse(repository.hasWiki, @"should have set hasWiki to true");
  STAssertNil(repository.language, @"should not have set language");
  STAssertNil(repository.masterBranch, @"should not have set masterBranch");
  STAssertNil(repository.name, @"should not have set name");
  STAssertEquals(0U, repository.openIssueCount, @"should have set openIssueCount to 0");
  STAssertNil(repository.organization, @"should not have set organization");
  STAssertNil(repository.owner, @"should not have set owner");
  STAssertNil(repository.parent, @"should not have set parent");
  STAssertFalse(repository.private, @"should not have set private to true");
  STAssertNil(repository.pushedAt, @"should not have set pushedAt");
  STAssertEquals(0U, repository.size, @"should have set size to 0");
  STAssertNil(repository.source, @"should not have set source");
  STAssertNil(repository.sshURL, @"should not have set sshURL");
  STAssertNil(repository.svnURL, @"should not have set svnURL");
  STAssertEquals(0U, repository.watcherCount, @"should have set watcherCount to 0");
}

# pragma mark - Github interaction

/** apiPath tests */

- (void)testApiPath_PublicRepositoryPath {
  NSString *owner = @"tester";
  NSString *name = @"testing";
  repository = [repository initWithOwner:owner name:name];
  NSString *path = [NSString stringWithFormat:@"/repos/%@/%@", owner, name];
  STAssertTrue([path isEqualToString:[repository apiPath]], @"should return the public repository path");
}

/** fetchMasterHeadRef: tests */

- (void)testFetchMasterHeadRef_NoMasterBranch_FetchHeadsMasterRef {
  [repository setValue:nil forKey:@"_masterBranch"];
  id mockRepository = [OCMockObject partialMockForObject:repository];
  NSString *headsMasterRef = @"heads/master";
  [[mockRepository expect] fetchRef:[OCMArg checkWithSelector:@selector(isEqualToString:) onObject:headsMasterRef]];
  [mockRepository fetchMasterHeadRef];
  STAssertNoThrow([mockRepository verify], @"should have called fetchRef:heads/master");
}

- (void)testFetchMasterHeadRef_MasterBranch_FetchMasterBranchHeadRef {
  [repository setValue:@"test" forKey:@"_masterBranch"];
  id mockRepository = [OCMockObject partialMockForObject:repository];
  NSString *headsMasterRef = @"heads/test";
  [[mockRepository expect] fetchRef:[OCMArg checkWithSelector:@selector(isEqualToString:) onObject:headsMasterRef]];
  [mockRepository fetchMasterHeadRef];
  STAssertNoThrow([mockRepository verify], @"should have called fetchRef:heads/test");
}

/** fetchRef: tests */

- (void)testFetchRef_Ref_FetchDataAtRefPathWithNoParameters {
  id mockRepository = [OCMockObject partialMockForObject:repository];
  NSString *repoPath = [[mockRepository apiPath] stringByAppendingString:@"/git/refs/heads/blah"];
  [[mockRepository expect] fetchDataAtApiPath:[OCMArg checkWithSelector:@selector(isEqualToString:) onObject:repoPath] withParameters:nil];
  [mockRepository fetchRef:@"heads/blah"];
  STAssertNoThrow([mockRepository verify], @"should have called fetchDataAtApiPath:/users/:user/:repo/git/refs/heads/blah withParameters:nil");
}

# pragma mark - SBJsonStreamParserAdapter delegate methods

/** parser:foundObject: tests */

- (void)testParserFoundObject_Nil_UnchangedAttributes {
  NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
  [attributes setValue:@"testCloneURL" forKey:@"cloneURL"];
  NSDate *createdAt = [NSDate date];
  [attributes setValue:createdAt forKey:@"createdAt"];
  [attributes setValue:@"testDescription" forKey:@"description"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"fork"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"forkCount"];
  [attributes setValue:@"testGithubURL" forKey:@"githubURL"];
  [attributes setValue:@"testGitURL" forKey:@"gitURL"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasDownloads"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasIssues"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasWiki"];
  [attributes setValue:@"testLanguage" forKey:@"language"];
  [attributes setValue:@"testMasterBranch" forKey:@"masterBranch"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"openIssueCount"];
  NSDictionary *organizationAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testOrganization", @"login", nil];
  [attributes setValue:organizationAttributes forKey:@"organization"];
  NSDictionary *ownerAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testOwner", @"login", nil];
  [attributes setValue:ownerAttributes forKey:@"owner"];
  NSDictionary *parentAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testParent", @"name", nil];
  [attributes setValue:parentAttributes forKey:@"parent"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"private"];
  NSDate *pushedAt = [NSDate dateWithTimeIntervalSinceNow:1000];
  [attributes setValue:pushedAt forKey:@"pushedAt"];
  [attributes setValue:[NSNumber numberWithInteger:3U] forKey:@"size"];
  NSDictionary *sourceAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testSource", @"name", nil];
  [attributes setValue:sourceAttributes forKey:@"source"];
  [attributes setValue:@"testSshURL" forKey:@"sshURL"];
  [attributes setValue:@"testSvnURL" forKey:@"svnURL"];
  [attributes setValue:[NSNumber numberWithInteger:4U] forKey:@"watcherCount"];
  repository = [repository initWithAttributes:attributes];
  [repository parser:nil foundObject:nil];
  STAssertTrue([@"testCloneURL" isEqualToString:repository.cloneURL], @"should have set cloneURL");
  STAssertTrue([createdAt isEqualToDate:repository.createdAt], @"should have set createdAt");
  STAssertTrue([@"testDescription" isEqualToString:repository.description], @"should have set description");
  STAssertTrue(repository.fork, @"should have set fork");
  STAssertEquals(1U, repository.forkCount, @"should have set forkCount");
  STAssertTrue([@"testGithubURL" isEqualToString:repository.githubURL], @"should have set githubURL");
  STAssertTrue([@"testGitURL" isEqualToString:repository.gitURL], @"should have set gitURL");
  STAssertTrue(repository.hasDownloads, @"should have set hasDownloads to true");
  STAssertTrue(repository.hasIssues, @"should have set hasIssues to true");
  STAssertTrue(repository.hasWiki, @"should have set hasWiki to true");
  STAssertTrue([@"testLanguage" isEqualToString:repository.language], @"should have set language");
  STAssertTrue([@"testMasterBranch" isEqualToString:repository.masterBranch], @"should have set masterBranch");
  STAssertTrue([@"testName" isEqualToString:repository.name], @"should have set name");
  STAssertEquals(2U, repository.openIssueCount, @"should have set openIssueCount");
  STAssertTrue([@"testOrganization" isEqualToString:repository.organization.login], @"should have set organization");
  STAssertTrue([@"testOwner" isEqualToString:repository.owner.login], @"should have set owner");
  STAssertTrue([@"testParent" isEqualToString:repository.parent.name], @"should have set parent");
  STAssertTrue(repository.private, @"should have set private");
  STAssertTrue([pushedAt isEqualToDate:repository.pushedAt], @"should have set pushedAt");
  STAssertEquals(3U, repository.size, @"should have set size");
  STAssertTrue([@"testSource" isEqualToString:repository.source.name], @"should have set source");
  STAssertTrue([@"testSshURL" isEqualToString:repository.sshURL], @"should have set sshURL");
  STAssertTrue([@"testSvnURL" isEqualToString:repository.svnURL], @"should have set svnURL");
  STAssertEquals(4U, repository.watcherCount, @"should have set watcherCount");
}

- (void)testParserFoundObject_Nil_UnnotifiedDelegate {
  MockGHRepositoryDelegate *delegate = [[MockGHRepositoryDelegate alloc] init];
  repository.delegate = delegate;
  id parser = [OCMockObject mockForClass:[SBJsonStreamParserAdapter class]];
  [repository parser:parser foundObject:nil];
  STAssertFalse(delegate.called, @"should not have called the delegate for a nil object");
}

- (void)testParserFoundObject_DifferentKeys_UnchangedAttributes {
  NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
  [attributes setValue:@"testCloneURL" forKey:@"cloneURL"];
  NSDate *createdAt = [NSDate date];
  [attributes setValue:createdAt forKey:@"createdAt"];
  [attributes setValue:@"testDescription" forKey:@"description"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"fork"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"forkCount"];
  [attributes setValue:@"testGithubURL" forKey:@"githubURL"];
  [attributes setValue:@"testGitURL" forKey:@"gitURL"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasDownloads"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasIssues"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasWiki"];
  [attributes setValue:@"testLanguage" forKey:@"language"];
  [attributes setValue:@"testMasterBranch" forKey:@"masterBranch"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"openIssueCount"];
  NSDictionary *organizationAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testOrganization", @"login", nil];
  [attributes setValue:organizationAttributes forKey:@"organization"];
  NSDictionary *ownerAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testOwner", @"login", nil];
  [attributes setValue:ownerAttributes forKey:@"owner"];
  NSDictionary *parentAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testParent", @"name", nil];
  [attributes setValue:parentAttributes forKey:@"parent"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"private"];
  NSDate *pushedAt = [NSDate dateWithTimeIntervalSinceNow:1000];
  [attributes setValue:pushedAt forKey:@"pushedAt"];
  [attributes setValue:[NSNumber numberWithInteger:3U] forKey:@"size"];
  NSDictionary *sourceAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testSource", @"name", nil];
  [attributes setValue:sourceAttributes forKey:@"source"];
  [attributes setValue:@"testSshURL" forKey:@"sshURL"];
  [attributes setValue:@"testSvnURL" forKey:@"svnURL"];
  [attributes setValue:[NSNumber numberWithInteger:4U] forKey:@"watcherCount"];
  repository = [repository initWithAttributes:attributes];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParser class]];
  [repository parser:parser foundObject:[NSDictionary dictionaryWithObjectsAndKeys:@"new name", @"namee", nil]];
  STAssertTrue([@"testCloneURL" isEqualToString:repository.cloneURL], @"should have set cloneURL");
  STAssertTrue([createdAt isEqualToDate:repository.createdAt], @"should have set createdAt");
  STAssertTrue([@"testDescription" isEqualToString:repository.description], @"should have set description");
  STAssertTrue(repository.fork, @"should have set fork");
  STAssertEquals(1U, repository.forkCount, @"should have set forkCount");
  STAssertTrue([@"testGithubURL" isEqualToString:repository.githubURL], @"should have set githubURL");
  STAssertTrue([@"testGitURL" isEqualToString:repository.gitURL], @"should have set gitURL");
  STAssertTrue(repository.hasDownloads, @"should have set hasDownloads to true");
  STAssertTrue(repository.hasIssues, @"should have set hasIssues to true");
  STAssertTrue(repository.hasWiki, @"should have set hasWiki to true");
  STAssertTrue([@"testLanguage" isEqualToString:repository.language], @"should have set language");
  STAssertTrue([@"testMasterBranch" isEqualToString:repository.masterBranch], @"should have set masterBranch");
  STAssertTrue([@"testName" isEqualToString:repository.name], @"should have set name");
  STAssertEquals(2U, repository.openIssueCount, @"should have set openIssueCount");
  STAssertTrue([@"testOrganization" isEqualToString:repository.organization.login], @"should have set organization");
  STAssertTrue([@"testOwner" isEqualToString:repository.owner.login], @"should have set owner");
  STAssertTrue([@"testParent" isEqualToString:repository.parent.name], @"should have set parent");
  STAssertTrue(repository.private, @"should have set private");
  STAssertTrue([pushedAt isEqualToDate:repository.pushedAt], @"should have set pushedAt");
  STAssertEquals(3U, repository.size, @"should have set size");
  STAssertTrue([@"testSource" isEqualToString:repository.source.name], @"should have set source");
  STAssertTrue([@"testSshURL" isEqualToString:repository.sshURL], @"should have set sshURL");
  STAssertTrue([@"testSvnURL" isEqualToString:repository.svnURL], @"should have set svnURL");
  STAssertEquals(4U, repository.watcherCount, @"should have set watcherCount");
}

- (void)testParserFoundObject_Ref_NotifiedDelegate {
  MockGHRepositoryDelegate *delegate = [[MockGHRepositoryDelegate alloc] init];
  repository.delegate = delegate;
  NSDictionary *object = [NSDictionary dictionaryWithObjectsAndKeys:@"commit", @"type",
                          @"testSha", @"sha", nil];
  NSDictionary *refAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"refs/heads/test", @"ref",
                                 object, @"object", nil];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParserAdapter class]];
  [repository parser:parser foundObject:refAttributes];
  STAssertTrue([@"refs/heads/test" isEqualToString:delegate.ref.ref], @"should have found ref correctly");
  STAssertTrue([@"testSha" isEqualToString:delegate.ref.object.sha], @"should have stored ref sha correctly");
}

- (void)testParserFoundObject_Repository_CorrectAttributes {
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHUser timestampFormat]];
  NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
  [attributes setValue:@"testCloneURL" forKey:@"clone_url"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:@"testDescription" forKey:@"description"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"fork"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"forks"];
  [attributes setValue:@"testGitURL" forKey:@"git_url"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"has_downloads"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"has_issues"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"has_wiki"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testLanguage" forKey:@"language"];
  [attributes setValue:@"testMasterBranch" forKey:@"master_branch"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"open_issues"];
  NSDictionary *organizationAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testOrganization", @"login", nil];
  [attributes setValue:organizationAttributes forKey:@"organization"];
  NSDictionary *ownerAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testOwner", @"login", nil];
  [attributes setValue:ownerAttributes forKey:@"owner"];
  NSDictionary *parentAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testParent", @"name", nil];
  [attributes setValue:parentAttributes forKey:@"parent"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"private"];
  NSDate *pushedAt = [NSDate dateWithTimeIntervalSince1970:2000];
  [attributes setValue:[formatter stringFromDate:pushedAt] forKey:@"pushed_at"];
  [attributes setValue:[NSNumber numberWithInteger:3U] forKey:@"size"];
  NSDictionary *sourceAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testSource", @"name", nil];
  [attributes setValue:sourceAttributes forKey:@"source"];
  [attributes setValue:@"testSshURL" forKey:@"ssh_url"];
  [attributes setValue:@"testSvnURL" forKey:@"svn_url"];
  [attributes setValue:[NSNumber numberWithInteger:4U] forKey:@"watchers"];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParser class]];
  [repository parser:parser foundObject:attributes];
  STAssertTrue([@"testCloneURL" isEqualToString:repository.cloneURL], @"should have set cloneURL");
  STAssertTrue([createdAt isEqualToDate:repository.createdAt], @"should have set createdAt");
  STAssertTrue([@"testDescription" isEqualToString:repository.description], @"should have set description");
  STAssertTrue(repository.fork, @"should have set fork");
  STAssertEquals(1U, repository.forkCount, @"should have set forkCount");
  STAssertTrue([@"testGithubURL" isEqualToString:repository.githubURL], @"should have set githubURL");
  STAssertTrue([@"testGitURL" isEqualToString:repository.gitURL], @"should have set gitURL");
  STAssertTrue(repository.hasDownloads, @"should have set hasDownloads to true");
  STAssertTrue(repository.hasIssues, @"should have set hasIssues to true");
  STAssertTrue(repository.hasWiki, @"should have set hasWiki to true");
  STAssertTrue([@"testLanguage" isEqualToString:repository.language], @"should have set language");
  STAssertTrue([@"testMasterBranch" isEqualToString:repository.masterBranch], @"should have set masterBranch");
  STAssertTrue([@"testName" isEqualToString:repository.name], @"should have set name");
  STAssertEquals(2U, repository.openIssueCount, @"should have set openIssueCount");
  STAssertTrue([@"testOrganization" isEqualToString:repository.organization.login], @"should have set organization");
  STAssertTrue([@"testOwner" isEqualToString:repository.owner.login], @"should have set owner login");
  STAssertTrue([@"testParent" isEqualToString:repository.parent.name], @"should have set parent");
  STAssertTrue(repository.private, @"should have set private");
  STAssertTrue([pushedAt isEqualToDate:repository.pushedAt], @"should have set pushedAt");
  STAssertEquals(3U, repository.size, @"should have set size");
  STAssertTrue([@"testSource" isEqualToString:repository.source.name], @"should have set source");
  STAssertTrue([@"testSshURL" isEqualToString:repository.sshURL], @"should have set sshURL");
  STAssertTrue([@"testSvnURL" isEqualToString:repository.svnURL], @"should have set svnURL");
  STAssertEquals(4U, repository.watcherCount, @"should have set watcherCount");
}

- (void)testParserFoundObject_ExistingData_UpdatedAttributes {
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHUser timestampFormat]];
  NSMutableDictionary *oldAttributes = [[NSMutableDictionary alloc] init];
  [oldAttributes setValue:@"oldCloneURL" forKey:@"cloneURL"];
  NSDate *oldCreatedAt = [NSDate dateWithTimeIntervalSinceNow:100];
  [oldAttributes setValue:oldCreatedAt forKey:@"createdAt"];
  [oldAttributes setValue:@"oldDescription" forKey:@"description"];
  [oldAttributes setValue:[NSNumber numberWithBool:YES] forKey:@"fork"];
  [oldAttributes setValue:[NSNumber numberWithInteger:10U] forKey:@"forkCount"];
  [oldAttributes setValue:@"oldGithubURL" forKey:@"githubURL"];
  [oldAttributes setValue:@"oldGitURL" forKey:@"gitURL"];
  [oldAttributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasDownloads"];
  [oldAttributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasIssues"];
  [oldAttributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasWiki"];
  [oldAttributes setValue:@"oldLanguage" forKey:@"language"];
  [oldAttributes setValue:@"oldMasterBranch" forKey:@"masterBranch"];
  [oldAttributes setValue:@"oldName" forKey:@"name"];
  [oldAttributes setValue:[NSNumber numberWithInteger:20U] forKey:@"openIssueCount"];
  NSDictionary *oldOrganizationAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"oldOrganization", @"login", nil];
  [oldAttributes setValue:oldOrganizationAttributes forKey:@"organization"];
  NSDictionary *oldOwnerAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"oldOwner", @"login", nil];
  [oldAttributes setValue:oldOwnerAttributes forKey:@"owner"];
  NSDictionary *oldParentAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"oldParent", @"name", nil];
  [oldAttributes setValue:oldParentAttributes forKey:@"parent"];
  [oldAttributes setValue:[NSNumber numberWithBool:YES] forKey:@"private"];
  NSDate *oldPushedAt = [NSDate dateWithTimeIntervalSinceNow:10];
  [oldAttributes setValue:oldPushedAt forKey:@"pushedAt"];
  [oldAttributes setValue:[NSNumber numberWithInteger:30U] forKey:@"size"];
  NSDictionary *oldSourceAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"oldSource", @"name", nil];
  [oldAttributes setValue:oldSourceAttributes forKey:@"source"];
  [oldAttributes setValue:@"oldSshURL" forKey:@"sshURL"];
  [oldAttributes setValue:@"oldSvnURL" forKey:@"svnURL"];
  [oldAttributes setValue:[NSNumber numberWithInteger:40U] forKey:@"watcherCount"];
  repository = [repository initWithAttributes:oldAttributes];
  NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
  [attributes setValue:@"testCloneURL" forKey:@"clone_url"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:@"testDescription" forKey:@"description"];
  [attributes setValue:[NSNumber numberWithBool:NO] forKey:@"fork"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"forks"];
  [attributes setValue:@"testGitURL" forKey:@"git_url"];
  [attributes setValue:[NSNumber numberWithBool:NO] forKey:@"has_downloads"];
  [attributes setValue:[NSNumber numberWithBool:NO] forKey:@"has_issues"];
  [attributes setValue:[NSNumber numberWithBool:NO] forKey:@"has_wiki"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testLanguage" forKey:@"language"];
  [attributes setValue:@"testMasterBranch" forKey:@"master_branch"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"open_issues"];
  NSDictionary *organizationAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testOrganization", @"login", nil];
  [attributes setValue:organizationAttributes forKey:@"organization"];
  NSDictionary *ownerAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testOwner", @"login", nil];
  [attributes setValue:ownerAttributes forKey:@"owner"];
  NSDictionary *parentAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testParent", @"name", nil];
  [attributes setValue:parentAttributes forKey:@"parent"];
  [attributes setValue:[NSNumber numberWithBool:NO] forKey:@"private"];
  NSDate *pushedAt = [NSDate dateWithTimeIntervalSince1970:2000];
  [attributes setValue:[formatter stringFromDate:pushedAt] forKey:@"pushed_at"];
  [attributes setValue:[NSNumber numberWithInteger:3U] forKey:@"size"];
  NSDictionary *sourceAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"testSource", @"name", nil];
  [attributes setValue:sourceAttributes forKey:@"source"];
  [attributes setValue:@"testSshURL" forKey:@"ssh_url"];
  [attributes setValue:@"testSvnURL" forKey:@"svn_url"];
  [attributes setValue:[NSNumber numberWithInteger:4U] forKey:@"watchers"];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParser class]];
  [repository parser:parser foundObject:attributes];
  STAssertTrue([@"testCloneURL" isEqualToString:repository.cloneURL], @"should have set cloneURL");
  STAssertTrue([createdAt isEqualToDate:repository.createdAt], @"should have set createdAt");
  STAssertTrue([@"testDescription" isEqualToString:repository.description], @"should have set description");
  STAssertFalse(repository.fork, @"should have set fork");
  STAssertEquals(1U, repository.forkCount, @"should have set forkCount");
  STAssertTrue([@"testGithubURL" isEqualToString:repository.githubURL], @"should have set githubURL");
  STAssertTrue([@"testGitURL" isEqualToString:repository.gitURL], @"should have set gitURL");
  STAssertFalse(repository.hasDownloads, @"should have set hasDownloads to true");
  STAssertFalse(repository.hasIssues, @"should have set hasIssues to true");
  STAssertFalse(repository.hasWiki, @"should have set hasWiki to true");
  STAssertTrue([@"testLanguage" isEqualToString:repository.language], @"should have set language");
  STAssertTrue([@"testMasterBranch" isEqualToString:repository.masterBranch], @"should have set masterBranch");
  STAssertTrue([@"testName" isEqualToString:repository.name], @"should have set name");
  STAssertEquals(2U, repository.openIssueCount, @"should have set openIssueCount");
  STAssertTrue([@"testOrganization" isEqualToString:repository.organization.login], @"should have set organization");
  STAssertTrue([@"testOwner" isEqualToString:repository.owner.login], @"should have set owner login");
  STAssertTrue([@"testParent" isEqualToString:repository.parent.name], @"should have set parent");
  STAssertFalse(repository.private, @"should have set private");
  STAssertTrue([pushedAt isEqualToDate:repository.pushedAt], @"should have set pushedAt");
  STAssertEquals(3U, repository.size, @"should have set size");
  STAssertTrue([@"testSource" isEqualToString:repository.source.name], @"should have set source");
  STAssertTrue([@"testSshURL" isEqualToString:repository.sshURL], @"should have set sshURL");
  STAssertTrue([@"testSvnURL" isEqualToString:repository.svnURL], @"should have set svnURL");
  STAssertEquals(4U, repository.watcherCount, @"should have set watcherCount");
}

- (void)testParserFoundObject_RepositoryWithNSNullAttributes_CorrectAttributes {
  NSMutableDictionary *oldAttributes = [[NSMutableDictionary alloc] init];
  [oldAttributes setValue:@"oldCloneURL" forKey:@"cloneURL"];
  NSDate *oldCreatedAt = [NSDate dateWithTimeIntervalSinceNow:100];
  [oldAttributes setValue:oldCreatedAt forKey:@"createdAt"];
  [oldAttributes setValue:@"oldDescription" forKey:@"description"];
  [oldAttributes setValue:[NSNumber numberWithBool:YES] forKey:@"fork"];
  [oldAttributes setValue:[NSNumber numberWithInteger:10U] forKey:@"forkCount"];
  [oldAttributes setValue:@"oldGithubURL" forKey:@"githubURL"];
  [oldAttributes setValue:@"oldGitURL" forKey:@"gitURL"];
  [oldAttributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasDownloads"];
  [oldAttributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasIssues"];
  [oldAttributes setValue:[NSNumber numberWithBool:YES] forKey:@"hasWiki"];
  [oldAttributes setValue:@"oldLanguage" forKey:@"language"];
  [oldAttributes setValue:@"oldMasterBranch" forKey:@"masterBranch"];
  [oldAttributes setValue:@"oldName" forKey:@"name"];
  [oldAttributes setValue:[NSNumber numberWithInteger:20U] forKey:@"openIssueCount"];
  NSDictionary *oldOrganizationAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"oldOrganization", @"login", nil];
  [oldAttributes setValue:oldOrganizationAttributes forKey:@"organization"];
  NSDictionary *oldOwnerAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"oldOwner", @"login", nil];
  [oldAttributes setValue:oldOwnerAttributes forKey:@"owner"];
  NSDictionary *oldParentAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"oldParent", @"name", nil];
  [oldAttributes setValue:oldParentAttributes forKey:@"parent"];
  [oldAttributes setValue:[NSNumber numberWithBool:YES] forKey:@"private"];
  NSDate *oldPushedAt = [NSDate dateWithTimeIntervalSinceNow:10];
  [oldAttributes setValue:oldPushedAt forKey:@"pushedAt"];
  [oldAttributes setValue:[NSNumber numberWithInteger:30U] forKey:@"size"];
  NSDictionary *oldSourceAttributes = [NSDictionary dictionaryWithObjectsAndKeys:@"oldSource", @"name", nil];
  [oldAttributes setValue:oldSourceAttributes forKey:@"source"];
  [oldAttributes setValue:@"oldSshURL" forKey:@"sshURL"];
  [oldAttributes setValue:@"oldSvnURL" forKey:@"svnURL"];
  [oldAttributes setValue:[NSNumber numberWithInteger:40U] forKey:@"watcherCount"];
  repository = [repository initWithAttributes:oldAttributes];
  NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
  [attributes setValue:[NSNull null] forKey:@"clone_url"];
  [attributes setValue:[NSNull null] forKey:@"created_at"];
  [attributes setValue:[NSNull null] forKey:@"description"];
  [attributes setValue:[NSNull null] forKey:@"fork"];
  [attributes setValue:[NSNull null] forKey:@"forks"];
  [attributes setValue:[NSNull null] forKey:@"git_url"];
  [attributes setValue:[NSNull null] forKey:@"has_downloads"];
  [attributes setValue:[NSNull null] forKey:@"has_issues"];
  [attributes setValue:[NSNull null] forKey:@"has_wiki"];
  [attributes setValue:[NSNull null] forKey:@"html_url"];
  [attributes setValue:[NSNull null] forKey:@"language"];
  [attributes setValue:[NSNull null] forKey:@"master_branch"];
  [attributes setValue:[NSNull null] forKey:@"name"];
  [attributes setValue:[NSNull null] forKey:@"open_issues"];
  [attributes setValue:[NSNull null] forKey:@"organization"];
  [attributes setValue:[NSNull null] forKey:@"owner"];
  [attributes setValue:[NSNull null] forKey:@"parent"];
  [attributes setValue:[NSNull null] forKey:@"private"];
  [attributes setValue:[NSNull null] forKey:@"pushed_at"];
  [attributes setValue:[NSNull null] forKey:@"size"];
  [attributes setValue:[NSNull null] forKey:@"source"];
  [attributes setValue:[NSNull null] forKey:@"ssh_url"];
  [attributes setValue:[NSNull null] forKey:@"svn_url"];
  [attributes setValue:[NSNull null] forKey:@"watchers"];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParser class]];
  [repository parser:parser foundObject:attributes];
  STAssertNil(repository.cloneURL, @"should not have set cloneURL");
  STAssertNil(repository.createdAt, @"should not have set createdAt");
  STAssertNil(repository.description, @"should not have set description");
  STAssertFalse(repository.fork, @"should not have set fork to true");
  STAssertEquals(0U, repository.forkCount, @"should have set forkCount to 0");
  STAssertNil(repository.githubURL, @"should not have set githubURL");
  STAssertNil(repository.gitURL, @"should not have set gitURL");
  STAssertFalse(repository.hasDownloads, @"should not have set hasDownloads to true");
  STAssertFalse(repository.hasIssues, @"should not have set hasIssues to true");
  STAssertFalse(repository.hasWiki, @"should not have set hasWiki to true");
  STAssertNil(repository.language, @"should not have set language");
  STAssertNil(repository.masterBranch, @"should not have set masterBranch");
  STAssertNil(repository.name, @"should not have set name");
  STAssertEquals(0U, repository.openIssueCount, @"should have set openIssueCount to 0");
  STAssertNil(repository.organization, @"should not have set organization");
  STAssertNil(repository.owner, @"should not have set owner");
  STAssertNil(repository.parent, @"should not have set parent");
  STAssertFalse(repository.private, @"should not have set private to true");
  STAssertNil(repository.pushedAt, @"should not have set pushedAt");
  STAssertEquals(0U, repository.size, @"should have set size to 0");
  STAssertNil(repository.source, @"should not have set source");
  STAssertNil(repository.sshURL, @"should not have set sshURL");
  STAssertNil(repository.svnURL, @"should not have set svnURL");
  STAssertEquals(0U, repository.watcherCount, @"should have set watcherCount to 0");
}

@end
