#import "GithubTests.h"
#import "GHTree.h"

@interface GHTreeTests : GithubTests {
  GHTree *tree;
}

# pragma mark - Attributes

/** leaves tests */

/**
 * Should have an NSArray attribute called leaves.
 */
- (void)testLeaves_NSArray;

/**
 * Should not allow writes to leaves.
 */
- (void)testSetLeaves_NoMethod;

/** type tests */

/**
 * Should be a tree type.
 */
- (void)testType_Tree;

# pragma mark - Initializers

/** initWithRepository:sha tests */

/**
 * Should create a tree with nil attributes if nil attributes are given.
 */
- (void)testInitWithRepositorySha_NilAttributes_NewTreeWithNilAttributes;

/**
 * Should create a tree with the given attributes and nil other attributes.
 */
- (void)testInitWithRepositorySha_Attributes_NewTreeWithCorrectAttributes;

/** initWithRepository:mode:path:sha: tests */

/**
 * Should create a tree with nil attributes if nil attributes are given.
 */
- (void)testInitWithRepositoryModePathSha_NilAttributes_NewTreeWithNilAttributes;

/**
 * Should create a tree with the given attributes and nil leaves.
 */
- (void)testInitWithRepositoryModePathSha_Attributes_NewTreeWithCorrectAttributes;

/** initWithRepository:leaves:mode:path:sha: tests */

/**
 * Should create a tree with nil attributes if nil attributes are given.
 */
- (void)testInitWithRepositoryLeavesModePathSha_NilAttributes_NewTreeWithNilAttributes;

/**
 * Should create a tree with the given attributes.
 */
- (void)testInitWithRepositoryLeavesModePathSha_Attributes_NewTreeWithCorrectAttributes;

# pragma mark - Github interaction

/** apiPath tests */

/**
 * Should return the path to this tree with sha.
 */
- (void)testApiPath_TreePath;

/** fetchImmediateLeaves tests */

/**
 * Should call fetchDataWithParameters: and provide a recursive parameter.
 */
- (void)testFetchImmediateLeaves_FetchDataWithNoParameters;

/** fetchLeaves tests */

/**
 * Should call fetchData.
 */
- (void)testFetchLeaves_FetchDataWithRecursiveParameter;

# pragma mark - NSCopying

/**
 * Should allow copying via copyWithZone:.
 */
- (void)testCopying_CopyWithZone_SameObjects;

/**
 * Should allow copying via copy.
 */
- (void)testCopying_Copy_SameObjects;

# pragma mark - NSFastEnumeration

/**
 * Should allow fast enumeration of the leaves via a for...in loop.
 */
- (void)testFastEnumeration_ForInLoop_IterateOverLeaves;

# pragma mark - SBJsonStreamParserAdapter delegate methods

/** parser:foundObject: tests */

/**
 * Should not store anything given nil.
 */
- (void)testParserFoundObject_Nil_UnchangedAttributes;

/**
 * Should not store anything given an object with the wrong keys.
 */
- (void)testParserFoundObject_DifferentKeys_UnchangedAttributes;

/**
 * Should store data correctly.
 */
- (void)testParserFoundObject_Tree_CorrectAttributes;

/**
 * Should overwrite the existing data with the data found in the object.
 */
- (void)testParserFoundObject_ExistingData_UpdatedAttributes;

/**
 * Should store NSNull data correctly.
 */
- (void)testParserFoundObject_TreeWithNSNullAttributes_CorrectAttributes;

@end
