#import "GHUserTests.h"
#import <OCMock/OCMock.h>
#import "GHUserDelegate.h"
#import "GHRepository.h"

@interface MockGHUserDelegate : MockGHDelegate <GHUserDelegate>
@property (nonatomic, copy, readonly) NSArray *repositories;
@property (nonatomic, strong) GHUser *user;
@end
@implementation MockGHUserDelegate
@synthesize repositories = _repositories;
@synthesize user = _user;
- (void)user:(GHUser *)user didFindRepository:(GHRepository *)repository {
  self.called = YES;
  self.user = user;
  if (!_repositories) {
    _repositories = [NSArray array];
  }
  _repositories = [_repositories arrayByAddingObject:repository];
}
@end

@implementation GHUserTests

- (void)setUp {
  [super setUp];
  user = [[GHUser alloc] init];
}

- (void)tearDown {
  user = nil;
  [super tearDown];
}

# pragma mark - Attributes

/** avatar_url tests */

- (void)testAvatar_url_NSString {
  [user setValue:@"test" forKey:@"_avatarURL"];
  STAssertTrue([@"test" isEqualToString:user.avatar_url], @"should have set avatar_url");
}

- (void)testSetAvatar_url_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setAvatar_url:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setAvatar_url: selector");
}

/** avatarURL tests */

- (void)testAvatarURL_NSString {
  [user setValue:@"test" forKey:@"_avatarURL"];
  STAssertTrue([@"test" isEqualToString:user.avatarURL], @"should have set avatarURL");
}

- (void)testSetAvatarURL_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setAvatarURL:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setAvatarURL: selector");
}

/** bio tests */

- (void)testBio_NSString {
  [user setValue:@"test" forKey:@"_bio"];
  STAssertTrue([@"test" isEqualToString:user.bio], @"should have set bio");
}

- (void)testSetBio_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setBio:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setBio: selector");
}

/** blog tests */

- (void)testBlog_NSString {
  [user setValue:@"test" forKey:@"_blog"];
  STAssertTrue([@"test" isEqualToString:user.blog], @"should have set blog");
}

- (void)testSetBlog_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setBlog:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setBlog: selector");
}

/** collaboratorCount tests */

- (void)testCollaboratorCount_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_collaboratorCount"];
  STAssertEquals(1U, user.collaboratorCount, @"should have set collaboratorCount");
}

- (void)testSetCollaboratorCount_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setCollaboratorCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCollaboratorCount: selector");
}

/** collaborators tests */

- (void)testCollaborators_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_collaboratorCount"];
  STAssertEquals(1U, user.collaborators, @"should have set collaborators");
}

- (void)testSetCollaborators_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setCollaborators:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCollaborators: selector");
}

/** company tests */

- (void)testCompany_NSString {
  [user setValue:@"test" forKey:@"_company"];
  STAssertTrue([@"test" isEqualToString:user.company], @"should have set company");
}

- (void)testSetCompany_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setCompany:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCompany: selector");
}

/** created_at tests */

- (void)testCreated_at_NSDate {
  NSDate *date = [NSDate date];
  [user setValue:date forKey:@"_createdAt"];
  STAssertTrue([date isEqualToDate:user.created_at], @"should have set created_at");
}

- (void)testSetCreated_at_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setCreated_at:) withObject:[NSDate date]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCreated_at: selector");
}

/** createdAt tests */

- (void)testCreatedAt_NSDate {
  NSDate *date = [NSDate date];
  [user setValue:date forKey:@"_createdAt"];
  STAssertTrue([date isEqualToDate:user.createdAt], @"should have set createdAt");
}

- (void)testSetCreatedAt_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setCreatedAt:) withObject:[NSDate date]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setCreatedAt: selector");
}

/** disk_usage tests */

- (void)testDisk_usage_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_diskUsage"];
  STAssertEquals(1U, user.disk_usage, @"should have set disk_usage");
}

- (void)testSetDisk_usage_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setDisk_usage:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setDisk_usage: selector");
}

/** diskUsage tests */

- (void)testDiskUsage_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_diskUsage"];
  STAssertEquals(1U, user.diskUsage, @"should have set diskUsage");
}

- (void)testSetDiskUsage_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setDiskUsage:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setDiskUsage: selector");
}

/** email tests */

- (void)testEmail_NSString {
  [user setValue:@"test" forKey:@"_email"];
  STAssertTrue([@"test" isEqualToString:user.email], @"should have set email");
}

- (void)testSetEmail_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setEmail:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setEmail: selector");
}

/** followerCount tests */

- (void)testFollowerCount_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_followerCount"];
  STAssertEquals(1U, user.followerCount, @"should have set followerCount");
}

- (void)testSetFollowerCount_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setFollowerCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setFollowerCount: selector");
}

/** followers tests */

- (void)testFollowers_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_followerCount"];
  STAssertEquals(1U, user.followers, @"should have set followers");
}

- (void)testSetFollowers_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setFollowers:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setFollowers: selector");
}

/** following tests */

- (void)testFollowing_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_followingCount"];
  STAssertEquals(1U, user.following, @"should have set following");
}

- (void)testSetFollowing_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setFollowing:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setFollowing: selector");
}

/** followingCount tests */

- (void)testFollowingCount_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_followingCount"];
  STAssertEquals(1U, user.followingCount, @"should have set followingCount");
}

- (void)testSetFollowingCount_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setFollowingCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setFollowingCount: selector");
}

/** githubURL tests */

- (void)testGithubURL_NSString {
  [user setValue:@"test" forKey:@"_githubURL"];
  STAssertTrue([@"test" isEqualToString:user.githubURL], @"should have set githubURL");
}

- (void)testSetGithubURL_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setGithubURL:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setGithubURL: selector");
}

/** gravatar_id tests */

- (void)testGravatar_id_NSString {
  [user setValue:@"test" forKey:@"_gravatarId"];
  STAssertTrue([@"test" isEqualToString:user.gravatar_id], @"should have set gravatar_id");
}

- (void)testSetGravatar_id_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setGravatar_id:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setGravatar_id: selector");
}

/** gravatarId tests */

- (void)testGravatarId_NSString {
  [user setValue:@"test" forKey:@"_gravatarId"];
  STAssertTrue([@"test" isEqualToString:user.gravatarId], @"should have set gravatarId");
}

- (void)testSetGravatarId_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setGravatarId:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setGravatarId: selector");
}

/** hireable tests */

- (void)testHireable_Bool {
  [user setValue:[NSNumber numberWithBool:YES] forKey:@"_hireable"];
  STAssertEquals(YES, user.hireable, @"should have set hireable");
}

- (void)testSetHireable_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setHireable:) withObject:[NSNumber numberWithBool:YES]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setHireable: selector");
}

/** html_url tests */

- (void)testHtml_url_NSString {
  [user setValue:@"test" forKey:@"_githubURL"];
  STAssertTrue([@"test" isEqualToString:user.html_url], @"should have set html_url");
}

- (void)testSetHtml_url_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setHtml_url:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setHtml_url: selector");
}

/** location tests */

- (void)testLocation_NSString {
  [user setValue:@"test" forKey:@"_location"];
  STAssertTrue([@"test" isEqualToString:user.location], @"should have set location");
}

- (void)testSetLocation_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setLocation:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setLocation: selector");
}

/** login tests */

- (void)testLogin_NSString {
  [user setValue:@"test" forKey:@"_login"];
  STAssertTrue([@"test" isEqualToString:user.login], @"should have set login");
}

- (void)testSetLogin_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setLogin:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setLogin: selector");
}

/** name tests */

- (void)testName_NSString {
  [user setValue:@"test" forKey:@"_name"];
  STAssertTrue([@"test" isEqualToString:user.name], @"should have set name");
}

- (void)testSetName_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setName:) withObject:[NSString string]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setName: selector");
}

/** owned_private_repos tests */

- (void)testOwned_private_repos_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_ownedRepositoryCount"];
  STAssertEquals(1U, user.owned_private_repos, @"should have set owned_private_repos");
}

- (void)testSetOwned_private_repos_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setOwned_private_repos:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setOwned_private_repos: selector");
}

/** ownedRepositoryCount tests */

- (void)testOwnedRepositoryCount_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_ownedRepositoryCount"];
  STAssertEquals(1U, user.ownedRepositoryCount, @"should have set ownedRepositoryCount");
}

- (void)testSetOwnedRepositoryCount_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setOwnedRepositoryCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setOwnedRepositoryCount: selector");
}

/** plan tests */

- (void)testPlan_NSDictionary {
  [user setValue:[NSDictionary dictionary] forKey:@"_plan"];
  STAssertEquals([NSDictionary dictionary], user.plan, @"should have set plan");
}

- (void)testSetPlan_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setPlan:) withObject:[NSDictionary dictionary]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPlan: selector");
}

/** private_gists tests */

- (void)testPrivate_gists_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_privateGistCount"];
  STAssertEquals(1U, user.private_gists, @"should have set private_gists");
}

- (void)testSetPrivate_gists_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setPrivate_gists:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPrivate_gists: selector");
}

/** privateGistCount tests */

- (void)testPrivateGistCount_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_privateGistCount"];
  STAssertEquals(1U, user.privateGistCount, @"should have set privateGistCount");
}

- (void)testSetPrivateGistCount_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setPrivateGistCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPrivateGistCount: selector");
}

/** privateRepositoryCount tests */

- (void)testPrivateRepositoryCount_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_privateRepositoryCount"];
  STAssertEquals(1U, user.privateRepositoryCount, @"should have set privateRepositoryCount");
}

- (void)testSetPrivateRepositoryCount_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setPrivateRepositoryCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPrivateRepositoryCount: selector");
}

/** public_gists tests */

- (void)testPublic_gists_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_publicGistCount"];
  STAssertEquals(1U, user.public_gists, @"should have set public_gists");
}

- (void)testSetPublic_gists_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setPublic_gists:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPublic_gists: selector");
}

/** public_repos tests */

- (void)testPublic_repos_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_publicRepositoryCount"];
  STAssertEquals(1U, user.public_repos, @"should have set public_repos");
}

- (void)testSetPublic_repos_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setPublic_repos:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPublic_repos: selector");
}

/** publicGistCount tests */

- (void)testPublicGistCount_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_publicGistCount"];
  STAssertEquals(1U, user.publicGistCount, @"should have set publicGistCount");
}

- (void)testSetPublicGistCount_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setPublicGistCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPublicGistCount: selector");
}

/** publicRepositoryCount tests */

- (void)testPublicRepositoryCount_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_publicRepositoryCount"];
  STAssertEquals(1U, user.publicRepositoryCount, @"should have set publicRepositoryCount");
}

- (void)testSetPublicRepositoryCount_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setPublicRepositoryCount:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setPublicRepositoryCount: selector");
}

/** total_private_repos tests */

- (void)testTotal_private_repos_NSUInteger {
  [user setValue:[NSNumber numberWithInteger:1U] forKey:@"_privateRepositoryCount"];
  STAssertEquals(1U, user.total_private_repos, @"should have set total_private_repos");
}

- (void)testSetTotal_private_repos_NoMethod {
  STAssertThrowsSpecificNamed([user performSelector:@selector(setTotal_private_repos:) withObject:[NSNumber numberWithInteger:1U]],
                              NSException, NSInvalidArgumentException,
                              @"should have thrown invalid argument exception for setTotal_private_repos: selector");
}

# pragma mark - Initializers

/** initWithLogin: tests */

- (void)testInitWithLogin_NilLogin_NilAttributes {
  user = [user initWithLogin:nil];
  STAssertNil(user.avatarURL, @"should have nil avatar URL");
  STAssertNil(user.bio, @"should have nil bio");
  STAssertNil(user.blog, @"should have nil blog");
  STAssertEquals(0U, user.collaboratorCount, @"should have 0 collaborators");
  STAssertNil(user.company, @"should have nil company");
  STAssertNil(user.createdAt, @"should have nil createdAt");
  STAssertEquals(0U, user.diskUsage, @"should have 0 disk usage");
  STAssertNil(user.email, @"should have nil email");
  STAssertEquals(0U, user.followerCount, @"should have 0 followers");
  STAssertEquals(0U, user.followingCount, @"should have 0 following");
  STAssertNil(user.githubURL, @"should have nil Github URL");
  STAssertNil(user.gravatarId, @"should have nil Gravatar ID");
  STAssertFalse(user.hireable, @"should have false hireable");
  STAssertNil(user.location, @"should have nil location");
  STAssertNil(user.login, @"should have nil login");
  STAssertNil(user.name, @"should have nil name");
  STAssertEquals(0U, user.ownedRepositoryCount, @"should have 0 owned repositories");
  STAssertNil(user.plan, @"should have nil plan");
  STAssertEquals(0U, user.privateGistCount, @"should have 0 private gists");
  STAssertEquals(0U, user.privateRepositoryCount, @"should have 0 private repositories");
  STAssertEquals(0U, user.publicGistCount, @"should have 0 public gists");
  STAssertEquals(0U, user.publicRepositoryCount, @"should have 0 public repositories");
}

- (void)testInitWithLogin_Login_StoredLogin {
  user = [user initWithLogin:@"tester"];
  STAssertNil(user.avatarURL, @"should have nil avatar URL");
  STAssertNil(user.bio, @"should have nil bio");
  STAssertNil(user.blog, @"should have nil blog");
  STAssertEquals(0U, user.collaboratorCount, @"should have 0 collaborators");
  STAssertNil(user.company, @"should have nil company");
  STAssertNil(user.createdAt, @"should have nil createdAt");
  STAssertEquals(0U, user.diskUsage, @"should have 0 disk usage");
  STAssertNil(user.email, @"should have nil email");
  STAssertEquals(0U, user.followerCount, @"should have 0 followers");
  STAssertEquals(0U, user.followingCount, @"should have 0 following");
  STAssertNil(user.githubURL, @"should have nil Github URL");
  STAssertNil(user.gravatarId, @"should have nil Gravatar ID");
  STAssertFalse(user.hireable, @"should have false hireable");
  STAssertNil(user.location, @"should have nil location");
  STAssertTrue([@"tester" isEqualToString:user.login], @"should have set login");
  STAssertNil(user.name, @"should have nil name");
  STAssertEquals(0U, user.ownedRepositoryCount, @"should have 0 owned repositories");
  STAssertNil(user.plan, @"should have nil plan");
  STAssertEquals(0U, user.privateGistCount, @"should have 0 private gists");
  STAssertEquals(0U, user.privateRepositoryCount, @"should have 0 private repositories");
  STAssertEquals(0U, user.publicGistCount, @"should have 0 public gists");
  STAssertEquals(0U, user.publicRepositoryCount, @"should have 0 public repositories");
}

/** initWithAttributes: tests */

- (void)testInitWithAttributes_Nil_NilAttributes {
  user = [user initWithAttributes:nil];
  STAssertNil(user.avatarURL, @"should have nil avatar URL");
  STAssertNil(user.bio, @"should have nil bio");
  STAssertNil(user.blog, @"should have nil blog");
  STAssertEquals(0U, user.collaboratorCount, @"should have 0 collaborators");
  STAssertNil(user.company, @"should have nil company");
  STAssertNil(user.createdAt, @"should have nil createdAt");
  STAssertEquals(0U, user.diskUsage, @"should have 0 disk usage");
  STAssertNil(user.email, @"should have nil email");
  STAssertEquals(0U, user.followerCount, @"should have 0 followers");
  STAssertEquals(0U, user.followingCount, @"should have 0 following");
  STAssertNil(user.githubURL, @"should have nil Github URL");
  STAssertNil(user.gravatarId, @"should have nil Gravatar ID");
  STAssertFalse(user.hireable, @"should have false hireable");
  STAssertNil(user.location, @"should have nil location");
  STAssertNil(user.login, @"should have nil login");
  STAssertNil(user.name, @"should have nil name");
  STAssertEquals(0U, user.ownedRepositoryCount, @"should have 0 owned repositories");
  STAssertNil(user.plan, @"should have nil plan");
  STAssertEquals(0U, user.privateGistCount, @"should have 0 private gists");
  STAssertEquals(0U, user.privateRepositoryCount, @"should have 0 private repositories");
  STAssertEquals(0U, user.publicGistCount, @"should have 0 public gists");
  STAssertEquals(0U, user.publicRepositoryCount, @"should have 0 public repositories");
}

- (void)testInitWithAttributes_Empty_NilAttributes {
  user = [user initWithAttributes:[NSDictionary dictionary]];
  STAssertNil(user.avatarURL, @"should have nil avatar URL");
  STAssertNil(user.bio, @"should have nil bio");
  STAssertNil(user.blog, @"should have nil blog");
  STAssertEquals(0U, user.collaboratorCount, @"should have 0 collaborators");
  STAssertNil(user.company, @"should have nil company");
  STAssertNil(user.createdAt, @"should have nil createdAt");
  STAssertEquals(0U, user.diskUsage, @"should have 0 disk usage");
  STAssertNil(user.email, @"should have nil email");
  STAssertEquals(0U, user.followerCount, @"should have 0 followers");
  STAssertEquals(0U, user.followingCount, @"should have 0 following");
  STAssertNil(user.githubURL, @"should have nil Github URL");
  STAssertNil(user.gravatarId, @"should have nil Gravatar ID");
  STAssertFalse(user.hireable, @"should have false hireable");
  STAssertNil(user.location, @"should have nil location");
  STAssertNil(user.login, @"should have nil login");
  STAssertNil(user.name, @"should have nil name");
  STAssertEquals(0U, user.ownedRepositoryCount, @"should have 0 owned repositories");
  STAssertNil(user.plan, @"should have nil plan");
  STAssertEquals(0U, user.privateGistCount, @"should have 0 private gists");
  STAssertEquals(0U, user.privateRepositoryCount, @"should have 0 private repositories");
  STAssertEquals(0U, user.publicGistCount, @"should have 0 public gists");
  STAssertEquals(0U, user.publicRepositoryCount, @"should have 0 public repositories");
}

- (void)testInitWithAttributes_Attributes_StoredAttributes {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  [attributes setValue:@"testAvatarURL" forKey:@"avatarURL"];
  [attributes setValue:@"testBio" forKey:@"bio"];
  [attributes setValue:@"testBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"collaboratorCount"];
  [attributes setValue:@"testCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate date];
  [attributes setValue:createdAt forKey:@"createdAt"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"diskUsage"];
  [attributes setValue:@"testEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followerCount"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followingCount"];
  [attributes setValue:@"testGithubURL" forKey:@"githubURL"];
  [attributes setValue:@"testGravatarId" forKey:@"gravatarId"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hireable"];
  [attributes setValue:@"testLocation" forKey:@"location"];
  [attributes setValue:@"testLogin" forKey:@"login"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"ownedRepositoryCount"];
  [attributes setValue:[NSDictionary dictionary] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"privateGistCount"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"privateRepositoryCount"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"publicGistCount"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"publicRepositoryCount"];
  user = [user initWithAttributes:attributes];
  STAssertTrue([@"testAvatarURL" isEqualToString:user.avatarURL], @"should have stored avatar URL");
  STAssertTrue([@"testBio" isEqualToString:user.bio], @"should have stored bio");
  STAssertTrue([@"testBlog" isEqualToString:user.blog], @"should have stored blog");
  STAssertEquals(1U, user.collaboratorCount, @"should have stored collaborators");
  STAssertTrue([@"testCompany" isEqualToString:user.company], @"should have stored company");
  STAssertTrue([createdAt isEqualToDate:user.createdAt], @"should have stored createdAt");
  STAssertEquals(1U, user.diskUsage, @"should have stored disk usage");
  STAssertTrue([@"testEmail" isEqualToString:user.email], @"should have stored email");
  STAssertEquals(1U, user.followerCount, @"should have stored followers");
  STAssertEquals(1U, user.followingCount, @"should have stored following");
  STAssertTrue([@"testGithubURL" isEqualToString:user.githubURL], @"should have stored Github URL");
  STAssertTrue([@"testGravatarId" isEqualToString:user.gravatarId], @"should have stored Gravatar Id");
  STAssertTrue(user.hireable, @"should have stored hireable");
  STAssertTrue([@"testLocation" isEqualToString:user.location], @"should have stored location");
  STAssertTrue([@"testLogin" isEqualToString:user.login], @"should have stored login");
  STAssertTrue([@"testName" isEqualToString:user.name], @"should have stored name");
  STAssertEquals(1U, user.ownedRepositoryCount, @"should have stored owned repositories");
  STAssertTrue([[NSDictionary dictionary] isEqualToDictionary:user.plan], @"should have stored plan");
  STAssertEquals(1U, user.privateGistCount, @"should have stored private gists");
  STAssertEquals(1U, user.privateRepositoryCount, @"should have stored private repositories");
  STAssertEquals(1U, user.publicGistCount, @"should have stored public gists");
  STAssertEquals(1U, user.publicRepositoryCount, @"should have stored public repositories");
}

- (void)testInitWithAttributes_GithubAPIAttributes_StoredAttributes {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  [attributes setValue:@"testAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"testBio" forKey:@"bio"];
  [attributes setValue:@"testBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"collaborators"];
  [attributes setValue:@"testCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHUser timestampFormat]];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"disk_usage"];
  [attributes setValue:@"testEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"following"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hireable"];
  [attributes setValue:@"testLocation" forKey:@"location"];
  [attributes setValue:@"testLogin" forKey:@"login"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionary] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_repos"];
  user = [user initWithAttributes:attributes];
  STAssertTrue([@"testAvatarURL" isEqualToString:user.avatarURL], @"should have stored avatar URL");
  STAssertTrue([@"testBio" isEqualToString:user.bio], @"should have stored bio");
  STAssertTrue([@"testBlog" isEqualToString:user.blog], @"should have stored blog");
  STAssertEquals(1U, user.collaboratorCount, @"should have stored collaborators");
  STAssertTrue([@"testCompany" isEqualToString:user.company], @"should have stored company");
  STAssertTrue([createdAt isEqualToDate:user.createdAt], @"should have stored createdAt");
  STAssertEquals(1U, user.diskUsage, @"should have stored disk usage");
  STAssertTrue([@"testEmail" isEqualToString:user.email], @"should have stored email");
  STAssertEquals(1U, user.followerCount, @"should have stored followers");
  STAssertEquals(1U, user.followingCount, @"should have stored following");
  STAssertTrue([@"testGithubURL" isEqualToString:user.githubURL], @"should have stored Github URL");
  STAssertTrue([@"testGravatarId" isEqualToString:user.gravatarId], @"should have stored Gravatar Id");
  STAssertTrue(user.hireable, @"should have stored hireable");
  STAssertTrue([@"testLocation" isEqualToString:user.location], @"should have stored location");
  STAssertTrue([@"testLogin" isEqualToString:user.login], @"should have stored login");
  STAssertTrue([@"testName" isEqualToString:user.name], @"should have stored name");
  STAssertEquals(1U, user.ownedRepositoryCount, @"should have stored owned repositories");
  STAssertTrue([[NSDictionary dictionary] isEqualToDictionary:user.plan], @"should have stored plan");
  STAssertEquals(1U, user.privateGistCount, @"should have stored private gists");
  STAssertEquals(1U, user.privateRepositoryCount, @"should have stored private repositories");
  STAssertEquals(1U, user.publicGistCount, @"should have stored public gists");
  STAssertEquals(1U, user.publicRepositoryCount, @"should have stored public repositories");
}

- (void)testInitWithAttributes_GithubAPINullAttributes_StoredAttributes {
  NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
  [attributes setValue:[NSNull null] forKey:@"avatar_url"];
  [attributes setValue:[NSNull null] forKey:@"bio"];
  [attributes setValue:[NSNull null] forKey:@"blog"];
  [attributes setValue:[NSNull null] forKey:@"collaborators"];
  [attributes setValue:[NSNull null] forKey:@"company"];
  [attributes setValue:[NSNull null] forKey:@"created_at"];
  [attributes setValue:[NSNull null] forKey:@"disk_usage"];
  [attributes setValue:[NSNull null] forKey:@"email"];
  [attributes setValue:[NSNull null] forKey:@"followers"];
  [attributes setValue:[NSNull null] forKey:@"following"];
  [attributes setValue:[NSNull null] forKey:@"html_url"];
  [attributes setValue:[NSNull null] forKey:@"gravatar_id"];
  [attributes setValue:[NSNull null] forKey:@"hireable"];
  [attributes setValue:[NSNull null] forKey:@"location"];
  [attributes setValue:[NSNull null] forKey:@"login"];
  [attributes setValue:[NSNull null] forKey:@"name"];
  [attributes setValue:[NSNull null] forKey:@"owned_private_repos"];
  [attributes setValue:[NSNull null] forKey:@"plan"];
  [attributes setValue:[NSNull null] forKey:@"private_gists"];
  [attributes setValue:[NSNull null] forKey:@"total_private_repos"];
  [attributes setValue:[NSNull null] forKey:@"public_gists"];
  [attributes setValue:[NSNull null] forKey:@"public_repos"];
  user = [user initWithAttributes:attributes];
  STAssertNil(user.avatarURL, @"should have nil avatar URL");
  STAssertNil(user.bio, @"should have nil bio");
  STAssertNil(user.blog, @"should have nil blog");
  STAssertEquals(0U, user.collaboratorCount, @"should have 0 collaborators");
  STAssertNil(user.company, @"should have nil company");
  STAssertNil(user.createdAt, @"should have nil createdAt");
  STAssertEquals(0U, user.diskUsage, @"should have 0 disk usage");
  STAssertNil(user.email, @"should have nil email");
  STAssertEquals(0U, user.followerCount, @"should have 0 followers");
  STAssertEquals(0U, user.followingCount, @"should have 0 following");
  STAssertNil(user.githubURL, @"should have nil Github URL");
  STAssertNil(user.gravatarId, @"should have nil Gravatar ID");
  STAssertFalse(user.hireable, @"should have false hireable");
  STAssertNil(user.location, @"should have nil location");
  STAssertNil(user.login, @"should have nil login");
  STAssertNil(user.name, @"should have nil name");
  STAssertEquals(0U, user.ownedRepositoryCount, @"should have 0 owned repositories");
  STAssertNil(user.plan, @"should have nil plan");
  STAssertEquals(0U, user.privateGistCount, @"should have 0 private gists");
  STAssertEquals(0U, user.privateRepositoryCount, @"should have 0 private repositories");
  STAssertEquals(0U, user.publicGistCount, @"should have 0 public gists");
  STAssertEquals(0U, user.publicRepositoryCount, @"should have 0 public repositories");
}

# pragma mark - Github interaction

/** apiPath tests */

- (void)testApiPath_PublicUserPath {
  NSString *login = @"tester";
  user = [user initWithLogin:login];
  NSString *path = [NSString stringWithFormat:@"/users/%@", login];
  STAssertTrue([path isEqualToString:[user apiPath]], @"should return public user path");
}

/** fetchRepositories tests */

- (void)testFetchRepositories_FetchDataAtApiPathWithNoParameters {
  id mockUser = [OCMockObject partialMockForObject:user];
  NSString *repoPath = [[mockUser apiPath] stringByAppendingString:@"/repos"];
  [[mockUser expect] fetchDataAtApiPath:[OCMArg checkWithSelector:@selector(isEqualToString:) onObject:repoPath] withParameters:nil];
  [mockUser fetchRepositories];
  STAssertNoThrow([mockUser verify], @"should have called fetchDataAtApiPath:/users/:user/repos withParameters:nil");
}

# pragma mark - SBJsonStreamParserAdapter delegate methods

/** parser:foundArray: tests */

- (void)testParserFoundArray_Nil_UnnotifiedDelegate {
  MockGHUserDelegate *delegate = [[MockGHUserDelegate alloc] init];
  user.delegate = delegate;
  id parser = [OCMockObject mockForClass:[SBJsonStreamParserAdapter class]];
  [user parser:parser foundArray:nil];
  STAssertFalse(delegate.called, @"should not have called the delegate for an empty array");
}

- (void)testParserFoundArray_Empty_UnnotifiedDelegate {
  MockGHUserDelegate *delegate = [[MockGHUserDelegate alloc] init];
  user.delegate = delegate;
  id parser = [OCMockObject mockForClass:[SBJsonStreamParserAdapter class]];
  [user parser:parser foundArray:[NSArray array]];
  STAssertFalse(delegate.called, @"should not have called the delegate for an empty array");
}

- (void)testParserFoundArray_Repositories_NotifiedDelegate {
  MockGHUserDelegate *delegate = [[MockGHUserDelegate alloc] init];
  user.delegate = delegate;
  NSMutableDictionary *repository1Attributes = [NSDictionary dictionaryWithObjectsAndKeys:@"repository1", @"name", nil];
  NSMutableDictionary *repository2Attributes = [NSDictionary dictionaryWithObjectsAndKeys:@"repository2", @"name", nil];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParserAdapter class]];
  [user parser:parser foundArray:[NSArray arrayWithObjects:repository1Attributes, repository2Attributes, nil]];
  STAssertEquals(2U, [delegate.repositories count], @"should have found 2 repositories");
  GHRepository *repository1 = [delegate.repositories objectAtIndex:0U];
  STAssertTrue([@"repository1" isEqualToString:repository1.name], @"should have found first repository correctly");
  GHRepository *repository2 = [delegate.repositories objectAtIndex:1U];
  STAssertTrue([@"repository2" isEqualToString:repository2.name], @"should have found second repository correctly");
}

/** parser:foundObject: tests */

- (void)testParserFoundObject_Nil_UnchangedAttributes {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  [attributes setValue:@"testAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"testBio" forKey:@"bio"];
  [attributes setValue:@"testBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"collaborators"];
  [attributes setValue:@"testCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHUser timestampFormat]];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"disk_usage"];
  [attributes setValue:@"testEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"following"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hireable"];
  [attributes setValue:@"testLocation" forKey:@"location"];
  [attributes setValue:@"testLogin" forKey:@"login"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionary] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_repos"];
  user = [user initWithAttributes:attributes];
  [user parser:nil foundObject:nil];
  STAssertTrue([@"testAvatarURL" isEqualToString:user.avatarURL], @"should have stored avatar URL");
  STAssertTrue([@"testBio" isEqualToString:user.bio], @"should have stored bio");
  STAssertTrue([@"testBlog" isEqualToString:user.blog], @"should have stored blog");
  STAssertEquals(1U, user.collaboratorCount, @"should have stored collaborators");
  STAssertTrue([@"testCompany" isEqualToString:user.company], @"should have stored company");
  STAssertTrue([createdAt isEqualToDate:user.createdAt], @"should have stored createdAt");
  STAssertEquals(1U, user.diskUsage, @"should have stored disk usage");
  STAssertTrue([@"testEmail" isEqualToString:user.email], @"should have stored email");
  STAssertEquals(1U, user.followerCount, @"should have stored followers");
  STAssertEquals(1U, user.followingCount, @"should have stored following");
  STAssertTrue([@"testGithubURL" isEqualToString:user.githubURL], @"should have stored Github URL");
  STAssertTrue([@"testGravatarId" isEqualToString:user.gravatarId], @"should have stored Gravatar Id");
  STAssertTrue(user.hireable, @"should have stored hireable");
  STAssertTrue([@"testLocation" isEqualToString:user.location], @"should have stored location");
  STAssertTrue([@"testLogin" isEqualToString:user.login], @"should have stored login");
  STAssertTrue([@"testName" isEqualToString:user.name], @"should have stored name");
  STAssertEquals(1U, user.ownedRepositoryCount, @"should have stored owned repositories");
  STAssertTrue([[NSDictionary dictionary] isEqualToDictionary:user.plan], @"should have stored plan");
  STAssertEquals(1U, user.privateGistCount, @"should have stored private gists");
  STAssertEquals(1U, user.privateRepositoryCount, @"should have stored private repositories");
  STAssertEquals(1U, user.publicGistCount, @"should have stored public gists");
  STAssertEquals(1U, user.publicRepositoryCount, @"should have stored public repositories");
}

- (void)testParserFoundObject_DifferentKeys_UnchangedAttributes {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  [attributes setValue:@"testAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"testBio" forKey:@"bio"];
  [attributes setValue:@"testBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"collaborators"];
  [attributes setValue:@"testCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHUser timestampFormat]];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"disk_usage"];
  [attributes setValue:@"testEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"following"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hireable"];
  [attributes setValue:@"testLocation" forKey:@"location"];
  [attributes setValue:@"testLogin" forKey:@"login"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionary] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_repos"];
  user = [user initWithAttributes:attributes];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParser class]];
  [user parser:parser foundObject:[NSDictionary dictionaryWithObject:@"testing" forKey:@"loginn"]];
  STAssertTrue([@"testAvatarURL" isEqualToString:user.avatarURL], @"should have stored avatar URL");
  STAssertTrue([@"testBio" isEqualToString:user.bio], @"should have stored bio");
  STAssertTrue([@"testBlog" isEqualToString:user.blog], @"should have stored blog");
  STAssertEquals(1U, user.collaboratorCount, @"should have stored collaborators");
  STAssertTrue([@"testCompany" isEqualToString:user.company], @"should have stored company");
  STAssertTrue([createdAt isEqualToDate:user.createdAt], @"should have stored createdAt");
  STAssertEquals(1U, user.diskUsage, @"should have stored disk usage");
  STAssertTrue([@"testEmail" isEqualToString:user.email], @"should have stored email");
  STAssertEquals(1U, user.followerCount, @"should have stored followers");
  STAssertEquals(1U, user.followingCount, @"should have stored following");
  STAssertTrue([@"testGithubURL" isEqualToString:user.githubURL], @"should have stored Github URL");
  STAssertTrue([@"testGravatarId" isEqualToString:user.gravatarId], @"should have stored Gravatar Id");
  STAssertTrue(user.hireable, @"should have stored hireable");
  STAssertTrue([@"testLocation" isEqualToString:user.location], @"should have stored location");
  STAssertTrue([@"testLogin" isEqualToString:user.login], @"should have stored login");
  STAssertTrue([@"testName" isEqualToString:user.name], @"should have stored name");
  STAssertEquals(1U, user.ownedRepositoryCount, @"should have stored owned repositories");
  STAssertTrue([[NSDictionary dictionary] isEqualToDictionary:user.plan], @"should have stored plan");
  STAssertEquals(1U, user.privateGistCount, @"should have stored private gists");
  STAssertEquals(1U, user.privateRepositoryCount, @"should have stored private repositories");
  STAssertEquals(1U, user.publicGistCount, @"should have stored public gists");
  STAssertEquals(1U, user.publicRepositoryCount, @"should have stored public repositories");
}

- (void)testParserFoundObject_User_CorrectAttributes {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  [attributes setValue:@"testAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"testBio" forKey:@"bio"];
  [attributes setValue:@"testBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"collaborators"];
  [attributes setValue:@"testCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHUser timestampFormat]];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"disk_usage"];
  [attributes setValue:@"testEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"following"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hireable"];
  [attributes setValue:@"testLocation" forKey:@"location"];
  [attributes setValue:@"testLogin" forKey:@"login"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionary] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_repos"];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParser class]];
  [user parser:parser foundObject:attributes];
  STAssertTrue([@"testAvatarURL" isEqualToString:user.avatarURL], @"should have stored avatar URL");
  STAssertTrue([@"testBio" isEqualToString:user.bio], @"should have stored bio");
  STAssertTrue([@"testBlog" isEqualToString:user.blog], @"should have stored blog");
  STAssertEquals(1U, user.collaboratorCount, @"should have stored collaborators");
  STAssertTrue([@"testCompany" isEqualToString:user.company], @"should have stored company");
  STAssertTrue([createdAt isEqualToDate:user.createdAt], @"should have stored createdAt");
  STAssertEquals(1U, user.diskUsage, @"should have stored disk usage");
  STAssertTrue([@"testEmail" isEqualToString:user.email], @"should have stored email");
  STAssertEquals(1U, user.followerCount, @"should have stored followers");
  STAssertEquals(1U, user.followingCount, @"should have stored following");
  STAssertTrue([@"testGithubURL" isEqualToString:user.githubURL], @"should have stored Github URL");
  STAssertTrue([@"testGravatarId" isEqualToString:user.gravatarId], @"should have stored Gravatar Id");
  STAssertTrue(user.hireable, @"should have stored hireable");
  STAssertTrue([@"testLocation" isEqualToString:user.location], @"should have stored location");
  STAssertTrue([@"testLogin" isEqualToString:user.login], @"should have stored login");
  STAssertTrue([@"testName" isEqualToString:user.name], @"should have stored name");
  STAssertEquals(1U, user.ownedRepositoryCount, @"should have stored owned repositories");
  STAssertTrue([[NSDictionary dictionary] isEqualToDictionary:user.plan], @"should have stored plan");
  STAssertEquals(1U, user.privateGistCount, @"should have stored private gists");
  STAssertEquals(1U, user.privateRepositoryCount, @"should have stored private repositories");
  STAssertEquals(1U, user.publicGistCount, @"should have stored public gists");
  STAssertEquals(1U, user.publicRepositoryCount, @"should have stored public repositories");
}

- (void)testParserFoundObject_ExistingData_UpdatedAttributes {
  NSMutableDictionary *attributes = [NSMutableDictionary dictionary];
  [attributes setValue:@"oldAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"oldBio" forKey:@"bio"];
  [attributes setValue:@"oldBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"collaborators"];
  [attributes setValue:@"oldCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1100];
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHUser timestampFormat]];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"disk_usage"];
  [attributes setValue:@"oldEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"following"];
  [attributes setValue:@"oldGithubURL" forKey:@"html_url"];
  [attributes setValue:@"oldGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:NO] forKey:@"hireable"];
  [attributes setValue:@"oldLocation" forKey:@"location"];
  [attributes setValue:@"oldLogin" forKey:@"login"];
  [attributes setValue:@"oldName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionaryWithObject:@"medium" forKey:@"name"] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"public_repos"];
  user = [user initWithAttributes:attributes];
  [attributes setValue:@"testAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"testBio" forKey:@"bio"];
  [attributes setValue:@"testBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"collaborators"];
  [attributes setValue:@"testCompany" forKey:@"company"];
  createdAt = [NSDate dateWithTimeIntervalSince1970:1000];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"disk_usage"];
  [attributes setValue:@"testEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"following"];
  [attributes setValue:@"testGithubURL" forKey:@"html_url"];
  [attributes setValue:@"testGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:YES] forKey:@"hireable"];
  [attributes setValue:@"testLocation" forKey:@"location"];
  [attributes setValue:@"testLogin" forKey:@"login"];
  [attributes setValue:@"testName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionary] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:1U] forKey:@"public_repos"];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParser class]];
  [user parser:parser foundObject:attributes];
  STAssertTrue([@"testAvatarURL" isEqualToString:user.avatarURL], @"should have stored avatar URL");
  STAssertTrue([@"testBio" isEqualToString:user.bio], @"should have stored bio");
  STAssertTrue([@"testBlog" isEqualToString:user.blog], @"should have stored blog");
  STAssertEquals(1U, user.collaboratorCount, @"should have stored collaborators");
  STAssertTrue([@"testCompany" isEqualToString:user.company], @"should have stored company");
  STAssertTrue([createdAt isEqualToDate:user.createdAt], @"should have stored createdAt");
  STAssertEquals(1U, user.diskUsage, @"should have stored disk usage");
  STAssertTrue([@"testEmail" isEqualToString:user.email], @"should have stored email");
  STAssertEquals(1U, user.followerCount, @"should have stored followers");
  STAssertEquals(1U, user.followingCount, @"should have stored following");
  STAssertTrue([@"testGithubURL" isEqualToString:user.githubURL], @"should have stored Github URL");
  STAssertTrue([@"testGravatarId" isEqualToString:user.gravatarId], @"should have stored Gravatar Id");
  STAssertTrue(user.hireable, @"should have stored hireable");
  STAssertTrue([@"testLocation" isEqualToString:user.location], @"should have stored location");
  STAssertTrue([@"testLogin" isEqualToString:user.login], @"should have stored login");
  STAssertTrue([@"testName" isEqualToString:user.name], @"should have stored name");
  STAssertEquals(1U, user.ownedRepositoryCount, @"should have stored owned repositories");
  STAssertTrue([[NSDictionary dictionary] isEqualToDictionary:user.plan], @"should have stored plan");
  STAssertEquals(1U, user.privateGistCount, @"should have stored private gists");
  STAssertEquals(1U, user.privateRepositoryCount, @"should have stored private repositories");
  STAssertEquals(1U, user.publicGistCount, @"should have stored public gists");
  STAssertEquals(1U, user.publicRepositoryCount, @"should have stored public repositories");
}

- (void)testParserFoundObject_UserWithNSNullAttributes_CorrectAttributes {
  NSMutableDictionary *attributes = [[NSMutableDictionary alloc] init];
  [attributes setValue:@"oldAvatarURL" forKey:@"avatar_url"];
  [attributes setValue:@"oldBio" forKey:@"bio"];
  [attributes setValue:@"oldBlog" forKey:@"blog"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"collaborators"];
  [attributes setValue:@"oldCompany" forKey:@"company"];
  NSDate *createdAt = [NSDate dateWithTimeIntervalSince1970:1100];
  NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
  [formatter setDateFormat:[GHUser timestampFormat]];
  [attributes setValue:[formatter stringFromDate:createdAt] forKey:@"created_at"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"disk_usage"];
  [attributes setValue:@"oldEmail" forKey:@"email"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"followers"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"following"];
  [attributes setValue:@"oldGithubURL" forKey:@"html_url"];
  [attributes setValue:@"oldGravatarId" forKey:@"gravatar_id"];
  [attributes setValue:[NSNumber numberWithBool:NO] forKey:@"hireable"];
  [attributes setValue:@"oldLocation" forKey:@"location"];
  [attributes setValue:@"oldLogin" forKey:@"login"];
  [attributes setValue:@"oldName" forKey:@"name"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"owned_private_repos"];
  [attributes setValue:[NSDictionary dictionaryWithObject:@"medium" forKey:@"name"] forKey:@"plan"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"private_gists"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"total_private_repos"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"public_gists"];
  [attributes setValue:[NSNumber numberWithInteger:2U] forKey:@"public_repos"];
  user = [user initWithAttributes:attributes];
  [attributes setValue:[NSNull null] forKey:@"avatar_url"];
  [attributes setValue:[NSNull null] forKey:@"bio"];
  [attributes setValue:[NSNull null] forKey:@"blog"];
  [attributes setValue:[NSNull null] forKey:@"collaborators"];
  [attributes setValue:[NSNull null] forKey:@"company"];
  [attributes setValue:[NSNull null] forKey:@"created_at"];
  [attributes setValue:[NSNull null] forKey:@"disk_usage"];
  [attributes setValue:[NSNull null] forKey:@"email"];
  [attributes setValue:[NSNull null] forKey:@"followers"];
  [attributes setValue:[NSNull null] forKey:@"following"];
  [attributes setValue:[NSNull null] forKey:@"html_url"];
  [attributes setValue:[NSNull null] forKey:@"gravatar_id"];
  [attributes setValue:[NSNull null] forKey:@"hireable"];
  [attributes setValue:[NSNull null] forKey:@"location"];
  [attributes setValue:[NSNull null] forKey:@"login"];
  [attributes setValue:[NSNull null] forKey:@"name"];
  [attributes setValue:[NSNull null] forKey:@"owned_private_repos"];
  [attributes setValue:[NSNull null] forKey:@"plan"];
  [attributes setValue:[NSNull null] forKey:@"private_gists"];
  [attributes setValue:[NSNull null] forKey:@"total_private_repos"];
  [attributes setValue:[NSNull null] forKey:@"public_gists"];
  [attributes setValue:[NSNull null] forKey:@"public_repos"];
  id parser = [OCMockObject mockForClass:[SBJsonStreamParser class]];
  [user parser:parser foundObject:attributes];
  STAssertNil(user.avatarURL, @"should have nil avatar URL");
  STAssertNil(user.bio, @"should have nil bio");
  STAssertNil(user.blog, @"should have nil blog");
  STAssertEquals(0U, user.collaboratorCount, @"should have 0 collaborators");
  STAssertNil(user.company, @"should have nil company");
  STAssertNil(user.createdAt, @"should have nil createdAt");
  STAssertEquals(0U, user.diskUsage, @"should have 0 disk usage");
  STAssertNil(user.email, @"should have nil email");
  STAssertEquals(0U, user.followerCount, @"should have 0 followers");
  STAssertEquals(0U, user.followingCount, @"should have 0 following");
  STAssertNil(user.githubURL, @"should have nil Github URL");
  STAssertNil(user.gravatarId, @"should have nil Gravatar ID");
  STAssertFalse(user.hireable, @"should have false hireable");
  STAssertNil(user.location, @"should have nil location");
  STAssertNil(user.login, @"should have nil login");
  STAssertNil(user.name, @"should have nil name");
  STAssertEquals(0U, user.ownedRepositoryCount, @"should have 0 owned repositories");
  STAssertNil(user.plan, @"should have nil plan");
  STAssertEquals(0U, user.privateGistCount, @"should have 0 private gists");
  STAssertEquals(0U, user.privateRepositoryCount, @"should have 0 private repositories");
  STAssertEquals(0U, user.publicGistCount, @"should have 0 public gists");
  STAssertEquals(0U, user.publicRepositoryCount, @"should have 0 public repositories");
}

@end
