#import <SenTestingKit/SenTestingKit.h>
#import "SBJson.h"
#import "GHDelegate.h"

/**
 * Creates a mock GHCommit with its attributes stubbed with default values, each optionally overridden by providing it
 * in the parameter.
 *
 * \param attributes a dictionary whose keys are property names to override on the returned commit. May be nil.
 * \return a mock GHCommit with default attributes and attributes defined in the parameter
 */
id mockGHCommit(NSDictionary *attributes);

/**
 * Creates a mock GHRepository with its attributes stubbed with default values, each optionally overridden by providing
 * it in the parameter.
 *
 * \param attributes a dictionary whose keys are property names to override on the returned repository. May be nil.
 * \return a mock GHRepository with default attributes and attributes defined in the parameter
 */
id mockGHRepository(NSDictionary *attributes);

/**
 * Returns a mock NSHTTPURLResponse with the Status header stubbed to be the correct content for the given status.
 *
 * \param status the desired HTTP status code for the response
 * \return a mock NSHTTPURLResponse with the Status header stubbed to be the correct content for the given status
 */
id mockNSHTTPURLResponse(int status);

/**
 * Returns a mock NSURLConnection with the #cancel method stubbed.
 *
 * \return a mock NSURLConnection with the #cancel method stubbed
 */
id mockNSURLConnection(void);

/**
 * OCMock does not play well with the weak delegate property, so the tests with expectations must use an explicit
 * conforming class.
 */
@interface MockGHDelegate : NSObject <GHDelegate>
@property (nonatomic, assign) BOOL called;
@end
@interface MockFinishedGHDelegate : MockGHDelegate
@end
@interface MockCancelGHDelegate : MockGHDelegate
@property (nonatomic, strong) NSError *error;
@end
@interface MockCannotGHDelegate : MockGHDelegate
@property (nonatomic, copy) NSError *error;
@end
@interface MockFailureGHDelegate : MockGHDelegate
@property (nonatomic, copy) NSError *error;
@end

/**
 * Superclass for all Github framework test classes. Provides some common helper methods.
 */
@interface GithubTests : SenTestCase
@end
