#import "GithubTests.h"
#import <OCMock/OCMock.h>
#import "GHCommit.h"
#import "GHRepository.h"
#import "GHUser.h"
#import "GHDelegate.h"

id mockGHRepository(NSDictionary *attributes) {
  id repository = [OCMockObject mockForClass:[GHRepository class]];
  // default values
  NSString *cloneURL = @"testCloneURL";
  NSDate *createdAt = [NSDate date];
  NSString *description = @"testDescription";
  BOOL fork = NO;
  NSUInteger forkCount = 1U;
  NSString *gitURL = @"testGitURL";
  NSString *githubURL = @"testGithubURL";
  NSString *language = @"testLanguage";
  NSString *masterBranch = @"testMasterBranch";
  NSString *name = @"testName";
  NSUInteger openIssueCount = 2U;
  GHUser *owner = [[GHUser alloc] initWithLogin:@"tester"];
  BOOL private = NO;
  NSDate *pushedAt = [NSDate dateWithTimeIntervalSinceNow:1000];
  NSUInteger size = 3U;
  NSString *sshURL = @"testSshURL";
  NSString *svnURL = @"testSvnURL";
  NSUInteger watcherCount = 4U;
  // overrides
  if (attributes) {
    NSString *cloneURLAttribute = [attributes valueForKey:@"cloneURL"];
    if (cloneURLAttribute) {
      cloneURL = [cloneURLAttribute copy];
    }
    NSDate *createdAtAttribute = [attributes valueForKey:@"createdAt"];
    if (createdAtAttribute) {
      createdAt = [createdAtAttribute copy];
    }
    NSString *descriptionAttribute = [attributes valueForKey:@"description"];
    if (descriptionAttribute) {
      description = [descriptionAttribute copy];
    }
    NSNumber *forkAttribute = [attributes valueForKey:@"fork"];
    if (forkAttribute) {
      fork = [forkAttribute boolValue];
    }
    NSNumber *forkCountAttribute = [attributes valueForKey:@"forkCount"];
    if (forkCountAttribute) {
      forkCount = [forkCountAttribute integerValue];
    }
    NSString *githubURLAttribute = [attributes valueForKey:@"githubURL"];
    if (githubURLAttribute) {
      githubURL = [githubURLAttribute copy];
    }
    NSString *gitURLAttribute = [attributes valueForKey:@"gitURL"];
    if (gitURLAttribute) {
      gitURL = [gitURLAttribute copy];
    }
    NSString *languageAttribute = [attributes valueForKey:@"language"];
    if (languageAttribute) {
      language = [languageAttribute copy];
    }
    NSString *masterBranchAttribute = [attributes valueForKey:@"masterBranch"];
    if (masterBranchAttribute) {
      masterBranch = [masterBranchAttribute copy];
    }
    NSString *nameAttribut = [attributes valueForKey:@"name"];
    if (nameAttribut) {
      name = [nameAttribut copy];
    }
    NSNumber *openIssueCountAttribute = [attributes valueForKey:@"openIssueCount"];
    if (openIssueCountAttribute) {
      openIssueCount = [openIssueCountAttribute integerValue];
    }
    id ownerAttribute = [attributes valueForKey:@"owner"];
    if ([ownerAttribute isKindOfClass:[NSDictionary class]]) {
      owner = [[GHUser alloc] initWithAttributes:ownerAttribute];
    } else {
      owner = ownerAttribute;
    }
    NSNumber *privateAttribute = [attributes valueForKey:@"private"];
    if (privateAttribute) {
      private = [privateAttribute boolValue];
    }
    NSDate *pushedAtAttribute = [attributes valueForKey:@"pushedAt"];
    if (pushedAtAttribute) {
      pushedAt = [pushedAtAttribute copy];
    }
    NSNumber *sizeAttribute = [attributes valueForKey:@"size"];
    if (sizeAttribute) {
      size = [sizeAttribute integerValue];
    }
    NSString *sshURLAttribute = [attributes valueForKey:@"sshURL"];
    if (sshURLAttribute) {
      sshURL = [sshURLAttribute copy];
    }
    NSString *svnURLAttribute = [attributes valueForKey:@"svnURL"];
    if (svnURLAttribute) {
      svnURL = [svnURLAttribute copy];
    }
    NSNumber *watcherCountAttribute = [attributes valueForKey:@"watcherCount"];
    if (watcherCountAttribute) {
      watcherCount = [watcherCountAttribute integerValue];
    }
  }
  // stubs
  [[[repository stub] andReturn:cloneURL] clone_url];
  [[[repository stub] andReturn:cloneURL] cloneURL];
  [[[repository stub] andReturn:createdAt] created_at];
  [[[repository stub] andReturn:createdAt] createdAt];
  [[[repository stub] andReturn:description] description];
  [[[repository stub] andReturn:OCMOCK_VALUE(fork)] fork];
  [[[repository stub] andReturn:OCMOCK_VALUE(forkCount)] forkCount];
  [[[repository stub] andReturn:OCMOCK_VALUE(forkCount)] forks];
  [[[repository stub] andReturn:gitURL] git_url];
  [[[repository stub] andReturn:githubURL] githubURL];
  [[[repository stub] andReturn:gitURL] gitURL];
  [[[repository stub] andReturn:githubURL] html_url];
  [[[repository stub] andReturn:language] language];
  [[[repository stub] andReturn:masterBranch] master_branch];
  [[[repository stub] andReturn:masterBranch] masterBranch];
  [[[repository stub] andReturn:name] name];
  [[[repository stub] andReturn:OCMOCK_VALUE(openIssueCount)] open_issues];
  [[[repository stub] andReturn:OCMOCK_VALUE(openIssueCount)] openIssueCount];
  [[[repository stub] andReturn:owner] owner];
  [[[repository stub] andReturn:OCMOCK_VALUE(private)] private];
  [[[repository stub] andReturn:pushedAt] pushed_at];
  [[[repository stub] andReturn:pushedAt] pushedAt];
  // TODO: multiple methods found [[[repository stub] andReturn:OCMOCK_VALUE(size)] size];
  [[[repository stub] andReturn:sshURL] ssh_url];
  [[[repository stub] andReturn:sshURL] sshURL];
  [[[repository stub] andReturn:svnURL] svn_url];
  [[[repository stub] andReturn:svnURL] svnURL];
  [[[repository stub] andReturn:OCMOCK_VALUE(watcherCount)] watcherCount];
  [[[repository stub] andReturn:OCMOCK_VALUE(watcherCount)] watchers];
  return repository;
}

id mockGHCommit(NSDictionary *attributes) {
  id commit = [OCMockObject mockForClass:[GHCommit class]];
  // default values
  id repository = mockGHRepository(nil);
  NSString *sha = @"testSha";
  NSDictionary *author = [NSDictionary dictionaryWithObjectsAndKeys:@"tester", @"name", nil];
  NSDictionary *committer = [author copy];
  NSString *message = @"testMessage";
  NSArray *parents = [NSArray array];
  GHTree *tree = [[GHTree alloc] initWithRepository:repository sha:@"testTreeSha"];
  // overrides
  if (attributes) {
    id repositoryAttribute = [attributes valueForKey:@"repository"];
    if (repositoryAttribute) {
      repository = repositoryAttribute;
    }
    NSString *shaAttribute = [attributes valueForKey:@"sha"];
    if (shaAttribute) {
      sha = [shaAttribute copy];
    }
    NSDictionary *authorAttribute = [attributes valueForKey:@"author"];
    if (authorAttribute) {
      author = [authorAttribute copy];
    }
    NSDictionary *committerAttribute = [attributes valueForKey:@"committer"];
    if (committerAttribute) {
      committer = [committerAttribute copy];
    }
    NSString *messageAttribute = [attributes valueForKey:@"message"];
    if (messageAttribute) {
      message = [messageAttribute copy];
    }
    NSArray *parentsAttribute = [attributes valueForKey:@"parents"];
    if (parentsAttribute) {
      parents = [parentsAttribute copy];
    }
    GHTree *treeAttribute = [attributes valueForKey:@"tree"];
    if (treeAttribute) {
      tree = treeAttribute;
    }
  }
  // stubs
  [[[commit stub] andReturn:repository] repository];
  [[[commit stub] andReturn:sha] sha];
  [[[commit stub] andReturn:author] author];
  [[[commit stub] andReturn:committer] committer];
  [[[commit stub] andReturn:message] message];
  [[[commit stub] andReturn:parents] parents];
  [[[commit stub] andReturn:tree] tree];
  BOOL kind = YES;
  [[[commit stub] andReturnValue:[NSValue value:&kind withObjCType:@encode(BOOL)]] isKindOfClass:[GHCommit class]];
  return commit;
}

id mockNSHTTPURLResponse(int status) {
  NSString *statusString = [NSString stringWithFormat:@"%d", status];
  if (status == 200) {
    statusString = @"200 OK";
  } else if (status == 304) {
    statusString = @"304 Not Modified";
  } else if (status == 401) {
    statusString = @"401 Unauthorized";
  } else if (status == 403) {
    statusString = @"403 Forbidden";
  } else if (status == 404) {
    statusString = @"404 Not Found";
  }
  id response = [OCMockObject mockForClass:[NSHTTPURLResponse class]];
  NSMutableDictionary *headers = [NSMutableDictionary dictionaryWithObjects:[NSArray arrayWithObject:statusString]
                                                                    forKeys:[NSArray arrayWithObject:@"Status"]];
  [[[response stub] andReturn:headers] allHeaderFields];
  BOOL kindOfClass = YES;
  [[[response stub] andReturnValue:OCMOCK_VALUE(kindOfClass)] isKindOfClass:[OCMArg any]];
  return response;
}

id mockNSURLConnection(void) {
  id connection = [OCMockObject mockForClass:[NSURLConnection class]];
  [[connection stub] cancel];
  return connection;
}

/**
 * Neuter NSURLConnection so that #fetchContent does not make an actual connection.
 */
@implementation NSURLConnection (GHBlobTests)
+ (id)connectionWithRequest:(NSURLRequest *)request delegate:(id<NSURLConnectionDelegate>)delegate {
  if ([delegate respondsToSelector:@selector(connection:didReceiveRequest:)]) {
    [delegate performSelector:@selector(connection:didReceiveRequest:) withObject:self withObject:request];
  }
  return mockNSURLConnection();
}
@end

@implementation MockGHDelegate
@synthesize called = _called;
- (id)init {
  self = [super init];
  if (self) {
    self.called = NO;
  }
  return self;
}
- (void)githubObjectConnectionDidFinish:(GHAbstractResource *)githubObject {
}
@end
@implementation MockFinishedGHDelegate
- (void)githubObjectConnectionDidFinish:(GHAbstractResource *)githubObject {
  self.called = YES;
}
@end
@implementation MockCancelGHDelegate
@synthesize error = _error;
- (void)githubObject:(GHAbstractResource *)githubObject didCancelFetchWithError:(NSError *)error {
  self.called = YES;
  self.error = error;
}
@end
@implementation MockCannotGHDelegate
@synthesize error = _error;
- (void)githubObject:(GHAbstractResource *)githubObject cannotFetchWithError:(NSError *)error {
  self.called = YES;
  self.error = error;
}
@end
@implementation MockFailureGHDelegate
@synthesize error = _error;
- (void)githubObject:(GHAbstractResource *)githubObject connectionDidFailWithError:(NSError *)error {
  self.called = YES;
  self.error = error;
}
@end

@implementation GithubTests
@end
